*****************************************************************************************
*                                                                                       *
*  AppleIIgsKarate.Engine.s :  Sprite Engine functions.                                 *
*                                                                                       *
*****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  July 2022  *
*****************************************************************************************

**********************************************************
*  ResetSpriteEngine() :  Reset Sprite Animation Engine. *
**********************************************************
ResetSpriteEngine   STZ        SE_Player1Input       ; Player 1 Move + Button 
                    STZ        SE_Player2Input       ; Player 2 Move + Button
                    STZ        SE_GlobalInput        ; Global Input (Esc, Pause)
                    STZ        SE_GAME_TIMER_FRAME   ; Frame # (0-N)
                    STZ        SE_GAME_STATUS        ; Game Status
                    STZ        SE_GAME_STATUS_EXIT   ; Game Exit Status (0:Win,1:Loose)
                    STZ        SE_UPDATE_SCORE       ; Score needs to be updated on screen
                    STZ        SE_SOUND_PENDING_NB   ; Number of pending sounds
*--
                    LDA        SE_MAX_SPRITE_NB      ; (Max Number of Sprites x 2)-2
                    DEC
                    ASL
                    TAX
RSE_1               STZ        SE_SPRITE_STATUS,X    ; 0:None, 1:Init, 2:Pending, 3:Move, 4:Zombi, 5:Dead
                    STZ        SE_SPRITE_FUNC_MOVE,X ; Sprite AI function                     
                    STZ        SE_SPRITE_FUNC_SEND,X ; Sprite Send function
                    STZ        SE_SPRITE_FUNC_RECV,X ; Sprite Receive function                 
                    STZ        SE_SPRITE_DATA_1,X    ; Sprite AI Function Private Data #1
                    STZ        SE_SPRITE_DATA_2,X    ; Sprite AI Function Private Data #2
                    STZ        SE_SPRITE_DATA_3,X    ; Sprite AI Function Private Data #3
                    STZ        SE_SPRITE_DRAW,X      ; 0:NoDraw, 1:Draw
                    STZ        SE_SPRITE_ERASE_ID,X  ; Sprite Erase ID (0:No Erase)        
                    STZ        SE_SPRITE_MOVE_SEQ,X  ; Current Sprite Move Sequence ID (1-N)
                    STZ        SE_SPRITE_MOVE_FRAM,X ; Current Sprite Move Frame #
                    STZ        SE_SPRITE_ID,X        ; Current Sprite ID
                    STZ        SE_SPRITE_X0,X        ; Sprite X0 coordinate (byte 00-9F)
                    STZ        SE_SPRITE_Y0,X        ; Sprite Y0 coordinate (line 00-C7)
                    STZ        SE_SPRITE_X1,X        ; Sprite X1 coordinate (byte 00-9F)
                    STZ        SE_SPRITE_Y1,X        ; Sprite Y1 coordinate (line 00-C7)
                    STZ        SE_SPRITE_ADDR,X      ; Sprite Screen Address (2000-9CFF)
                    STZ        SE_SPRITE_PREV_ID,X   ; Previous Sprite ID
                    STZ        SE_SPRITE_PREV_X0,X   ; Sprite previous X0 coordinate (byte 00-9F)
                    STZ        SE_SPRITE_PREV_Y0,X   ; Sprite previous Y0 coordinate (line 00-C7)
                    STZ        SE_SPRITE_PREV_X1,X   ; Sprite previous X1 coordinate (byte 00-9F)
                    STZ        SE_SPRITE_PREV_Y1,X   ; Sprite previous Y1 coordinate (line 00-C7)
                    STZ        SE_SPRITE_PREV_ADDR,X ; Sprite previous Screen Address (2000-9CFF)
                    STZ        SE_SPRITE_OVERLAP,X   ; Sprite Overlap Table
                    STZ        SE_SPRITE_OVERLAPR,X  ; Sprite Overlap Table Prev
                    STZ        SE_SPRITE_NEXT_SEQ,X
                    STZ        SE_SPRITE_MOVE_WAIT,X ; Wait Tempo between each Move                    
                    DEX
                    DEX
                    BNE        RSE_1
*--
                    LDX        #$000E                ; Reset Next Moves List
RSE_2               STZ        SE_SPRITE_MOVE0,X
                    STZ        SE_SPRITE_MOVE1,X
                    STZ        SE_SPRITE_MOVE2,X
                    STZ        SE_SPRITE_MOVE3,X
                    STZ        SE_SPRITE_MOVE4,X
                    STZ        SE_SPRITE_MOVE5,X
                    STZ        SE_SPRITE_MOVE6,X
                    STZ        SE_SPRITE_MOVE7,X
                    STZ        SE_SPRITE_MOVE8,X
                    STZ        SE_SPRITE_MOVE9,X
                    DEX
                    DEX
                    BPL        RSE_2
*--
                    LDX        #$0018                  ; Reset Oscillator usage 
RSE_3               STZ        SE_SOUND_OSCILLATOR+4,X
                    DEX
                    DEX
                    BPL        RSE_3
*--
                    RTS


****************************************************************
*  SpriteEngine() :  Sprite Animation Engine (OUT:A=ExitCode). *
****************************************************************
SpriteEngine        STZ        SE_GAME_TIMER_FRAME  ; First Frame
                    LDA        #$0035
                    STA        SE_JOYSTICK_INPUT    ; Default Apple Joystick position
                    JSR        ClearCollisionTable  ; Clear Collision Table
*--
                    LDA        Data3Bank
                    STA        SE_Init+2
                    LDA        #$0002               ; Init Log Storage
SE_Init             STAL       $AA0000

*-------------------------------------------------------------    
*------------- [Step #0 : Begining of the Frame Loop] --------
SE_S0_BEGIN_FRAME   LDA        #$0000               ; Black Border
                    JSL        SetBorderColor
                    STZ        SE_TIMING_INDEX      ; Reset Stat
                    STZ        SE_ERASE_COUNT
                    STZ        SE_DRAW_COUNT
                    STZ        SE_COLLISION_COUNT
                    LDA        #$0019               ; Wait for Line #25
                    JSL        WaitForSpot
                    
*-------------------------------------------------------------    
*------------- [Step #1 : Compute Overlap Tables] ------------
SE_S1_OVERLAP       JSR        BuildOverlapTable    ; Build Sprite Overlap Table (Backup to previous)
                                        
*-------------------------------------------------------------
*------------- [Step #2 : Compute Erase/Draw Strategy] -------
SE_S2_ERASE_DRAW    LDA        SE_MAX_SPRITE_NB     ; Compute if the Sprites must be Erased / (Re)Drawn
                    DEC
                    ASL
                    TAX
SE_S2_ED_Loop       JSR        ComputeEraseDraw
                    DEX
                    DEX
                    BPL        SE_S2_ED_Loop
                                                                                
*-------------------------------------------------------------
*------------- [Step #3 : Compute EraseID] -------------------
SE_S3_ERASE_ID      LDA        SE_MAX_SPRITE_NB     ; Compute the Erase ID for all Sprites to be erased
                    DEC
                    ASL
                    TAX
SE_S3_EID_Loop      LDA        SE_SPRITE_ERASE_ID,X ; Check if a Erase ID is required
                    BEQ        SE_S3_EID_Next
                    JSR        ComputeEraseID       ; Compute Erase ID
                    STA        SE_SPRITE_ERASE_ID,X 
SE_S3_EID_Next      DEX
                    DEX
                    BPL        SE_S3_EID_Loop
                                        
*-------------------------------------------------------------
*------------- [Step #4 : Erase all Sprites] -----------------
SE_S4_ERASE         LDA        SE_MAX_SPRITE_NB     ; Erase all required Sprites 
                    DEC
                    ASL
                    TAX
SE_S4_ERASE_Loop    LDA        SE_SPRITE_ERASE_ID,X ; Erase Sprite if needed
                    BEQ        SE_S4_ERASE_Next                 
                    JSR        EraseSpriteByIndex                 
                    INC        SE_ERASE_COUNT       ; Statistics
                    LDA        SE_SPRITE_STATUS,X
                    CMP        #$0004               ; A Zombi Sprite is deleted after its erasement
                    BNE        SE_S4_ERASE_Next
                    JSR        ClearAllSpriteInfo
SE_S4_ERASE_Next    DEX
                    DEX
                    BPL        SE_S4_ERASE_Loop
                                        
*-------------------------------------------------------------
*------------- [Step #5 : Draw all Sprites] ------------------
SE_S5_DRAW          LDA        SE_MAX_SPRITE_NB     ; Draw all required Sprites
                    DEC
                    ASL
                    TAX
SE_S5_DRAW_Loop     LDA        SE_SPRITE_DRAW,X     ; Draw Sprite if needed
                    BEQ        SE_S5_DRAW_Next
                    JSR        DrawSpriteByIndex
                    INC        SE_DRAW_COUNT        ; Statistics            
SE_S5_DRAW_Next     DEX
                    DEX
                    BPL        SE_S5_DRAW_Loop

*-------------------------------------------------------------
*------------- [Step #6 : Compute Sprite Collision] ----------
SE_S6_COLLISION     JSR        BuildCollisionTable  ; Build Send Collision Table
*--
                    LDA        SE_MAX_SPRITE_NB     ; Call Collision Send / Received IA Functions
                    DEC
                    ASL
                    TAX
SE_S6_Loop          LDA        SE_SPRITE_STATUS,X   ; Sprite Status 
                    CMP        #$0002               ; Pending
                    BEQ        SE_S6_Send
                    CMP        #$0003               ; Move
                    BEQ        SE_S6_Send
                    BRA        SE_S6_Next           ; Init / Zombi / Dead
*--
SE_S6_Send          LDA        SE_SPRITE_FUNC_SEND,X ; Ask IA Send Function to react to Collision 
                    BEQ        SE_S6_Receive
                    JSR        (SE_SPRITE_FUNC_SEND,X)
*--
SE_S6_Receive       LDA        SE_SPRITE_FUNC_RECV,X ; Ask IA Receive Function to react to Collision 
                    BEQ        SE_S6_Next
                    JSR        (SE_SPRITE_FUNC_RECV,X)
*--
SE_S6_Next          DEX
                    DEX
                    BPL        SE_S6_Loop
                                        
*-------------------------------------------------------------
*------------- [Step #7 : Get all Input] ---------------------
SE_S7_GET_INPUT     JSR        GetGlobalInput       ; Get Global Input (Pause / Escape / Audio)
                    STA        SE_GlobalInput
                    LDY        SE_JOYSTICK_INPUT
                    JSR        GetPlayer1Input      ; Get Player #1 Inputs (Buttons + Direction)
                    STA        SE_Player1Input
                    LDY        SE_JOYSTICK_INPUT
                    JSR        GetPlayer2Input      ; Get Player #2 Inputs (Buttons + Direction)
                    STA        SE_Player2Input
                    STAL       KBDSTRB              ; Clear Strobe
                                                            
*-------------------------------------------------------------
*------------- [Step #8 : Process Global Input] --------------
SE_S8_GLOBAL_INPUT  LDA        SE_GlobalInput       ; Process Global Imputs (Pause / Escape / Audio)
                    BEQ        SE_S8_END
                    CMP        #$0050               ; P = Pause Game
                    BEQ        SE_S8_GI_PAUSE
                    CMP        #$001B               ; Esc = Quit Game
                    BEQ        SE_S8_GI_ESC
                    CMP        #$0041               ; A = Audio On/Off
                    BEQ        SE_S8_GI_AUDIO
                    BRA        SE_S8_END
SE_S8_GI_PAUSE      JSL        PauseGame            ; Pause Game
                    BRA        SE_S8_END
SE_S8_GI_ESC        LDA        #$001B               ; Quit Game
                    JMP        SE_Exit
SE_S8_GI_AUDIO      JSL        MuteUnmuteAudio      ; Mute / Unmute Audio
*--
SE_S8_END           
                    
*-------------------------------------------------------------
*------------- [Step #9 : Process Sprite Motion] -------------
SE_S9_SPRITE_MOTION LDA        SE_MAX_SPRITE_NB     ; Compute Sprites next position
                    DEC
                    ASL
                    TAX
SE_S9_SM_Loop       LDA        SE_SPRITE_STATUS,X   ; Sprite Status 
                    CMP        #$0001               ; Init
                    BEQ        SE_S9_SM_Init
                    CMP        #$0002               ; Pending
                    BEQ        SE_S9_SM_Pending
                    CMP        #$0003               ; Move
                    BEQ        SE_S9_SM_Move
                    BRA        SE_S9_SM_Next
*--
SE_S9_SM_Init       LDA        #$0002               ; Move Init Status as Pending
                    STA        SE_SPRITE_STATUS,X
SE_S9_SM_Pending    JSR        (SE_SPRITE_FUNC_MOVE,X) ; Ask IA Function for next move
                    BRA        SE_S9_SM_Next
*--
SE_S9_SM_Move       JSR        ProcessSequenceNext  ; Compute next Sprite ID/Position in Sequence                    
*--
SE_S9_SM_Next       DEX
                    DEX
                    BPL        SE_S9_SM_Loop
                    
*-------------------------------------------------------------
*------------ [Step #10 : Process Timer Action] --------------
SE_S10_TIMER_ACTION LDA        SE_GAME_TIMER_FRAME  ; Process Timer based actions (Time-out, New sprite entrance)
                    CMP        #$FFFF
                    BEQ        SE_S10_TA_1          ; Block Frame Counter to FFFF (91 minutes)
                    INC
SE_S10_TA_1         STA        SE_GAME_TIMER_FRAME
                    JSR        ProcessGameTimer
                    
*-------------------------------------------------------------
*------------ [Step #11 : Update Static Display] -------------
SE_S11_STATIC       LDA        SE_UPDATE_SCORE      ; Update Score / Life / Timer values on Screen
                    BEQ        SE_S11_END
                    LDA        #PG_Score            ; Draw Score on screen
                    JSR        DrawScore
                    STZ        SE_UPDATE_SCORE
*--
SE_S11_END

*-------------------------------------------------------------
*------------ [Step #12 : Play Game Sounds] ------------------
SE_S12_SOUND        LDA        SE_MAX_SPRITE_NB     ; Detect Startup Sequence with Sound
                    DEC
                    ASL
                    TAX
SE_S12_Search_Loop  LDA        SE_SPRITE_MOVE_FRAM,X ; Frame #
                    CMP        #$0001
                    BNE        SE_S12_Search_Next 
                    LDA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID
                    BEQ        SE_S12_Search_Next
                    JSR        GetSequenceSoundID   ; Get Sound ID for this Sequence
                    BEQ        SE_S12_Search_Next
                    JSR        AppendToSoundList    ; Add this Sound ID to the list
SE_S12_Search_Next  DEX
                    DEX
                    BPL        SE_S12_Search_Loop
*-------
SE_S12_Play         LDA        SE_SOUND_PENDING_NB  ; Plays pending Sounds
                    BEQ        SE_S12_Play_End
                    DEC
                    ASL
                    TAY
SE_S12_Play_Loop    LDA        SE_SOUND_PENDING_ID,Y ; Sound ID
                    STA        SE_S12_Play_SoundID+1
                    JSR        GetAvailableOsc      ; Search for a free oscillator
                    CPX        #$FFFF               ; No more availability
                    BEQ        SE_S12_Play_End                     
                    PHX                             ; X = Oscillator*2
                    PHY
                    TXY                             ; Y = Oscillator [0-31]
SE_S12_Play_SoundID LDA        #$AAAA               ; Sound ID
                    JSR        PlayGameSound        ; Start Playing Sound
                    PLY
                    PLX
                    STA        SE_SOUND_OSCILLATOR,X ; A = Sound duration (in frames)
                    DEY
                    DEY
                    BPL        SE_S12_Play_Loop
SE_S12_Play_End     STZ        SE_SOUND_PENDING_NB  ; No more pending sounds
                    
*-------------------------------------------------------------
*------------- [Step #13 : Check End of Game] ----------------
SE_S13_END          LDA        SE_GAME_STATUS       ; Check if we have to stop the game now
                    BNE        SE_S13_END_SOON
*--
SE_S13_CHECK        JSR        CheckGameEnding
                    STA        SE_GAME_STATUS       ; 0:Continue, 1+:Stop soon (# Frame delay)
                    STX        SE_GAME_STATUS_EXIT  ; 0:Win, 1:Loose
                    BRA        SE_S13_END_End
*--
SE_S13_END_SOON     DEC                             ; Game stops soon
                    STA        SE_GAME_STATUS
                    BNE        SE_S13_END_End        
*--
SE_S13_END_STOP     LDA        SE_GAME_STATUS_EXIT
                    JMP        SE_Exit               
*--
SE_S13_END_End

*-------------------------------------------------------------
*------------- [Step #14 : Wait for few VBLs] ----------------
SE_S14_TEMPO        JSR        ClearCollisionTable  ; Clear Collision Table
*--
                    LDA        #$0040               ; Init VBL Synchro
                    STA        SE_S14_SYNCHRO+1
                    LDY        #$0005               ; Number of VBLs to wait
SE_S14_LOOP         PHY
                    JSR        GameMusicFeed        ; Feed DOC Ram with Music data
                    JSR        UpdateOscStatusVBL   ; Update remaining songs play duration
                    JSL        GetJoystickInput     ; Read Apple Analog Joystick position + Buttons
                    STA        SE_JOYSTICK_INPUT
*--
SE_S14_SYNCHRO      LDA        #$0040
                    JSL        WaitForSpot
                    DEC        SE_S14_SYNCHRO+1     ; Decrement Spot Position
                    DEC        SE_S14_SYNCHRO+1
                    PLY
                    DEY
                    BNE        SE_S14_LOOP                    
*--
                    JMP        SE_S0_BEGIN_FRAME
*-------------                                                
SE_End              LDA        #$0000               ; Return Code = Next Level
SE_Exit             PHA
                    JSL        FadeOut
                    PLA
                    RTS
                                                
*----------   SpriteEngine Variables   ---------  
SE_Player1Input     HEX        0000                 ; Player 1 Move + Button 
SE_Player2Input     HEX        0000                 ; Player 2 Move + Button
SE_GlobalInput      HEX        0000                 ; Global Input (Esc, Pause)

SE_GAME_STATUS      HEX        0000                 ; 0:Running, 1+:Ending Soon
SE_GAME_STATUS_EXIT HEX        0000                 ; 0:Win, 1:Lose

SE_GAME_TIMER_FRAME HEX        0000                 ; Frame # (0-N)

SE_UPDATE_SCORE     HEX        0000                 ; Score needs to be updated on screen

SE_MAX_SPRITE_NB    HEX        0A00                 ; Maximum Sprite Number on Screen = 10, ordered by Z axis (closest first)
*--                                                               Blue Red1 Red2
SE_SPRITE_STATUS    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; 0:None, 1:Init, 2:Pending, 3:Move, 4:Zombi, 5:Dead
*--
SE_SPRITE_FUNC_MOVE HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite AI Move function = Compute Next Move
SE_SPRITE_FUNC_SEND HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite AI Send function = React on Collision Sent
SE_SPRITE_FUNC_RECV HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite AI Receive function = React on Collision Received                     
SE_SPRITE_DATA_1    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite AI Function Private Data #1
SE_SPRITE_DATA_2    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite AI Function Private Data #2
SE_SPRITE_DATA_3    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite AI Function Private Data #3
*--
SE_SPRITE_DRAW      HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; 0:NoDraw, 1:Draw
SE_SPRITE_ERASE_ID  HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite Erase ID (0:No Erase)
*--
SE_SPRITE_MOVE_SEQ  HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Current Sprite Move Sequence ID (1-N)
SE_SPRITE_MOVE_FRAM HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Current Sprite Move Frame #
*--
SE_SPRITE_ID        HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Current Sprite ID
SE_SPRITE_X0        HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite X0 coordinate (byte 00-9F)
SE_SPRITE_Y0        HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite Y0 coordinate (line 00-C7)
SE_SPRITE_X1        HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite X1 coordinate (byte 00-9F)
SE_SPRITE_Y1        HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite Y1 coordinate (line 00-C7)
SE_SPRITE_ADDR      HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite Screen Address (2000-9CFF)
*--
SE_SPRITE_PREV_ID   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Previous Sprite ID
SE_SPRITE_PREV_X0   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite previous X0 coordinate (byte 00-9F)
SE_SPRITE_PREV_Y0   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite previous Y0 coordinate (line 00-C7)
SE_SPRITE_PREV_X1   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite previous X1 coordinate (byte 00-9F)
SE_SPRITE_PREV_Y1   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite previous Y1 coordinate (line 00-C7)
SE_SPRITE_PREV_ADDR HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite previous Screen Address (2000-9CFF)
*--
SE_SPRITE_NEXT_SEQ  HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Next Sprite Move Sequence ID (0:None,FFFF:Death)
*----
SE_SPRITE_OVERLAP   HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite Overlap Table
SE_SPRITE_OVERLAPR  HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Sprite Overlap Table Prev
*----
SE_SPRITE_RECV      DA         SE_SPRITE_RECV0,SE_SPRITE_RECV1,SE_SPRITE_RECV2,SE_SPRITE_RECV3,SE_SPRITE_RECV4
                    DA         SE_SPRITE_RECV5,SE_SPRITE_RECV6,SE_SPRITE_RECV7,SE_SPRITE_RECV8,SE_SPRITE_RECV9
SE_SPRITE_RECV0     HEX        0000,0000,0000                           ; Sprite #0 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV1     HEX        0000,0000,0000                           ; Sprite #1 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV2     HEX        0000,0000,0000                           ; Sprite #2 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV3     HEX        0000,0000,0000                           ; Sprite #3 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV4     HEX        0000,0000,0000                           ; Sprite #4 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV5     HEX        0000,0000,0000                           ; Sprite #5 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV6     HEX        0000,0000,0000                           ; Sprite #6 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV7     HEX        0000,0000,0000                           ; Sprite #7 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV8     HEX        0000,0000,0000                           ; Sprite #8 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV9     HEX        0000,0000,0000                           ; Sprite #9 Receive Collision : Score, Max, Inpact @
                    HEX        0000,0000,0000                           ;   From Sprite #1
                    HEX        0000,0000,0000                           ;   From Sprite #2
                    HEX        0000,0000,0000                           ;   From Sprite #3
                    HEX        0000,0000,0000                           ;   From Sprite #4
                    HEX        0000,0000,0000                           ;   From Sprite #5
                    HEX        0000,0000,0000                           ;   From Sprite #6
                    HEX        0000,0000,0000                           ;   From Sprite #7
                    HEX        0000,0000,0000                           ;   From Sprite #8
                    HEX        0000,0000,0000                           ;   From Sprite #9
SE_SPRITE_RECV_END                                  ; Placeholder to mark the end of the Collision Table (used in ClearCollisionTable function)
*--
SE_SPRITE_RANDOM    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Current Sprite Random counter
*--
SE_SPRITE_MOVE      DA         SE_SPRITE_MOVE0,SE_SPRITE_MOVE1,SE_SPRITE_MOVE2,SE_SPRITE_MOVE3,SE_SPRITE_MOVE4
                    DA         SE_SPRITE_MOVE5,SE_SPRITE_MOVE6,SE_SPRITE_MOVE7,SE_SPRITE_MOVE8,SE_SPRITE_MOVE9
SE_SPRITE_MOVE_WAIT HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000  ; Wait between each Move
SE_SPRITE_MOVE0     HEX        0000                                     ; Sprite #0 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE1     HEX        0000                                     ; Sprite #1 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE2     HEX        0000                                     ; Sprite #2 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE3     HEX        0000                                     ; Sprite #3 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE4     HEX        0000                                     ; Sprite #4 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE5     HEX        0000                                     ; Sprite #5 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE6     HEX        0000                                     ; Sprite #6 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE7     HEX        0000                                     ; Sprite #7 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE8     HEX        0000                                     ; Sprite #8 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
SE_SPRITE_MOVE9     HEX        0000                                     ; Sprite #9 Next Move Index (0000=No Move)
                    HEX        0000,0000,0000,0000,0000,0000,0000       ; Move List (0000=End Of List)
*--
SE_SOUND_PENDING_NB HEX        0000                                     ; Number of pending sounds
SE_SOUND_PENDING_ID HEX        0000,0000,0000,0000,0000,0000,0000       ; Sound IDs waiting to be played
                    HEX        0000,0000,0000,0000,0000,0000,0000
SE_SOUND_OSCILLATOR HEX        FFFF,FFFF,0000,0000,0000,0000,0000,0000  ; [0-15] Oscillators Status (FFFF:NotAvailable,0000=Free,XXXX:Playing Sound)
                    HEX        0000,0000,0000,0000,0000,0000,0000,FFFF
*--
SE_JOYSTICK_INPUT   HEX        0000                                     ; Apple Analog Joystick Input
*--
SE_ERASE_COUNT      HEX        0000                                     ; Number of erased sprites during this frame
SE_DRAW_COUNT       HEX        0000                                     ; Number of drawn sprites during this frame
SE_COLLISION_COUNT  HEX        0000                                     ; Number of collision during this frame
SE_TIMING_INDEX     HEX        0000                                     ; Index Timing
SE_TIMING           HEX        0000,0000,0000,0000,0000,0000,0000,0000  ; Operations Timing 
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000                                        
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000


**********************************************************************************
*  ComputeEraseDraw() :  Compute Erase / Draw Status for a Sprite (IN:X=Sprite). *
**********************************************************************************                    
ComputeEraseDraw    LDA        SE_SPRITE_STATUS,X   ; Sprite Status
                    CMP        #$0001               ; Init
                    BEQ        CED_Init
                    CMP        #$0002               ; Pending
                    BEQ        CED_Pending_Move
                    CMP        #$0003               ; Move
                    BEQ        CED_Pending_Move
                    CMP        #$0004               ; Zombi
                    BEQ        CED_Zombi
                    CMP        #$0005               ; Dead
                    BEQ        CED_Dead
                    STZ        SE_SPRITE_ERASE_ID,X ; Unknown Status => Do Nothing
                    STZ        SE_SPRITE_DRAW,X
                    BRA        CED_End
*----
CED_Init            STZ        SE_SPRITE_ERASE_ID,X ; Init => No Erase + Always Draw
                    LDA        #$0001               ; Draw Reason = #1 (Init)
                    STA        SE_SPRITE_DRAW,X
                    BRA        CED_End
*----
CED_Pending_Move    JSR        HasSpriteMoved       ; Pending or Move => Erase if Motion + Draw if Motion or Overlap
                    BEQ        CED_NoMove
*--
CED_Move            LDA        #$0001               ; Motion => Compute an EraseID + Draw Sprite        
                    STA        SE_SPRITE_ERASE_ID,X
                    LDA        #$0002               ; Draw Reason = #2 (Motion)
                    STA        SE_SPRITE_DRAW,X
                    BRA        CED_End
*--
CED_NoMove          STZ        SE_SPRITE_ERASE_ID,X ; No Motion => No Erase
                    LDA        SE_SPRITE_OVERLAP,X  ; Draw Sprite if Overlap
                    BNE        CED_NoMove_Over4
                    LDA        SE_SPRITE_OVERLAPR,X ; Draw Sprite if Overlap Prev
                    BNE        CED_NoMove_Over3
                    LDA        #$0000               ; No Draw
                    BRA        CED_NoMove_1
CED_NoMove_Over3    LDA        #$0003               ; Draw Reason = #3 (Overlap Prev)
                    BRA        CED_NoMove_1
CED_NoMove_Over4    LDA        #$0004               ; Draw Reason = #4 (Overlap)
CED_NoMove_1        STA        SE_SPRITE_DRAW,X
                    BRA        CED_End            
*----
CED_Zombi           LDA        #$0001               ; Zombi => Always Erase + Never Draw
                    STA        SE_SPRITE_ERASE_ID,X
                    STZ        SE_SPRITE_DRAW,X
                    BRA        CED_End
*----
CED_Dead            STZ        SE_SPRITE_ERASE_ID,X ; Dead => No Erase + Draw if Overlap
                    LDA        SE_SPRITE_OVERLAP,X  ; Draw Sprite if Overlap
                    BNE        CED_Dead_Over6
                    LDA        SE_SPRITE_OVERLAPR,X ; Draw Sprite if Overlap Prev
                    BNE        CED_Dead_Over5
                    LDA        #$0000               ; No Draw
                    BRA        CED_Dead_1
CED_Dead_Over5      LDA        #$0005               ; Draw Reason = #5 (Overlap Prev Dead)
                    BRA        CED_Dead_1
CED_Dead_Over6      LDA        #$0006               ; Draw Reason = #6 (Overlap Dead)
CED_Dead_1          STA        SE_SPRITE_DRAW,X
*----
CED_End             RTS                  
                       
                                        
****************************************************************************************************
*  HasSpriteMoved() :  Check if the Sprite has moved since last frame (IN:X=Sprite,OUT:A=1 if move). *
****************************************************************************************************
HasSpriteMoved      LDA        SE_SPRITE_ADDR,X     ; Same Scren Address ?
                    CMP        SE_SPRITE_PREV_ADDR,X
                    BNE        HasSpriteMoved_Yes
                    LDA        SE_SPRITE_ID,X       ; Same Sprite ID ?
                    CMP        SE_SPRITE_PREV_ID,X
                    BNE        HasSpriteMoved_Yes
*--
HasSpriteMoved_No   LDA        #$0000               ; Sprite has not moved
                    RTS
HasSpriteMoved_Yes  LDA        #$0001               ; Sprite has moved
                    RTS


*********************************************************************
*  SetSpriteZombi() :  Change Sprite status as Zombi (IN:X=Sprite). *
*********************************************************************
SetSpriteZombi      LDA        #$0004               ; Zombi
                    STA        SE_SPRITE_STATUS,X
                    RTS


***********************************************************************
*  RecordVideoSpot() :  Record Spot Video position into Timing table. *
***********************************************************************       
RecordVideoSpot     JSL        ReadVideoSpot        ; Get Video Spot position
                    LDY        SE_TIMING_INDEX      ; Store in Timing Table
                    STA        SE_TIMING,Y
                    INY
                    INY
                    STY        SE_TIMING_INDEX
                    RTS
                    

*************************************************************
*  LogTimingInfo() :  Log Sprite Engine Timing information. *
*************************************************************
LogTimingInfo       LDA        Data3Bank
                    STA        LTI_1+2
                    STA        LTI_2+2
                    STA        LTI_3+2
                    STA        LTI_4+2
                    STA        LTI_5+2
*--
LTI_1               LDAL       $AA0000              ; Current Log Data Index in Data3Bank
                    CMP        #$FF00
                    BGE        LTI_End
                    TAX     
*--
                    LDA        SE_GAME_TIMER_FRAME  ; Current Frame #
LTI_2               STAL       $AA0000,X
                    INX
                    INX                         
                    LDA        SE_ERASE_COUNT       ; # Erase + # Draw
                    XBA
                    ORA        SE_DRAW_COUNT
LTI_3               STAL       $AA0000,X
                    INX
                    INX     
*--                                        
                    LDY        #$0000               
LTI_Loop            LDA        SE_TIMING+2,Y
                    SEC
                    SBC        SE_TIMING+0,Y
LTI_4               STAL       $AA0000,X
                    INX
                    INX
                    INY
                    INY
                    CPY        SE_TIMING_INDEX
                    BNE        LTI_Loop
                    TXA
LTI_5               STAL       $AA0000
LTI_End             RTS


*************************************************************************
*  LogSpriteInfo() :  Log Sprite information from Tables (IN:X=Sprite). *
*************************************************************************
LogSpriteInfo       TXY                             ; Log Sprite information
                    LDA        Data3Bank
                    STA        LSI_1+2
                    STA        LSI_2+2
                    STA        LSI_5+2
                    STA        LSI_6+2
                    LDA        #LSI_Source
                    STA        LSI_3+1
*--
LSI_1               LDAL       $AA0000              ; Next Storage Index
                    CMP        #$FF00
                    BGE        LSI_End              ; Already full
                    TAX     
                    LDA        SE_GAME_TIMER_FRAME  ; Current Frame
LSI_2               STAL       $AA0000,X
                    INX
                    INX  
*--
LSI_3               LDA        $BBBB                ; LSI_Source Table 
                    BEQ        LSI_Final            ; End of Table
                    STA        LSI_4+1
                    INC        LSI_3+1
                    INC        LSI_3+1              ; Next LSI_Source Table value
LSI_4               LDA        $AAAA,Y
LSI_5               STAL       $AA0000,X
                    INX
                    INX
                    BRA        LSI_3
*--
LSI_Final           TXA
LSI_6               STAL       $AA0000              ; Update Storage Index
LSI_End             RTS

*----------   LogSpriteInfo Variables   ---------
LSI_Source          DA         SE_SPRITE_PREV_ID,SE_SPRITE_PREV_X0,SE_SPRITE_PREV_Y0
                    DA         SE_SPRITE_PREV_X1,SE_SPRITE_PREV_Y1,SE_SPRITE_PREV_ADDR
                    DA         SE_SPRITE_ID,SE_SPRITE_X0,SE_SPRITE_Y0
                    DA         SE_SPRITE_X1,SE_SPRITE_Y1,SE_SPRITE_ADDR
                    DA         SE_SPRITE_STATUS,SE_SPRITE_DRAW,SE_SPRITE_ERASE_ID
                    DA         SE_SPRITE_MOVE_SEQ,SE_SPRITE_MOVE_FRAM,SE_SPRITE_NEXT_SEQ 
                    HEX        0000                 ; End of list


******************************************************
*  LogVideoSpotInfo() :  Log Video Spot information. *
******************************************************
LogVideoSpotInfo    LDA        Data3Bank
                    STA        LVSI_1+2
                    STA        LVSI_2+2
                    STA        LVSI_3+2
*--
LVSI_1              LDAL       $AA0000              ; Current Log Data Index in Data3Bank
                    CMP        #$FF00
                    BGE        LVSI_End
                    TAX     
*--
                    JSL        ReadVideoSpot        ; Get Video Spot position
LVSI_2              STAL       $AA0000,X
                    INX
                    INX
                    TXA
*--
LVSI_3              STAL       $AA0000
LVSI_End            RTS


****************************************************************************
*  BackupSpriteInfo() :  Backup Previous Sprite information (IN:X=Sprite). *
****************************************************************************
BackupSpriteInfo    PHA                             ; Move Current Sprite info to Previous
                    LDA        SE_SPRITE_ID,X
                    STA        SE_SPRITE_PREV_ID,X
                    LDA        SE_SPRITE_X0,X
                    STA        SE_SPRITE_PREV_X0,X
                    LDA        SE_SPRITE_Y0,X
                    STA        SE_SPRITE_PREV_Y0,X
                    LDA        SE_SPRITE_X1,X
                    STA        SE_SPRITE_PREV_X1,X
                    LDA        SE_SPRITE_Y1,X
                    STA        SE_SPRITE_PREV_Y1,X
                    LDA        SE_SPRITE_ADDR,X
                    STA        SE_SPRITE_PREV_ADDR,X
                    PLA
                    RTS


************************************************************************************
*  ClearAllSpriteInfo() :  Clear All Sprite information from Tables (IN:X=Sprite). *
************************************************************************************
ClearAllSpriteInfo  STZ        SE_SPRITE_STATUS,X   ; Delete Sprite information 
                    STZ        SE_SPRITE_FUNC_MOVE,X
                    STZ        SE_SPRITE_FUNC_SEND,X
                    STZ        SE_SPRITE_FUNC_RECV,X                     
                    STZ        SE_SPRITE_DATA_1,X
                    STZ        SE_SPRITE_DATA_2,X
                    STZ        SE_SPRITE_DATA_3,X
                    STZ        SE_SPRITE_DRAW,X
                    STZ        SE_SPRITE_ERASE_ID,X        
                    STZ        SE_SPRITE_MOVE_SEQ,X 
                    STZ        SE_SPRITE_MOVE_FRAM,X
                    STZ        SE_SPRITE_ID,X
                    STZ        SE_SPRITE_X0,X
                    STZ        SE_SPRITE_Y0,X
                    STZ        SE_SPRITE_X1,X
                    STZ        SE_SPRITE_Y1,X
                    STZ        SE_SPRITE_ADDR,X
                    STZ        SE_SPRITE_PREV_ID,X
                    STZ        SE_SPRITE_PREV_X0,X
                    STZ        SE_SPRITE_PREV_Y0,X
                    STZ        SE_SPRITE_PREV_X1,X
                    STZ        SE_SPRITE_PREV_Y1,X
                    STZ        SE_SPRITE_PREV_ADDR,X
                    STZ        SE_SPRITE_NEXT_SEQ,X
                    RTS


******************************************************************************************
*  ClearPrevSpriteInfo() :  Clear Previous Sprite information from Tables (IN:X=Sprite). *
******************************************************************************************
ClearPrevSpriteInfo STZ        SE_SPRITE_PREV_ID,X  ; Clear previous Sprite information 
                    STZ        SE_SPRITE_PREV_X0,X
                    STZ        SE_SPRITE_PREV_Y0,X
                    STZ        SE_SPRITE_PREV_X1,X
                    STZ        SE_SPRITE_PREV_Y1,X
                    STZ        SE_SPRITE_PREV_ADDR,X
                    STZ        SE_SPRITE_NEXT_SEQ,X
                    STZ        SE_SPRITE_STATUS,X
                    STZ        SE_SPRITE_FUNC_MOVE,X
                    STZ        SE_SPRITE_FUNC_SEND,X
                    STZ        SE_SPRITE_FUNC_RECV,X
                    STZ        SE_SPRITE_DATA_1,X
                    STZ        SE_SPRITE_DATA_2,X
                    STZ        SE_SPRITE_DATA_3,X
                    STZ        SE_SPRITE_DRAW,X
                    STZ        SE_SPRITE_ERASE_ID,X
                    STZ        SE_SPRITE_MOVE_SEQ,X
                    STZ        SE_SPRITE_MOVE_FRAM,X
                    STZ        SE_SPRITE_NEXT_SEQ,X
                    RTS


*****************************************************************************************************************************
*  InstallNewSprite() :  Install a new Sprite in the Sprite Tables (IN:A=SpriteID,IN:X=SpriteIndex,IN:Y=SpriteX0|SpriteY0). *
*****************************************************************************************************************************
InstallNewSprite    JSR        ClearAllSpriteInfo   ; Clear All Sprite information 
*--
                    STA        SE_SPRITE_ID,X       ; Sprite ID
                    TYA
                    AND        #$00FF
                    STA        INewSprite_Y0+1      ; Sprite Y0
                    TYA
                    XBA
                    AND        #$00FF
                    STA        INewSprite_X0+1      ; Sprite X0
                    LDA        SE_SPRITE_ID,X
                    DEC
                    ASL
                    TAY
                    LDA        TAB_SPRITE_SIZE,Y    ; Y : Sprite Size
                    TAY                    
*----
INewSprite_X0       LDA        #$AAAA               ; Sprite X0
                    STA        SE_SPRITE_X0,X
INewSprite_Y0       LDA        #$AAAA               ; Sprite Y0
                    STA        SE_SPRITE_Y0,X
INewSprite_X1       LDA        SE_SPRITE_X0,X
                    CLC
                    ADC:       $0000,Y              ; Sprite Width (byte)
                    DEC        
                    STA        SE_SPRITE_X1,X
INewSprite_Y1       LDA        SE_SPRITE_Y0,X
                    CLC
                    ADC:       $0002,Y              ; Sprite Height (line)
                    DEC
                    STA        SE_SPRITE_Y1,X
INewSprite_ADDR     LDA        INewSprite_Y0+1      ; Sprite Address
                    ASL
                    TAY
                    LDA        SHR_LINE_TABLE,Y
                    CLC
                    ADC        INewSprite_X0+1
                    CLC
                    ADC        #$2000
                    STA        SE_SPRITE_ADDR,X
                    LDA        SE_SPRITE_STATUS,X   ; Sprite Status : None
                    BNE        INewSprite_End
                    LDA        #$0001               ; Sprite Status : Init
                    STA        SE_SPRITE_STATUS,X
*----
INewSprite_End      RTS


*********************************************************************************************************************************
*  InstallNextSprite() :  Install the next Sprite in the Sprite Tables (IN:A=SpriteID,IN:X=SpriteIndex,IN:Y=SpriteX0|SpriteY0). *
*********************************************************************************************************************************
InstallNextSprite   JSR        BackupSpriteInfo     ; Move Current Sprite info to Previous
*--
                    STA        SE_SPRITE_ID,X       ; Sprite ID
                    TYA
                    AND        #$00FF
                    STA        INextSprite_Y0+1     ; Sprite Y0
                    TYA
                    XBA
                    AND        #$00FF
                    STA        INextSprite_X0+1     ; Sprite X0
                    LDA        SE_SPRITE_ID,X
                    DEC
                    ASL
                    TAY
                    LDA        TAB_SPRITE_SIZE,Y    ; Y : Sprite Size
                    TAY                    
*----
INextSprite_X0      LDA        #$AAAA               ; Sprite X0
                    STA        SE_SPRITE_X0,X
INextSprite_Y0      LDA        #$AAAA               ; Sprite Y0
                    STA        SE_SPRITE_Y0,X
INextSprite_X1      LDA        SE_SPRITE_X0,X
                    CLC
                    ADC:       $0000,Y              ; Sprite Width (byte)
                    DEC        
                    STA        SE_SPRITE_X1,X
INextSprite_Y1      LDA        SE_SPRITE_Y0,X
                    CLC
                    ADC:       $0002,Y              ; Sprite Height (line)
                    DEC
                    STA        SE_SPRITE_Y1,X
INextSprite_ADDR    LDA        INextSprite_Y0+1     ; Sprite Address
                    ASL
                    TAY
                    LDA        SHR_LINE_TABLE,Y
                    CLC
                    ADC        INextSprite_X0+1
                    CLC
                    ADC        #$2000
                    STA        SE_SPRITE_ADDR,X
                    LDA        SE_SPRITE_STATUS,X   ; Sprite Status : None
                    BNE        INextSprite_End
                    LDA        #$0001               ; Sprite Status : Init
                    STA        SE_SPRITE_STATUS,X
*----
INextSprite_End     RTS


**********************************************************************
*  WorkPendingSprite() :  Workflow for Pending Sprite (IN:X=Sprite). *
**********************************************************************
WorkPendingSprite   PHX
*--
                    JSR        (SE_SPRITE_FUNC_MOVE,X) ; Call AI Move Function
*--
                    PLX
                    RTS


********************************************************************************************
*  ProcessSequenceNext() :  Compute next Sprite ID / Position in a Sequence (IN:X=Sprite). *
********************************************************************************************                 
ProcessSequenceNext PHX                             ; Compute next Sprite ID / X0 / Y0 
*--
                    LDA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    BNE        PSN_Begin
                    JMP        PSN_NoMove
PSN_Begin           ASL
                    TAY
                    LDA        TAB_MOVE_SEQUENCE,Y  ; Sequence Data
                    TAY
*--
                    LDA        SE_SPRITE_MOVE_FRAM,X ; Current Frame (1-N)
                    CMP:       $0000,Y              ; Total Frame number
                    BNE        PSN_Sequence
                    JMP        PSN_Pending                 
*--
PSN_Sequence        STY        PSN_Sequence_2+1
                    DEC                             ; Frame # (1-N)
                    ASL
                    STA        PSN_Sequence_1+1
                    ASL                             ; 6 bytes / Sequence (DeltaX + DeltaY + Next Sprite ID)
                    CLC
PSN_Sequence_1      ADC        #$AAAA               ; *6
                    ADC        #$0004               ; #Frame + First Sprite ID
PSN_Sequence_2      ADC        #$AAAA               ; @ Begin MOVE_SEQUENCE_XXX
                    TAY
                    LDA:       $0004,Y              ; Next Sprite ID
                    STA        PSN_SpriteID+1
*--
PSN_DeltaX          STZ        PSN_Sprite_SignX     ; Delta X
                    LDA:       $0000,Y
                    BIT        #$8000               ; Bit #15 set the sign
                    BEQ        PSN_DeltaX_1
                    INC        PSN_Sprite_SignX                      
PSN_DeltaX_1        AND        #$00FF
                    STA        PSN_Sprite_DeltaX
*--
PSN_DeltaY          STZ        PSN_Sprite_SignY     ; Delta Y
                    LDA:       $0002,Y
                    BIT        #$8000               ; Bit #15 set the sign
                    BEQ        PSN_DeltaY_1
                    INC        PSN_Sprite_SignY                      
PSN_DeltaY_1        AND        #$00FF
                    STA        PSN_Sprite_DeltaY
*--                                                     
PSN_New_X0          LDA        PSN_Sprite_SignX     ; Compute New X0
                    BEQ        PSN_New_X0_1
                    LDA        SE_SPRITE_X0,X
                    SEC
                    SBC        PSN_Sprite_DeltaX
                    STA        PSN_Sprite_X0
                    BRA        PSN_New_Y0
PSN_New_X0_1        LDA        SE_SPRITE_X0,X
                    CLC
                    ADC        PSN_Sprite_DeltaX
                    STA        PSN_Sprite_X0
*--                                                   
PSN_New_Y0          LDA        PSN_Sprite_SignY     ; Compute New Y0
                    BEQ        PSN_New_Y0_1
                    LDA        SE_SPRITE_Y0,X
                    SEC
                    SBC        PSN_Sprite_DeltaY
                    STA        PSN_Sprite_Y0
                    BRA        PSN_SpriteInfo
PSN_New_Y0_1        LDA        SE_SPRITE_Y0,X
                    CLC
                    ADC        PSN_Sprite_DeltaY
                    STA        PSN_Sprite_Y0            
*--
PSN_SpriteInfo      LDA        PSN_Sprite_X0        ; Next Sprite information
                    XBA
                    ORA        PSN_Sprite_Y0
                    TAY                             ; Y=SpriteX0|SpriteY0
PSN_SpriteID        LDA        #$AAAA               ; A=SpriteID
                    JSR        InstallNextSprite    ; Next Sprite
                    INC        SE_SPRITE_MOVE_FRAM,X ; Next Sequence Frame #
                    BRA        PSN_End
*--
PSN_Pending         LDA        #$0002               ; Sprite Status = Pending
                    STA        SE_SPRITE_STATUS,X
                    JSR        BackupSpriteInfo     ; Move Sprite Current to Sprite Prev
                    STZ        SE_SPRITE_MOVE_SEQ,X ; Reset Sequence ID
                    STZ        SE_SPRITE_MOVE_FRAM,X ; Reset Frame Counter
                    BRA        PSN_End
*--
PSN_NoMove          JSR        BackupSpriteInfo     ; Move Sprite Current to Sprite Prev
*--
PSN_End             PLX
                    RTS                    

*----------   ProcessSequenceNext Variables   ---------              
PSN_Sprite_DeltaX   HEX        0000                 ; Sequence Frame Delta X value
PSN_Sprite_DeltaY   HEX        0000                 ; Sequence Frame Delta Y value
PSN_Sprite_SignX    HEX        0000                 ; Sequence Frame Delta X sign
PSN_Sprite_SignY    HEX        0000                 ; Sequence Frame Delta Y sign
PSN_Sprite_X0       HEX        0000                 ; Sprite X0
PSN_Sprite_Y0       HEX        0000                 ; Sprite Y0


**************************************************************************************************
*  CheckScrRightBorder() :  Control Sprite Position on the Screen (IN:A=SpriteX0,IN:Y=SpriteID). *
**************************************************************************************************
CheckScrRightBorder STA        CSRB_Sprite_X0       ; Make Sure Sprite don't overlap right border
                    TYA
                    DEC
                    ASL
                    TAY
                    LDA        TAB_SPRITE_SIZE,Y    ; Sprite Size Tab
                    TAY
                    LDA:       $0000,Y              ; Sprite Width
                    CLC
                    ADC        CSRB_Sprite_X0
                    DEC
                    CMP        #$00A0               ; Screen width = 160 bytes
                    BLT        CSRB_NoOverlap
                    LDA        $00A0                ; Re-center the Sprite                  
                    SEC
                    SBC:       $0000,Y 
                    BRA        CSRB_End
*--
CSRB_NoOverlap      LDA        CSRB_Sprite_X0
CSRB_End            RTS

*----------   CheckScrRightBorder Variables   ---------    
CSRB_Sprite_X0      HEX        0000


*******************************************************
*  BuildOverlapTable() :  Build Sprite Overlap Table. *
*******************************************************
BuildOverlapTable   LDA        SE_MAX_SPRITE_NB
                    ASL
                    STA        BOT_Check_Limit+1
                    DEC
                    DEC
                    STA        BOT_Limit+1          ; Do not test last Sprite (already found overlap with previous one)
*--
                    TAX                             ; Backup Overlap Table to Prev
BOT_Backup          LDA        SE_SPRITE_OVERLAP,X
                    STA        SE_SPRITE_OVERLAPR,X
                    STZ        SE_SPRITE_OVERLAP,X  ; Clear Overlap Table
                    DEX
                    DEX
                    BPL        BOT_Backup
*--
                    LDX        #$0000
BOT_Loop            LDA        SE_SPRITE_ID,X       ; Is valid Sprite ?
                    BEQ        BOT_Next             ; No Sprite
*--
                    TXY                             ; Check if this Sprite overlap with following ones
                    INY
                    INY
BOT_Check_Loop      LDA        SE_SPRITE_ID,Y       ; Is valid Sprite ?
                    BEQ        BOT_Check_Next       ; No Sprite
                    LDA        SE_SPRITE_OVERLAP,X  ; Both alredy Overlap ?
                    BEQ        BOT_Check
                    LDA        SE_SPRITE_OVERLAP,Y
                    BNE        BOT_Check_Next
BOT_Check           JSR        ComputeSpriteOverlap ; Compute if sprites X and Y overlap (0=No,1=Yes)
                    BEQ        BOT_Check_Next
                    LDA        #$0001               ; Sprites X and Y Overlap, Mark both
                    STA        SE_SPRITE_OVERLAP,X
                    STA        SE_SPRITE_OVERLAP,Y                                           
BOT_Check_Next      INY                             ; Next one
                    INY
BOT_Check_Limit     CPY        #$AAAA
                    BNE        BOT_Check_Loop
*--
BOT_Next            INX
                    INX
BOT_Limit           CPX        #$AAAA                                         
                    BNE        BOT_Loop
*--
                    RTS
                    

*****************************************************************************************************************
*  ComputeSpriteOverlap() :  Compute if two sprites overlap each other(IN:X=SpriteX,IN:Y=SpriteY,OUT:A=Result). *
*****************************************************************************************************************
ComputeSpriteOverlap                                ; X and Y must be valid Sprites
*----
CSO_X_X0_X0X1       LDA        SE_SPRITE_X0,X       ; Check X overlap : X_X0 between Y_X0:Y_X1 
                    CMP        SE_SPRITE_X0,Y
                    BLT        CSO_X_X1_X0X1        ; Outside
                    CMP        SE_SPRITE_X1,Y
                    BLT        CSO_X_Y0_Y0Y1        ; Overlap
                    BEQ        CSO_X_Y0_Y0Y1        ; Overlap
*--
CSO_X_X1_X0X1       LDA        SE_SPRITE_X1,X       ; Check X overlap : X_X1 between Y_X0:Y_X1
                    CMP        SE_SPRITE_X0,Y
                    BLT        CSO_Y_X0_X0X1        ; Outside
                    CMP        SE_SPRITE_X1,Y
                    BLT        CSO_X_Y0_Y0Y1        ; Overlap
                    BEQ        CSO_X_Y0_Y0Y1        ; Overlap
*--
CSO_Y_X0_X0X1       LDA        SE_SPRITE_X0,Y       ; Check X overlap : Y_X0 between X_X0:X_X1
                    CMP        SE_SPRITE_X0,X
                    BLT        CSO_NoOverlap        ; Outside
                    CMP        SE_SPRITE_X1,X
                    BLT        CSO_X_Y0_Y0Y1        ; Overlap
                    BEQ        CSO_X_Y0_Y0Y1        ; Overlap
                    BRA        CSO_NoOverlap        ; Outside
*----
CSO_X_Y0_Y0Y1       LDA        SE_SPRITE_Y0,X       ; Check Y overlap : X_Y0 between Y_Y0:Y_Y1 
                    CMP        SE_SPRITE_Y0,Y
                    BLT        CSO_X_Y1_Y0Y1        ; Outside
                    CMP        SE_SPRITE_Y1,Y
                    BLT        CSO_Overlap          ; Overlap
                    BEQ        CSO_Overlap          ; Overlap
*--
CSO_X_Y1_Y0Y1       LDA        SE_SPRITE_Y1,X       ; Check Y overlap : X_Y1 between Y_Y0:Y_Y1
                    CMP        SE_SPRITE_Y0,Y
                    BLT        CSO_Y_Y0_Y0Y1        ; Outside
                    CMP        SE_SPRITE_Y1,Y
                    BLT        CSO_Overlap          ; Overlap
                    BEQ        CSO_Overlap          ; Overlap
*--
CSO_Y_Y0_Y0Y1       LDA        SE_SPRITE_Y0,Y       ; Check Y overlap : Y_Y0 between X_Y0:X_Y1
                    CMP        SE_SPRITE_Y0,X
                    BLT        CSO_NoOverlap        ; Outside
                    CMP        SE_SPRITE_Y1,X
                    BLT        CSO_Overlap          ; Overlap
                    BEQ        CSO_Overlap          ; Overlap
                    BRA        CSO_NoOverlap        ; Outside
*--
CSO_Overlap         LDA        #$0001
                    BRA        CSO_End
*--
CSO_NoOverlap       LDA        #$0000             
*--
CSO_End             RTS


*****************************************************************
*  BackupOverlapTable() :  Backup Sprite Overlap Table to Prev. *
*****************************************************************
BackupOverlapTable  LDA        SE_MAX_SPRITE_NB     ; Copy previous Overlap Table
                    DEC
                    ASL
                    TAX                             ; X = (SE_MAX_SPRITE_NB*SE_MAX_SPRITE_NB)*2 - 2                  
BOTP_Copy           LDA        SE_SPRITE_OVERLAP,X
                    STA        SE_SPRITE_OVERLAPR,X
                    DEX
                    DEX
                    BPL        BOTP_Copy
*--
                    RTS


********************************************************
*  BuildSequenceWidth() :  Build Sequence Width Table. *
********************************************************
BuildSequenceWidth  LDX        #$0000
BSW_Begin           LDA        TAB_MOVE_SEQUENCE,X
                    BEQ        BSW_End
                    PHX
                    TAY
*--
BSW_Init            STZ        BSW_Delta_X_POS      ; Init
                    STZ        BSW_Delta_X_NEG
                    LDA:       $0000,Y              ; Number of Sprites
                    BEQ        BSW_Compute          ; No Sprite
                    STY        BSW_Prepare+1
                    DEC
                    ASL
                    TAX
                    LDA        BES_MultX6,X         ; *6
                    CLC
                    ADC        #$0004               ; Sprite # + Sprite ID
BSW_Prepare         ADC        #$AAAA
                    STA        BSW_Limit+1
                    INY
                    INY
                    INY
                    INY
BSW_Loop            LDA:       $0000,Y              ; Delta X
                    BIT        #$8000
                    BNE        BSW_Neg 
                    CLC                             ; Positive
                    ADC        BSW_Delta_X_POS
                    STA        BSW_Delta_X_POS
                    BRA        BSW_Next
BSW_Neg             AND        #$7FFF               ; Negative
                    CLC
                    ADC        BSW_Delta_X_NEG
                    STA        BSW_Delta_X_NEG
BSW_Next            TYA
                    CLC
                    ADC        #$0006               ; Next Delta X
BSW_Limit           CMP        #$AAAA
                    BEQ        BSW_Compute
                    TAY
                    BRA        BSW_Loop
*--
BSW_Compute         LDA        BSW_Delta_X_POS      ; Compute Signed Delta X
                    CMP        BSW_Delta_X_NEG
                    BLT        BSW_DeltaNeg
                    SEC                             ; Positive
                    SBC        BSW_Delta_X_NEG
                    BRA        BSW_Store
BSW_DeltaNeg        LDA        BSW_Delta_X_NEG      ; Negative
                    SEC
                    SBC        BSW_Delta_X_POS
                    ORA        #$8000 
BSW_Store           PLX
                    STA        TAB_SEQUENCE_WIDTH,X ; Signed Delta X
                    INX
                    INX
                    BRA        BSW_Begin
*--
BSW_End             RTS

*----------   BuildSequenceWidth Variables   ---------              
BSW_Delta_X_POS     HEX        0000                 ; Debug
BSW_Delta_X_NEG     HEX        0000


******************************************************************************
*  BuildEraseSequence() :  Build Erase for all Sequences (OUT:A=NbEraseSeq). *
******************************************************************************
BuildEraseSequence  LDA        #$0001               ; Init
                    STA        BES_SpriteID
                    LDA        #ERASE_SPR_0XX       ; Sprite Erase Sequence Address
                    STA        BES_SprEraseSeqAddr
*--
BES_OneSprite       STZ        BES_SpriteEraseSeq   ; Number of Erase Sequence for this Sprite
                    LDX        #$0002               ; Search for this SpriteID as First Sprite                    
BES_OneSequence     LDA        TAB_MOVE_SEQUENCE,X
                    BEQ        BES_ExtraSequence
                    TAY
                    STY        BES_OneSequence_1+1
                    PHX
                    LDA:       $0000,Y              ; Number of Sprites
                    DEC
                    ASL
                    TAX
                    LDA        BES_MultX6,X         ; *6
                    CLC
BES_OneSequence_1   ADC        #$AAAA
                    STA        BES_OneSequence_5+1                    
BES_OneSequence_2   LDA:       $0002,Y              ; First Sprite ID
                    CMP        BES_SpriteID
                    BNE        BES_OneSequence_4
*--
                    PHY
                    JSR        AddEraseSequence     ; Add this Erase Sequence to Table
                    BCC        BES_OneSequence_3
                    INC        BES_SpriteEraseSeq
BES_OneSequence_3   PLY
*--
BES_OneSequence_4   TYA
                    CLC
                    ADC        #$0006
                    TAY
BES_OneSequence_5   CPY        #$AAAA               ; End of Sequence
                    BNE        BES_OneSequence_2        
*--                        
BES_NextSequence    PLX
                    INX
                    INX
                    BRA        BES_OneSequence
*----
BES_ExtraSequence   LDY        #ERASE_SEQ_EXTRA-2   ; Insert all Extra Sequence for this ID
BES_ExtraSequence_1 LDA:       $0002,Y
                    BEQ        BES_NextSprite
                    CMP        BES_SpriteID
                    BNE        BES_ExtraSequence_3
                    PHY
                    JSR        AddEraseSequence     ; Add this Erase Sequence to Table
                    BCC        BES_ExtraSequence_2
                    INC        BES_SpriteEraseSeq                    
BES_ExtraSequence_2 PLY
BES_ExtraSequence_3 TYA
                    CLC
                    ADC        #$0008               ; 8 bytes per Extra Erase Sequence
                    TAY
                    BRA        BES_ExtraSequence_1
*--
BES_NextSprite      LDA        BES_TotalEraseSeq    ; Global Stat
                    CLC
                    ADC        BES_SpriteEraseSeq
                    STA        BES_TotalEraseSeq
*-
BES_NextSprite_Addr LDA        BES_SpriteEraseSeq   ; Sprite Erase Sequence @
                    BEQ        BES_NextSprite_ID
                    ASL
                    TAX
                    LDA        BES_MultX10,X        ; 10 bytes per Erase Sequence 
                    STA        BES_NextSprite_1+1
                    LDA        BES_SpriteID
                    DEC
                    ASL
                    TAX
                    LDA        BES_SprEraseSeqAddr  ; Begin Erase Sequence Address for current Sprite ID
                    STA        TAB_ERASE_BEGIN,X
                    CLC
BES_NextSprite_1    ADC        #$AAAA
                    STA        BES_SprEraseSeqAddr  ; Begin Erase Sequence Address for next Sprite ID
*-
BES_NextSprite_ID   LDA        BES_SpriteID         ; Next Sprite ID
                    INC
                    STA        BES_SpriteID
                    CMP        #$008B               ; 8A = Last Sprite ID
                    BEQ        BES_End
                    JMP        BES_OneSprite
*--         
BES_End             LDA        BES_TotalEraseSeq    ; Number of Erase Sequence built
                    RTS
                    
*----------   BuildEraseCode Variables   ---------   
BES_SpriteID        HEX        0000                 ; Sprite ID
BES_SpriteEraseSeq  HEX        0000                 ; Sprite Number of Erase Sequence
BES_TotalEraseSeq   HEX        0000                 ; Total Number of Erase Sequence
BES_SprEraseSeqAddr HEX        0000                 ; Sprite Erase Sequence Address
BES_MultX6          HEX        0000,0600,0C00,1200  ; *6 Multiplication Table
                    HEX        1800,1E00,2400,2A00
                    HEX        3000,3600,3C00,4200
                    HEX        4800,4E00,5400,5A00
                    HEX        6000,6600,6C00,7200
                    HEX        7800,7E00,8400,8A00
BES_MultX10         HEX        0000,0A00,1400,1E00  ; *10 Multiplication Table
                    HEX        2800,3200,3C00,4600
                    HEX        5000,5A00,6400,6E00
                    HEX        7800,8200,8C00,9600
                    HEX        A000,AA00,B400,BE00
                    HEX        C800,D200,DC00,E600
                    HEX        F000,FA00,0401,0E01
                    HEX        1801,2201,2C01,3601


*****************************************************************************************
*  AddEraseSequence() :  Add Erase Sequence in Table (IN:Y=@Sequence,OUT:Carry=1IfNew). *
*****************************************************************************************
AddEraseSequence    LDA:       $0002,Y              ; Sprite Src ID
                    ASL
                    TAX
                    LDA        TAB_ERASE_EQUIV,X    ; Get Shadow equivalence for this Sprite ID
                    STA        AES_Sequence+0
                    LDA:       $0004,Y              ; Delta X
                    STA        AES_Sequence+2
                    LDA:       $0006,Y              ; Delta Y
                    STA        AES_Sequence+4
                    LDA:       $0008,Y              ; Sprite Dst ID
                    ASL
                    TAX
                    LDA        TAB_ERASE_EQUIV,X    ; Get Shadow equivalence for this Sprite ID
                    STA        AES_Sequence+6
*--
AES_Search          LDX        #ERASE_SPR_0XX       ; Search for a free space
AES_Search_1        LDA:       $0002,X              ; Sprite Src ID
                    BEQ        AES_Insert
                    CMP        AES_Sequence+0       ; Check Duplicates
                    BNE        AES_Next
                    LDA:       $0004,X              ; Delta X
                    CMP        AES_Sequence+2
                    BNE        AES_Next          
                    LDA:       $0006,X              ; Delta Y
                    CMP        AES_Sequence+4
                    BNE        AES_Next          
                    LDA:       $0008,X              ; Sprite Dst ID
                    CMP        AES_Sequence+6
                    BEQ        AES_End              ; Same Sequence => Exit
*--
AES_Next            TXA
                    CLC
                    ADC        #$000A               ; Erase Sequence Size
                    TAX
                    BRA        AES_Search_1
*--
AES_Insert          LDA        #$AAAA               ; Future Sequence ID
                    STA:       $0000,X
                    LDA        AES_Sequence+0       ; Sprite Src ID
                    STA:       $0002,X
                    LDA        AES_Sequence+2       ; Delta X
                    STA:       $0004,X
                    LDA        AES_Sequence+4       ; Delta Y
                    STA:       $0006,X
                    LDA        AES_Sequence+6       ; Sprite Dst ID
                    STA:       $0008,X
                    SEC                             ; Added to the Erase Sequence List
                    RTS
*--                    
AES_End             CLC                             ; Not Added to the Erase Sequence List
                    RTS

*----------   AddEraseSequence Variables   ---------   
AES_Sequence        HEX        0000,0000,0000,0000  ; SpriteID DeltaX DeltaY SpriteID


************************************************************
*  BuildAllEraseCode() :  Build Code for All Erase Sprite. *
************************************************************
BuildAllEraseCode   LDX        #ERASE_SPR_0XX       ; Init EraseID list
                    LDY        #$0001               ; EraseID : 1-N
BAEC_Init           LDA:       $0000,X
                    BEQ        BAEC_Code
                    TYA
                    STA:       $0000,X
                    TXA
                    CLC
                    ADC        #$000A               ; Next Erase
                    TAX
                    INY
                    BRA        BAEC_Init
*--                    
BAEC_Code           LDX        #ERASE_SPR_0XX
BAEC_Code_1         LDA:       $0000,X              ; Erase ID
                    BEQ        BAEC_End
                    PHX
                    JSR        BuildEraseCode
*-
*                    LDA        #$AAAA
*                    LDX        #$7CFE
*BAEC_Erase          STAL       $012000,X
*                    DEX
*                    DEX
*                    BPL        BAEC_Erase
*-
                    PLA
                    CLC
                    ADC        #$000A               ; Next Erase Sprite
                    TAX
                    BRA        BAEC_Code_1
*--
BAEC_End            RTS


*****************************************************************************
*  BuildEraseCode() :  Build Code for one Erase Sprite (IN:X=EraseSprite@). *
*****************************************************************************
BuildEraseCode      LDA:       $0000,X              ; Erase Sprite ID
                    STA        BEC_EraseSpriteID
                    LDA:       $0002,X              ; Sprite #1 ID
                    STA        BEC_Sprite1_ID
                    LDA:       $0008,X              ; Sprite #2 ID
                    STA        BEC_Sprite2_ID
                    LDA:       $0006,X              ; Signed Delta Y
                    STA        BEC_DrawSprite2_DY+1
                    TAY                            
                    LDA:       $0004,X              ; Signed Delta X
                    STA        BEC_DrawSprite2_DX+1
                    TAX                         
*------
BuildEraseCode_Dx   STZ        BEC_DeltaX_Sign      ; Compute Delta X
                    TXA
                    AND        #$7FFF
                    STA        BEC_DeltaX
                    TXA
                    AND        #$8000
                    BEQ        BuildEraseCode_Dy
                    INC        BEC_DeltaX_Sign
*--
BuildEraseCode_Dy   STZ        BEC_DeltaY_Sign      ; Compute Delta Y
                    TYA
                    AND        #$7FFF
                    STA        BEC_DeltaY
                    TYA
                    AND        #$8000
                    BEQ        BuildEraseCode_Sz
                    INC        BEC_DeltaY_Sign
*------
BuildEraseCode_Sz   LDA        BEC_Sprite1_ID       ; Get Sprite #1 size
                    DEC
                    ASL
                    TAX
                    LDA        TAB_SPRITE_SIZE,X
                    TAX
                    LDA:       $0000,X              ; Sprite Width (byte)
                    STA        BEC_Sprite1_Width
                    LDA:       $0002,X              ; Sprite High (line)
                    STA        BEC_Sprite1_Heigh
*--
                    LDA        BEC_Sprite2_ID       ; Get Sprite #2 size
                    DEC
                    ASL
                    TAX
                    LDA        TAB_SPRITE_SIZE,X
                    TAX
                    LDA:       $0000,X              ; Sprite Width (byte)
                    STA        BEC_Sprite2_Width
                    LDA:       $0002,X              ; Sprite Heigh (line)
                    STA        BEC_Sprite2_Heigh
*------
BEC_ClearArea1      LDA        #$2000               ; Clear Sprite #1 Area 
                    STA        BEC_ClearArea1Addr+1
                    LDY        BEC_Sprite1_Heigh
                    DEY
BEC_ClearArea1_1    SEP        #$20
                    LDX        BEC_Sprite1_Width
                    INX                             ; Erase 4 pixels more
                    LDA        #$FF
BEC_ClearArea1Addr  STAL       $012000,X
                    DEX
                    BPL        BEC_ClearArea1Addr
                    REP        #$30
                    LDA        BEC_ClearArea1Addr+1
                    CLC
                    ADC        #$00A0
                    STA        BEC_ClearArea1Addr+1
                    DEY
                    BPL        BEC_ClearArea1_1
*--
BEC_ClearArea2      LDA        #$2050               ; Clear Sprite #2 Area 
                    STA        BEC_ClearArea2Addr+1
                    LDY        BEC_Sprite2_Heigh
                    DEY
BEC_ClearArea2_1    SEP        #$20
                    LDX        BEC_Sprite2_Width
                    DEX
                    LDA        #$FF
BEC_ClearArea2Addr  STAL       $012050,X
                    DEX
                    BPL        BEC_ClearArea2Addr
                    REP        #$30
                    LDA        BEC_ClearArea2Addr+1
                    CLC
                    ADC        #$00A0
                    STA        BEC_ClearArea2Addr+1
                    DEY
                    BPL        BEC_ClearArea2_1
*------
BEC_DrawSprite1     LDA        BEC_Sprite1_ID       ; Draw Sprite #1 at $012000
                    LDY        #$2000               ; Y = @Screen
                    JSR        DrawSpriteByID       ; Draw Sprite / A = Sprite ID (1-N)
*--
BEC_DrawSprite2     LDA        BEC_Sprite2_ID       ; Draw Sprite #2 at $012050
                    CMP        BEC_Sprite1_ID       ; Check for Full Erase (Same ID,DeltaXY=0)
                    BNE        BEC_DrawSprite2_1
BEC_DrawSprite2_DX  LDA        #$AAAA               ; Delta X
                    BNE        BEC_DrawSprite2_1
BEC_DrawSprite2_DY  LDA        #$AAAA               ; Delta Y
                    BEQ        BEC_Build_X0_Width
BEC_DrawSprite2_1   LDY        #$2050               ; Y = @Screen
                    LDA        BEC_Sprite2_ID       ; A = Sprite ID (1-N)
                    JSR        DrawSpriteByID       ; Draw Sprite
*------
*      JSL        IncBorderColor
*      JSL        WaitForKey
*------
BEC_Build_X0_Width  LDA        BEC_DeltaX_Sign      ; Define overlap surface
                    BEQ        BEC_DX_Pos
BEC_DX_Neg          STZ        BEC_Sprite1_X0
                    LDA        BEC_DeltaX
                    CMP        BEC_Sprite2_Width    ; Dx >= Spr2Width => No Overlap
                    BLT        BEC_DX_Neg_1
                    JMP        BEC_NoOverlap
BEC_DX_Neg_1        STA        BEC_Sprite2_X0       ; Spr2X0 = Dx
                    LDA        BEC_Sprite2_Width
                    SEC
                    SBC        BEC_DeltaX
                    STA        BEC_Overlap_Width    ; OverlapWidth
                    CMP        BEC_Sprite1_Width
                    BLT        BEC_Build_Y0_Heigh
                    LDA        BEC_Sprite1_Width
                    STA        BEC_Overlap_Width
                    BRA        BEC_Build_Y0_Heigh
BEC_DX_Pos          LDA        BEC_DeltaX
                    STA        BEC_Sprite1_X0
                    CMP        BEC_Sprite1_Width
                    BGE        BEC_NoOverlap
                    STZ        BEC_Sprite2_X0
                    LDA        BEC_Sprite1_Width
                    SEC
                    SBC        BEC_DeltaX
                    STA        BEC_Overlap_Width
                    CMP        BEC_Sprite2_Width
                    BMI        BEC_Build_Y0_Heigh
                    LDA        BEC_Sprite2_Width
                    STA        BEC_Overlap_Width
*--
BEC_Build_Y0_Heigh  LDA        BEC_DeltaY_Sign
                    BEQ        BEC_DY_Pos
BEC_DY_Neg          STZ        BEC_Sprite1_Y0
                    LDA        BEC_DeltaY
                    CMP        BEC_Sprite2_Width    ; Dy >= Spr2Heigh => No Overlap
                    BGE        BEC_NoOverlap
                    STA        BEC_Sprite2_Y0
                    LDA        BEC_Sprite2_Heigh
                    SEC
                    SBC        BEC_DeltaY
                    STA        BEC_Overlap_Heigh
                    CMP        BEC_Sprite1_Heigh
                    BLT        BEC_ClearOverlap
                    LDA        BEC_Sprite1_Heigh
                    STA        BEC_Overlap_Heigh
                    BRA        BEC_ClearOverlap
BEC_DY_Pos          LDA        BEC_DeltaY
                    STA        BEC_Sprite1_Y0
                    CMP        BEC_Sprite1_Heigh
                    BGE        BEC_NoOverlap
                    STZ        BEC_Sprite2_Y0
                    LDA        BEC_Sprite1_Heigh
                    SEC
                    SBC        BEC_DeltaY
                    STA        BEC_Overlap_Heigh
                    CMP        BEC_Sprite2_Heigh
                    BMI        BEC_ClearOverlap
                    LDA        BEC_Sprite2_Heigh
                    STA        BEC_Overlap_Heigh
                    BRA        BEC_ClearOverlap
*------
BEC_NoOverlap       JMP        BEC_PrepareCode
*------
BEC_ClearOverlap    LDA        #$2050               ; Clear Pixels from Sprite #1 overlaping with Sprite #2  
                    CLC
                    ADC        BEC_Sprite2_X0
                    LDY        BEC_Sprite2_Y0
BEC_CO_Init_1       CPY        #$0000
                    BEQ        BEC_CO_Init_2
                    CLC
                    ADC        #$00A0
                    DEY
                    BRA        BEC_CO_Init_1
BEC_CO_Init_2       STA        BEC_CO_W+1
                    LDA        #$2000  
                    CLC
                    ADC        BEC_Sprite1_X0
                    LDY        BEC_Sprite1_Y0
BEC_CO_Init_3       CPY        #$0000
                    BEQ        BEC_CO_Init_4
                    CLC
                    ADC        #$00A0
                    DEY
                    BRA        BEC_CO_Init_3
BEC_CO_Init_4       STA        BEC_CO_W_2+1
                    STA        BEC_CO_W_3+1
*-
                    LDY        BEC_Overlap_Heigh
                    DEY
BEC_CO_H            LDX        BEC_Overlap_Width
                    DEX
                    SEP        #$20
*-
BEC_CO_W            LDAL       $012050,X            ; Sprite #2 data
                    CMP        #$FF
                    BEQ        BEC_CO_W_NB          ; FF
                    PHA
                    AND        #$0F
                    CMP        #$0F
                    BEQ        BEC_CO_W_0F          ; 0F
                    PLA       
                    AND        #$F0
                    CMP        #$F0
                    BEQ        BEC_CO_W_F0          ; F0
                    STZ        BEC_CO_W_AND+1
                    LDA        #$FF                 ; 00
                    BRA        BEC_CO_W_1
BEC_CO_W_0F         PLA
                    LDA        #$0F
                    STA        BEC_CO_W_AND+1
                    LDA        #$F0
                    BRA        BEC_CO_W_1
BEC_CO_W_F0         LDA        #$F0
                    STA        BEC_CO_W_AND+1
                    LDA        #$0F                                          
BEC_CO_W_1          STA        BEC_CO_W_ORA+1
BEC_CO_W_2          LDAL       $012000,X            ; Sprite #1 data
BEC_CO_W_AND        AND        #$AA                 ; 00 / 0F / F0
BEC_CO_W_ORA        ORA        #$BB                 ; FF / F0 / 0F
BEC_CO_W_3          STAL       $012000,X
BEC_CO_W_NB         DEX                             ; Next Byte
                    BPL        BEC_CO_W
*-
                    REP        #$30                 ; Next line
                    LDA        BEC_CO_W+1
                    CLC
                    ADC        #$00A0
                    STA        BEC_CO_W+1
                    LDA        BEC_CO_W_2+1
                    CLC
                    ADC        #$00A0
                    STA        BEC_CO_W_2+1
                    STA        BEC_CO_W_3+1
                    DEY
                    BPL        BEC_CO_H
*------
BEC_PrepareCode     LDA        #$2000               ; Turn Sprite #1 as 00 and FF
                    STA        BEC_PrepareCode_3+1
                    STA        BEC_PrepareCode_4+1
*-
BEC_PrepareCode_1   LDY        BEC_Sprite1_Heigh
                    DEY
*-
BEC_PrepareCode_2   LDX        BEC_Sprite1_Width
                    DEX
                    SEP        #$20
BEC_PrepareCode_3   LDAL       $012000,X
                    CMP        #$FF
                    BEQ        BEC_PrepareCode_5
                    LDA        #$00
BEC_PrepareCode_4   STAL       $012000,X
BEC_PrepareCode_5   DEX
                    BPL        BEC_PrepareCode_3
*-
                    REP        #$30
                    LDA        BEC_PrepareCode_3+1
                    CLC
                    ADC        #$00A0
                    STA        BEC_PrepareCode_3+1
                    STA        BEC_PrepareCode_4+1
                    DEY
                    BPL        BEC_PrepareCode_2
*------
BEC_BuildOffset     STZ        BEC_Code_Offset      ; Build LDA/STA Code at $01/6000
*--
BEC_BuildOffset16   STZ        BEC_Code_16Bit       ; Build 16 bit offset list
                    LDA        #$2000               
                    STA        BEC_BuildOffset16_2+1
                    STA        BEC_BuildOffset16_3+1
*-
                    LDY        #$0000
*-
BEC_BuildOffset16_1 LDX        #$0000
BEC_BuildOffset16_2 LDAL       $012000,X
                    CMP        #$FFFF               ; Full 16 bit background => Ignore 
                    BEQ        BEC_BuildOffset16_4
                    CMP        #$0000               ; Full 16 bit data => Process
                    BNE        BEC_BuildOffset16_5
                    TXA                             ; Keep this offset
                    CLC
                    ADC        BEC_BuildOffset16_2+1
                    SEC
                    SBC        #$2000
                    PHX
                    LDX        BEC_Code_Offset
                    STAL       $015E81,X            ; LDA @
                    STAL       $015E84,X            ; STA @
                    SEP        #$20
                    LDA        #$BD
                    STAL       $015E80,X            ; LDA ,X
                    LDA        #$99
                    STAL       $015E83,X            ; STA ,Y
                    REP        #$30
                    PLX           
                    LDA        BEC_Code_Offset      ; Next offset
                    CLC
                    ADC        #$0006
                    STA        BEC_Code_Offset
                    INC        BEC_Code_16Bit       ; # 16 bit pattern
                    LDA        #$FFFF               ; Clear this area
BEC_BuildOffset16_3 STAL       $012000,X
BEC_BuildOffset16_4 INX
BEC_BuildOffset16_5 INX
                    CPX        BEC_Sprite1_Width
                    BLT        BEC_BuildOffset16_2
*-
                    LDA        BEC_BuildOffset16_2+1 ; Next line
                    CLC
                    ADC        #$00A0
                    STA        BEC_BuildOffset16_2+1
                    STA        BEC_BuildOffset16_3+1
                    INY
                    CPY        BEC_Sprite1_Heigh
                    BNE        BEC_BuildOffset16_1
*--
                    STZ        BEC_Code_8Bit        ; Build 8 Bit offset list
                    LDA        #$2000               
                    STA        BEC_BuildOffset8_2+1
*-
                    LDY        #$0000
*-
BEC_BuildOffset8_1  LDX        #$0000
                    SEP        #$20
BEC_BuildOffset8_2  LDAL       $012000,X
                    CMP        #$FF                 ; Full 8 bit background => Ignore 
                    BEQ        BEC_BuildOffset8_3
                    REP        #$30                 ; Keep this offset
                    TXA                             
                    CLC
                    ADC        BEC_BuildOffset8_2+1
                    SEC
                    SBC        #$2000
                    PHX
                    LDX        BEC_Code_Offset
                    STAL       $015E81,X            ; LDA @
                    STAL       $015E84,X            ; STA @
                    SEP        #$20
                    LDA        #$BD
                    STAL       $015E80,X            ; LDA ,X
                    LDA        #$99
                    STAL       $015E83,X            ; STA ,Y
                    REP        #$30 
                    PLX           
                    LDA        BEC_Code_Offset      ; Next offset
                    CLC
                    ADC        #$0006
                    STA        BEC_Code_Offset
                    INC        BEC_Code_8Bit        ; # 8 bit pattern
                    SEP        #$20
BEC_BuildOffset8_3  INX
                    CPX        BEC_Sprite1_Width
                    BNE        BEC_BuildOffset8_2
*-
                    REP        #$30
                    LDA        BEC_BuildOffset8_2+1 ; Next line
                    CLC
                    ADC        #$00A0
                    STA        BEC_BuildOffset8_2+1
                    INY
                    CPY        BEC_Sprite1_Heigh
                    BNE        BEC_BuildOffset8_1
*-----
BEC_ComputeCodeSize LDA        #$0001               ; Final RTL
                    CLC
                    ADC        BEC_Code_Offset      ; Code Size
                    STA        BEC_Code_Size
*-                    
                    LDA        BEC_Code_16Bit       ; 16 Bit Size + 8 Bit Size == 0 ? => No Code
                    CLC
                    ADC        BEC_Code_8Bit        ; 8 Bit Size
                    BEQ        BEC_SearchAddress
*-
                    INC        BEC_Code_Size        ; First TAX
*-
                    LDA        BEC_Code_8Bit        ; 8 Bit Size extra
                    BEQ        BEC_SearchAddress
                    LDA        BEC_Code_Size
                    CLC
                    ADC        #$0004               ; SEP #$20 + REP #$30
                    STA        BEC_Code_Size
*----
BEC_SearchAddress   LDY        #$0000               ; Search Location where to store code
BEC_SearchAddress_0 LDA        TabEraseCodeSize,Y
                    CLC
                    ADC        BEC_Code_Size
                    CMP        TabEraseCodeSize,Y
                    BGE        BEC_SearchAddress_1
                    INY
                    INY
                    BRA        BEC_SearchAddress_0                    
BEC_SearchAddress_1 LDA        BEC_EraseSpriteID
                    DEC
                    ASL
                    TAX
                    LDA        TabEraseCodeSize,Y   ; Code Address
                    STA        TAB_ERASE_ADDRESS,X
                    LDA        TabEraseCodeSize,Y   ; New Code Size for this bank
                    CLC
                    ADC        BEC_Code_Size
                    STA        TabEraseCodeSize,Y
                    LDA        TabEraseCodeBank,Y   ; Code Bank
                    TAY
                    LDA:       $0000,Y
                    STA        TAB_ERASE_BANK,X  
*----
BEC_BuildCode       STA        BEC_BC_Begin_1+2     ; Code Bank
                    STA        BEC_BC_Final_1+2
                    STA        BEC_BC_8Bit_Sep+2
                    STA        BEC_BC_8Bit_Rep+2
                    XBA
                    ORA        #$0100               ; Bank Src = 01
                    STA        EOS_BC_16Bit_1+1
                    STA        EOS_BC_8Bit_1+1
                    LDA        TAB_ERASE_ADDRESS,X  ; Code Address
                    TAY                             ; Y = Dst @
*--                    
                    LDA        BEC_Code_16Bit       ; No Code ?
                    CLC
                    ADC        BEC_Code_8Bit
                    BEQ        BEC_BC_Final         ; Nothing
*--
BEC_BC_Begin        TYX
                    SEP        #$20
                    LDA        #$AA                 ; TAX
BEC_BC_Begin_1      STAL       $AA0000,X
                    REP        #$30
                    INY
*--
BEC_BC_16Bit        LDX        #$5E80               ; Src @ = $01/5E80
                    LDA        BEC_Code_16Bit       ; No 16 bit Code ?
                    BEQ        BEC_BC_8Bit
*-
                    ASL                             ; 16 bit Code Size
                    STA        EOS_BC_16Bit_0+1
                    ASL
                    CLC
EOS_BC_16Bit_0      ADC        #$AAAA               ; *6
                    DEC                             ; #bytes - 1
EOS_BC_16Bit_1      MVN        $01,$AA              ; Auto Increment X and Y
                    PHK
                    PLB
*--
BEC_BC_8Bit         LDA        BEC_Code_8Bit        ; No 8 bit Code ?
                    BEQ        BEC_BC_Final
*-
                    PHX
                    TYX                             ; SEP #$20
                    LDA        #$20E2
BEC_BC_8Bit_Sep     STAL       $AA0000,X
                    INY
                    INY
                    PLX
*-                    
                    LDA        BEC_Code_8Bit        ; 8 bit Code Size
                    ASL
                    STA        EOS_BC_8Bit_0+1
                    ASL
                    CLC
EOS_BC_8Bit_0       ADC        #$AAAA               ; *6
                    DEC                             ; #bytes - 1
EOS_BC_8Bit_1       MVN        $01,$AA              ; Auto Increment X and Y
                    PHK
                    PLB
*-
                    TYX                             ; REP #$30
                    LDA        #$30C2
BEC_BC_8Bit_Rep     STAL       $AA0000,X
                    INY
                    INY
*--
BEC_BC_Final        TYX
                    SEP        #$20
                    LDA        #$6B                 ; RTL
BEC_BC_Final_1      STAL       $AA0000,X
                    REP        #$30
*----
                    RTS

*----------   BuildEraseCode Variables   ---------   
BEC_Sprite1_ID      HEX        0000                 ; Sprite #1 ID
BEC_Sprite1_Width   HEX        0000                 ; Sprite #1 Width (byte)
BEC_Sprite1_Heigh   HEX        0000                 ; Sprite #1 Heigh (line)
BEC_Sprite2_ID      HEX        0000                 ; Sprite #2 ID
BEC_Sprite2_Width   HEX        0000                 ; Sprite #2 Width (byte)
BEC_Sprite2_Heigh   HEX        0000                 ; Sprite #2 Heigh (line)
BEC_DeltaX          HEX        0000                 ; Unsigned Delta X
BEC_DeltaX_Sign     HEX        0000                 ; Delta X Sign (0=+,1=-)
BEC_DeltaY          HEX        0000                 ; Unsigned Delta Y
BEC_DeltaY_Sign     HEX        0000                 ; Delta Y Signe(0=+,1=-)
BEC_Sprite1_X0      HEX        0000                 ; Overlap Area Coordinates in Sprite #1
BEC_Sprite1_Y0      HEX        0000
BEC_Sprite2_X0      HEX        0000                 ; Overlap Aread Coordinate in Sprite #2
BEC_Sprite2_Y0      HEX        0000
BEC_Overlap_Width   HEX        0000                 ; Sprites Overlap Area Width (byte)
BEC_Overlap_Heigh   HEX        0000                 ; Sprites Overlap Area Heigh (line)
BEC_Code_16Bit      HEX        0000                 ; # Address to patch in 16 bit
BEC_Code_8Bit       HEX        0000                 ; # Address to patch in 8 bit
BEC_Code_Offset     HEX        0000                 ; Current Code (16bit/8bit) Offset
BEC_Code_Size       HEX        0000                 ; Erase Code Size
BEC_EraseSpriteID   HEX        0000                 ; Current EraseSprite ID (1-N)


******************************************************************************
*  ComputeEraseID() :  Compute EraseID for a Sprite in Motion (IN:X=Sprite). *
******************************************************************************
ComputeEraseID      LDA        SE_SPRITE_PREV_ID,X  ; If Sprite Prev ID is 0000, no Erase ID
                    BNE        CEID_ShadowEquiv
                    JMP        CEID_End             ; Not Found
*--
CEID_ShadowEquiv    LDA        SE_SPRITE_ID,X       ; Get Sprite ID
                    ASL
                    TAY
                    LDA        TAB_ERASE_EQUIV,Y    ; Get Shadow equivalence for this Sprite ID
                    STA        CEID_SpriteID
*--
                    LDA        SE_SPRITE_PREV_ID,X  ; Get Sprite Prev ID
                    ASL
                    TAY
                    LDA        TAB_ERASE_EQUIV,Y    ; Get Shadow equivalence for this Sprite ID
                    STA        CEID_SpritePrevID
                    DEC
                    ASL
                    TAY
                    LDA        TAB_ERASE_BEGIN,Y    ; First EraseID for the Sprite Prev ID
                    TAY
*--
CEID_DeltaXPos      LDA        SE_SPRITE_X0,X       ; Compute Delta X
                    CMP        SE_SPRITE_PREV_X0,X
                    BLT        CEID_DeltaXNeg
                    SEC
                    SBC        SE_SPRITE_PREV_X0,X
                    STA        CEID_Delta_X
                    BRA        CEID_DeltaYPos
CEID_DeltaXNeg      LDA        SE_SPRITE_PREV_X0,X
                    SEC
                    SBC        SE_SPRITE_X0,X
                    ORA        #$8000
                    STA        CEID_Delta_X
*--
CEID_DeltaYPos      LDA        SE_SPRITE_Y0,X       ; Compute Delta Y
                    CMP        SE_SPRITE_PREV_Y0,X
                    BLT        CEID_DeltaYNeg
                    SEC
                    SBC        SE_SPRITE_PREV_Y0,X
                    STA        CEID_Delta_Y
                    BRA        CEID_Begin_Loop
CEID_DeltaYNeg      LDA        SE_SPRITE_PREV_Y0,X
                    SEC
                    SBC        SE_SPRITE_Y0,X
                    ORA        #$8000
                    STA        CEID_Delta_Y      
*--
CEID_Begin_Loop     LDA:       $0002,Y              ; Sprite Prev ID
                    CMP        CEID_SpritePrevID    ; Sprite Prev ID (Shadow Equivalence)
                    BNE        CEID_NotFound
                    LDA:       $0008,Y              ; Sprite ID
                    CMP        CEID_SpriteID        ; Sprite ID (Shadow Equivalence)
                    BNE        CEID_Next
                    LDA:       $0004,Y              ; Delta X
                    CMP        CEID_Delta_X
                    BNE        CEID_Next
                    LDA:       $0006,Y              ; Delta Y
                    CMP        CEID_Delta_Y
                    BEQ        CEID_Found
CEID_Next           TYA
                    CLC
                    ADC        #$000A
                    TAY
                    BRA        CEID_Begin_Loop
*--
CEID_Found          LDA:       $0000,Y              ; EraseID for this motion
                    RTS
CEID_NotFound       LDA        #$0000               ; No EraseID for this motion
*               brk        00
CEID_End            LDA        #$0000
                    RTS

*----------   ComputeEraseID Variables   ---------              
CEID_SpritePrevID   HEX        0000
CEID_Delta_X        HEX        0000
CEID_Delta_Y        HEX        0000
CEID_SpriteID       HEX        0000


*********************************************************************************************
*  EraseSpriteByIndex() :  Erase one Sprite from screen using its Index (IN:X=SpriteIndex). *
*********************************************************************************************
EraseSpriteByIndex  PHX
*--
                    LDA        SE_SPRITE_ERASE_ID,X ; Compute Erase Code Address
                    DEC
                    ASL
                    TAY
                    LDA        TAB_ERASE_BANK,Y     
                    STA        ESBI_Address+2
                    LDA        TAB_ERASE_ADDRESS,Y
                    STA        ESBI_Address+1              
*--
                    LDA        SE_SPRITE_PREV_ADDR,X ; Y = Sprite @Screen
                    TAY
                    SEC
                    SBC        #$35C0               ; A = Background @ (86 lines above Sprite @)
                    PEA        #$0101
                    PLB                             ; Data Bank = Bank 01
                    PLB
ESBI_Address        JSL        $AABBBB              ; Erase Sprite
                    PHK
                    PLB                             ; Data Bank = Current Bank
*--
                    PLX
                    RTS


************************************************************************************************
*  EraseSpriteByID() :  Erase one Sprite from screen using its ID (IN:A=EraseID,IN:Y=@Screen). *
************************************************************************************************
EraseSpriteByID     DEC                             ; Compute Erase Code Address
                    ASL
                    TAX
                    LDA        TAB_ERASE_BANK,X     
                    STA        ESBID_Address+2
                    LDA        TAB_ERASE_ADDRESS,X
                    STA        ESBID_Address+1              
*--
                    TYA                             ; Y = Sprite @Screen
                    SEC
                    SBC        #$35C0               ; A = Background @ (86 lines above Sprite @)
                    PEA        #$0101
                    PLB                             ; Data Bank = Bank 01
                    PLB
ESBID_Address       JSL        $AABBBB              ; Erase Sprite
                    PHK
                    PLB                             ; Data Bank = Current Bank
*-- 
                    RTS


*--------------------------------------------------------
EOS_Erase_Scr       LDA        #$31FF               ; #bytes - 1  (160*80)
                    LDX        #$2FA0               ; Src
                    LDY        #$6560               ; Dst
                    MVN        $01,$01
                    PHK
                    PLB
                    RTS


****************************************************************************************************
*  DrawSpriteByID() :  Draw one Sprite on the Screen using Sprite ID (IN:A=SpriteID,IN:Y=@Screen). *
****************************************************************************************************       
DrawSpriteByID      STA        DrawSpriteByID_1+1   ; SpriteID (1-N)
                    DEC                             ; Get Sprite Draw Function Address
                    ASL
                    TAX
                    LDA        TAB_SPRITE_BANK,X
                    ASL
                    ASL
                    TAX
                    LDA        TAB_SPRITE_ADDRESS,X
                    STA        DrawSpriteByID_2+1
                    LDA        TAB_SPRITE_ADDRESS+1,X
                    STA        DrawSpriteByID_2+2
DrawSpriteByID_1    LDA        #$AAAA               ; A = SpriteID
DrawSpriteByID_2    JSL        $AABBBB              ; Draw Sprite, Y=Screen @
                    RTS


************************************************************************************************
*  DrawSpriteByIndex() :  Draw one Sprite on the Screen using Sprite Index (IN:X=SpriteIndex). *
************************************************************************************************       
DrawSpriteByIndex   PHX
*--
                    LDA        SE_SPRITE_ID,X       ; Compute Sprite Code Bank
                    DEC
                    ASL
                    TAY
                    LDA        TAB_SPRITE_BANK,Y     
                    ASL
                    ASL
                    TAY
                    LDA        TAB_SPRITE_ADDRESS,Y
                    STA        DrawSpriteByIndex_1+1
                    LDA        TAB_SPRITE_ADDRESS+1,Y
                    STA        DrawSpriteByIndex_1+2              
*--
                    LDA        SE_SPRITE_ADDR,X     ; Y = @Screen
                    TAY
                    LDA        SE_SPRITE_ID,X       ; A = SpriteID
DrawSpriteByIndex_1 JSL        $AABBBB              ; Draw Sprite
*-- 
                    PLX
                    RTS


**************************************************************************************
*  AddSignedValues() :  Add two signed values (IN:A=Value#1,IN:X=Value#2,OUT:A=Sum). *
**************************************************************************************
AddSignedValues     STZ        ASV_Sign1            ; Init
                    STZ        ASV_Sign2
*-
                    ASL                             ; Value 1
                    BCC        ASV_1
                    INC        ASV_Sign1
ASV_1               LSR
                    STA        ASV_Value1
*-
                    TXA                             ; Value 2
                    ASL
                    BCC        ASV_2
                    INC        ASV_Sign2
ASV_2               LSR
                    STA        ASV_Value2
*--
                    LDA        ASV_Sign1            ; Value 1 + Value 2
                    CMP        ASV_Sign2
                    BNE        ASV_Diff
ASV_Add             LDA        ASV_Value1
                    CLC
                    ADC        ASV_Value2
                    STA        ASV_Sum
                    LDA        ASV_Sign1
                    BEQ        ASV_Exit             ; Positive
                    LDA        ASV_Sum
                    ORA        #$8000               ; Negative
                    BRA        ASV_End
*--                    
ASV_Diff            LDA        ASV_Value1
                    CMP        ASV_Value2
                    BGE        ASV_Diff_1
                    LDA        ASV_Value2           ; Value 2 - Value 1 
                    SEC
                    SBC        ASV_Value1
                    STA        ASV_Sum
                    LDA        ASV_Sign2
                    BEQ        ASV_Exit             ; Positive
                    LDA        ASV_Sum
                    ORA        #$8000               ; Negative
                    BRA        ASV_End
ASV_Diff_1          SEC                             ; Value 1 - Value 2 
                    SBC        ASV_Value2
                    STA        ASV_Sum
                    LDA        ASV_Sign1
                    BEQ        ASV_Exit             ; Positive
                    LDA        ASV_Sum
                    ORA        #$8000               ; Negative
                    BRA        ASV_End
*--
ASV_Exit            LDA        ASV_Sum
ASV_End             RTS                    
                    
*----------   AddSignedValues Variables   ---------
ASV_Value1          HEX        0000
ASV_Sign1           HEX        0000
ASV_Value2          HEX        0000
ASV_Sign2           HEX        0000
ASV_Sum             HEX        0000


********************************************************************
*  BuildAllCollisition() :  Build Collision table for all Sprites. *
********************************************************************
BuildAllCollisition LDAL       EraseCode3Bank       ; Use EraseCode3Bank for Collision Data Storage
                    STA        BOC_StoreLine_1+2
                    STA        BOC_StoreLine_2+2
                    STA        BOC_End_1+2
                    STA        BOC_End_2+2
                    STA        GCS_GetRecvAreaY_1+2
                    STA        GCS_GetRecvAreaY_2+2
                    STA        GCS_GetRecvAreaY_4+2
                    STA        GCS_SearchBegin_3+2
                    STA        GCS_CRLS_Loop+2
                    STA        GCS_CRLS_Process_1+2
                    STA        GCS_CRLS_Process_2+2
                    LDA        TabEraseCodeSize+4   ; Store data after the Erase data
                    CLC
                    ADC        #$0010
                    AND        #$FFF0
                    STA        BAC_ProcessSprite+1 
*--
                    LDY        #$0000
BAC_LoopSprite      STZ        BAC_TruncatureFlag+1 ; Truncature ON : Clear the lowest 5 lines of the Sprite
                    LDA        COLLISION_STATUS,Y   ; 01:Receive / 03:Receive+Send
                    AND        #$00FF
                    CMP        #$00FF               ; End of Table
                    BEQ        BAC_Duplicate
                    BIT        #$0080               ; Has Duplicate Flag ?
                    BNE        BAC_NextSprite
                    BIT        #$0001               ; Has Receive area ?
                    BEQ        BAC_NextSprite
                    BIT        #$0040               ; Has Truncature Flag ?
                    BEQ        BAC_ProcessSpriteT
                    LDA        #$8000               ; Truncature OFF : Do NOT clear the lowest 5 lines of the Sprite
                    STA        BAC_TruncatureFlag+1
*--
BAC_ProcessSpriteT  TYA                             ; Y = Sprite ID 
                    ASL
                    TAX
                    LDA        BAC_ProcessSprite+1
                    STA        SPRITE_RECEIVE_TAB,X ; @ Sprite Collision Data 
BAC_TruncatureFlag  LDA        #$0000
                    ASL                             ; Carry = Truncature Flag
                    TYA                             ; A = Sprite ID
BAC_ProcessSprite   LDX        #$AAAA               ; X = @Data
                    JSR        BuildOneCollision    ; Build Sprite Collision info
                    CLC                             ; Data Size
                    ADC        BAC_ProcessSprite+1
                    STA        BAC_ProcessSprite+1                  
*--                    
BAC_NextSprite      INY
                    BRA        BAC_LoopSprite
*--
BAC_Duplicate       LDX        #$007A               ; Duplicate 62 entries (62*2-2)
BAC_Duplicate_1     LDA        SPRITE_RECEIVE_TAB+2,X
                    STA        SPRITE_RECEIVE_TAB+130,X
                    DEX
                    DEX
                    BPL        BAC_Duplicate_1
*--
                    RTS


******************************************************************************************************************************
*  BuildOneCollision() :  Build Collision data for one Sprite (IN:A=SpriteID,IN:X=@Data,IN:Carry=Truncature,OUT:A=DataSize). *
******************************************************************************************************************************
BuildOneCollision   PHY                             ; Init
                    PHX
                    STA        BOC_SpriteID         
                    LDA        #$0005               ; Truncature of the latest sprite lines (erase Shadow)
                    BCC        BOC_Init
                    LDA        #$0000
BOC_Init            STA        BOC_Truncature+1
                    TXA
                    STA        BOC_End_1+1          ; @ Line Count (1 byte)
                    STA        BOC_End_2+1          ; @nLine Count (1 byte)
                    INC                             
                    STA        BOC_StoreLine_1+1    ; @ LineX,LineY
                    INC
                    INC
                    STA        BOC_StoreLine_2+1    ; @ Line Length
                    STZ        BOC_LineCount        ; Total Line Number
                    STZ        BOC_DataSize         ; Data Size
*--                    
                    LDA        BOC_SpriteID         ; Get Sprite size
                    DEC
                    ASL
                    TAX
                    LDA        TAB_SPRITE_SIZE,X
                    TAX
                    LDA:       $0000,X              ; Sprite Width (byte)
                    STA        BOC_SpriteWidth
                    LDA:       $0002,X              ; Sprite High (line)
                    STA        BOC_SpriteHeigh
*--
                    LDA        #$2000               ; Clear Sprite Area 
                    STA        BOC_ClearAreaAddr+1
                    LDY        BOC_SpriteHeigh
                    DEY
BOC_ClearArea       SEP        #$20
                    LDX        BOC_SpriteWidth
                    INX                             ; Erase 4 extra pixels
                    LDA        #$00
BOC_ClearAreaAddr   STAL       $012000,X
                    DEX
                    BPL        BOC_ClearAreaAddr
                    REP        #$30
                    LDA        BOC_ClearAreaAddr+1
                    CLC
                    ADC        #$00A0
                    STA        BOC_ClearAreaAddr+1
                    DEY
                    BPL        BOC_ClearArea
*--
                    LDA        BOC_SpriteID         ; Draw Sprite at $012000
                    LDY        #$2000               ; Y = @Screen
                    JSR        DrawSpriteByID       ; Draw Sprite / A = Sprite ID (1-N)
*--
                    LDA        BOC_SpriteHeigh      ; Ignore 5 last lines (Clear Shadow)
                    SEC
BOC_Truncature      SBC        #$0005
                    STA        BOC_SpriteHeigh
*--
                    LDA        #$2000               ; Init
                    STA        BOC_LoopColumn+1
                    STZ        BOC_HasLine
                    LDY        #$0000               ; Line Number
BOC_LoopLine        LDX        #$0000               ; Column Number
BOC_LoopColumn      LDAL       $012000,X
                    AND        #$00FF
                    BEQ        BOC_Pixel_00
                    BIT        #$000F
                    BEQ        BOC_Pixel_X0
                    BIT        #$00F0
                    BEQ        BOC_Pixel_0X
                    BRA        BOC_Pixel_XX
*--
BOC_Pixel_00        LDA        BOC_HasLine          ; End of previous Line ?
                    BEQ        BOC_NextColumn
                    JSR        BOC_StoreLine 
                    BRA        BOC_NextColumn           
*-
BOC_Pixel_X0        LDA        BOC_HasLine          ; End of previous Line ?
                    BNE        BOC_Pixel_X0_1
                    STY        BOC_BeginLineY       ; New line of 1 Pixel
                    TXA
                    ASL
                    STA        BOC_BeginLineX
                    STZ        BOC_LineLength
BOC_Pixel_X0_1      INC        BOC_LineLength
                    JSR        BOC_StoreLine
                    BRA        BOC_NextColumn
*-
BOC_Pixel_0X        LDA        BOC_HasLine          ; End of previous Line ?
                    BEQ        BOC_Pixel_0X_1
                    JSR        BOC_StoreLine        ; Store Current Line
BOC_Pixel_0X_1      INC        BOC_HasLine          ; Start New Line
                    STY        BOC_BeginLineY
                    TXA
                    ASL
                    INC
                    STA        BOC_BeginLineX
                    LDA        #$0001
                    STA        BOC_LineLength
                    BRA        BOC_NextColumn
*-
BOC_Pixel_XX        LDA        BOC_HasLine          ; Begining of a new line ?
                    BEQ        BOC_Pixel_XX_1
                    INC        BOC_LineLength       ; Length += 2
                    INC        BOC_LineLength
                    BRA        BOC_NextColumn
BOC_Pixel_XX_1      INC        BOC_HasLine          ; Start New Line
                    STY        BOC_BeginLineY
                    TXA
                    ASL
                    STA        BOC_BeginLineX
                    LDA        #$0002
                    STA        BOC_LineLength
*--                    
BOC_NextColumn      INX                             ; Next Column
                    CPX        BOC_SpriteWidth
                    BNE        BOC_LoopColumn
*--                    
BOC_NextLine        LDA        BOC_HasLine          ; End current Line ?
                    BEQ        BOC_NextLine_1
                    JSR        BOC_StoreLine
BOC_NextLine_1      LDA        BOC_LoopColumn+1     ; Next Line
                    CLC
                    ADC        #$00A0
                    STA        BOC_LoopColumn+1
                    INY
                    CPY        BOC_SpriteHeigh
                    BEQ        BOC_End
                    JMP        BOC_LoopLine
*--
BOC_End             PLX
                    PLY   
BOC_End_1           LDAL       $BB0000              ; Total Number of Line
                    AND        #$FF00
                    ORA        BOC_LineCount
BOC_End_2           STAL       $BB0000  
                    LDA        BOC_DataSize         ; Data Size
                    INC                             ; +1 for Line Number
                    RTS

*-- Internal Sub Function
BOC_StoreLine       LDA        BOC_BeginLineY       ; Store Current Line
                    XBA
                    ORA        BOC_BeginLineX       ; X,Y
BOC_StoreLine_1     STAL       $BBAAAA
                    LDA        BOC_LineLength       ; Length
BOC_StoreLine_2     STAL       $BBAAAA
                    LDA        BOC_StoreLine_1+1    ; Next 
                    CLC
                    ADC        #$0003
                    STA        BOC_StoreLine_1+1
                    ADC        #$0002
                    STA        BOC_StoreLine_2+1
                    INC        BOC_LineCount        ; One more line
                    STZ        BOC_HasLine          ; Reset
                    STZ        BOC_LineLength
                    INC        BOC_DataSize         ; Total Size += 3
                    INC        BOC_DataSize
                    INC        BOC_DataSize
                    RTS

*----------   BuildOneCollisition Variables   ---------
BOC_SpriteID        HEX        0000
BOC_SpriteWidth     HEX        0000
BOC_SpriteHeigh     HEX        0000
BOC_HasLine         HEX        0000                 ; 1 if a line has started
BOC_BeginLineX      HEX        0000
BOC_BeginLineY      HEX        0000
BOC_LineLength      HEX        0000                 ; in Pixel
BOC_LineCount       HEX        0000                 ; Total number of line for this Sprite
BOC_DataSize        HEX        0000                 ; Collision data size (byte)


*******************************************************************************
*  BuildCollisionTable() :  Build Sprites Collision Table (OUT:A=#Collision). *
*******************************************************************************
BuildCollisionTable STZ        BCT_Row_End+1        ; Init
                    LDA        SE_MAX_SPRITE_NB
                    ASL
                    STA        BCT_Row_Max+1
                    STA        BCT_Column_Max+1
*--
                    LDY        #$0000
BCT_Row             LDA        SE_SPRITE_OVERLAP,Y  ; Is Sprite Y Overlap by another Sprite ?
                    BEQ        BCT_Row_Next
                    LDX        SE_SPRITE_ID,Y       ; Has Receive Sprite Y a Receive Area ?
                    LDA        COLLISION_STATUS,X
                    AND        #$0001               ; Receive Flag
                    BEQ        BCT_Row_Next
*----                    
                    STY        BCT_Column_Y+1       ; Keep Current Y
                    LDX        #$0000               ; Loop over all Send Sprites
BCT_Column_Y        CPX        #$AAAA               ; Current Y
                    BEQ        BCT_Column_Next      ; Sprite X == Sprite Y => No Collision
                    LDA        SE_SPRITE_OVERLAP,X  ; Is Sprite X Overlap by another Sprite ?
                    BEQ        BCT_Column_Next
                    LDY        SE_SPRITE_ID,X       ; Has Send Sprite X a Send Area ?
                    LDA        COLLISION_STATUS,Y
                    AND        #$0002               ; Send Flag
                    BEQ        BCT_Column_Next
*----
BCT_Column          STX        BCT_Column_Next_X+1  ; X
                    LDY        BCT_Column_Y+1       ; Y  
                    JSR        GetCollisionScore    ; Check if Sprite X hits Sprite Y
                    BEQ        BCT_Column_Next_X    ; No Collision
*--
BCT_Store           PHX                             ; X = Impact Screen Address
                    PHY                             ; Y = Max Score
                    PHA                             ; A = Collision Score
*-
                    LDX        BCT_Column_Next_X+1  ; X
                    LDY        BCT_Column_Y+1       ; Y
                    LDA        SE_SPRITE_RECV,Y
                    CLC
                    ADC        BCT_Multx6,X
                    TAX                             ; Storage Address
                    PLA                             ; Collision Score
                    STA:       $0000,X
                    PLA                             ; Max Score
                    STA:       $0002,X
                    PLA                             ; Impact Screen Address
                    STA:       $0004,X
*-
*       JSL        IncBorderColor       ; Debug = Display Hit
                    INC        BCT_Row_End+1        ; Stat = Compute Total Collision #
*--
BCT_Column_Next_X   LDX        #$AAAA               ; Current X
BCT_Column_Next     INX                             ; Next Column
                    INX
BCT_Column_Max      CPX        #$AAAA
                    BNE        BCT_Column_Y
                    LDY        BCT_Column_Y+1       ; Get Current Y back
*----
BCT_Row_Next        INY                             ; Next Row
                    INY
BCT_Row_Max         CPY        #$AAAA
                    BNE        BCT_Row
*--
BCT_Row_End         LDA        #$0000               ; Number of Collision
                    RTS

*----------   BuildCollisionTable Variables   ---------                    
BCT_Multx6         HEX        0000,0600,0C00,1200,1800,1E00,2400,2A00
                   HEX        3000,3600,3C00,4200,4800,4E00,5400,5A00
                   HEX        6000,6600,6C00,7200,7800,7E00,8400,8A00
                   HEX        9000,9600,9C00,A200,A800,AE00,B400,BA00
                   

************************************************************
*  ClearCollisionTable() :  Clear Sprites Collision Table. *
************************************************************
ClearCollisionTable LDX        #SE_SPRITE_RECV_END-#SE_SPRITE_RECV0-2 ; Table Size - 2
CCT_Loop            STZ        SE_SPRITE_RECV0,X
                    DEX
                    DEX
                    BPL        CCT_Loop


*****************************************************************************************************************************************************************
*  GetCollisionScore()  Compute Collision Score between two Sprites (IN:X=SpriteSendIndex,IN:Y=SpriteReceiveIndex,OUT:A=Score,OUT:Y=MaxScore,OUT:X=ImpactAddr). *
*****************************************************************************************************************************************************************
GetCollisionScore   STZ        GCS_CollisionScore   ; Init
                    STZ        GCS_MaximumScore
                    STZ        GCS_CollisionScrAddr
                    STY        GCS_Equal+1          ; Quick Win : (X==Y) => No Collision
GCS_Equal           CPX        #$AAAA
                    BEQ        GCS_GotoEnd
                    LDA        SE_SPRITE_ID,X       ; Init
                    BEQ        GCS_GotoEnd          ; Not a Sprite
                    STA        GCS_SpriteSend_ID
                    LDA        SE_SPRITE_X0,X       ; Byte
                    ASL
                    STA        GCS_SpriteSend_X0    ; Pixel
                    LDA        SE_SPRITE_Y0,X
                    STA        GCS_SpriteSend_Y0                    
                    LDA        SE_SPRITE_ID,Y
                    BEQ        GCS_GotoEnd          ; Not a Sprite
                    STA        GCS_SpriteRecv_ID
                    LDA        SE_SPRITE_X0,Y       ; Byte
                    ASL
                    STA        GCS_SpriteRecv_X0    ; Pixel
                    LDA        SE_SPRITE_Y0,Y
                    STA        GCS_SpriteRecv_Y0
*--
GCS_SendArea        LDX        GCS_SpriteSend_ID    ; Has Send Sprite a Send Area ?
                    LDA        COLLISION_STATUS,X
                    AND        #$0002               ; Send Flag
                    BNE        GCS_ReceiveArea
GCS_GotoEnd         JMP        GCS_End
*--
GCS_ReceiveArea     LDX        GCS_SpriteRecv_ID    ; Has Receive Sprite a Receive Area ?
                    LDA        COLLISION_STATUS,X
                    AND        #$0001               ; Receive Flag
                    BEQ        GCS_GotoEnd
*--
GCS_GetAreaAddress  LDX        #$0000               ; Search Send Area Address into table
GCS_GetAreaAddr_1   LDA        COLLISTION_SEND_ID,X
                    BEQ        GCS_GotoEnd          ; End of Table
GCS_GetAreaAddr_2   CMP        GCS_SpriteSend_ID
                    BEQ        GCS_GetAreaAddr_3
                    INX
                    INX
                    BRA        GCS_GetAreaAddr_1
GCS_GetAreaAddr_3   LDA        COLLISTION_SEND_ADR,X
                    STA        GCS_SendAreaAddr     ; Beginning of Send data
*--
                    LDA        GCS_SpriteRecv_ID    ; Get Receive Area Address
                    ASL
                    TAX
                    LDA        SPRITE_RECEIVE_TAB,X
                    STA        GCS_RecvAreaAddr
*--                 
                    STZ        GCS_MaximumScore     ; Compute Maximum Score + Get Send Area limits (First / Last Y)   
                    LDX        GCS_SendAreaAddr
                    LDA:       $0000,X              ; Number of Lines
                    AND        #$00FF
                    STA        GCS_SendAreaNbLine
                    TAY
                    LDA:       $0002,X              ; First line Y
                    AND        #$00FF
                    STA        GCS_SendAreaFirstY
                    CLC
                    ADC        GCS_SpriteSend_Y0
                    STA        GCS_SendAreaY0
GCS_ComputeMaxScore INX
                    INX
                    INX
                    LDA:       $0000,X              ; Line Number of Pixels
                    AND        #$00FF
                    CLC
                    ADC        GCS_MaximumScore
                    STA        GCS_MaximumScore
                    DEY
                    BNE        GCS_ComputeMaxScore
                    DEX
                    LDA:       $0000,X              ; Last Line Y
                    AND        #$00FF  
                    STA        GCS_SendAreaLastY
                    CLC
                    ADC        GCS_SpriteSend_Y0
                    STA        GCS_SendAreaY1
                    INX
                    INX
                    STX        GCS_SendAreaAddrEnd  ; @ End of Send data
*--
                    LDX        GCS_RecvAreaAddr     ; Get Receive Area limits (First / Last Y)
GCS_GetRecvAreaY_1  LDAL       $BB0000,X            ; Number of Lines
                    AND        #$00FF
                    STA        GCS_RecvAreaNbLine
                    TAY
GCS_GetRecvAreaY_2  LDAL       $BB0002,X            ; First line Y
                    AND        #$00FF
                    STA        GCS_RecvAreaFirstY
                    CLC
                    ADC        GCS_SpriteRecv_Y0
                    STA        GCS_RecvAreaY0
                    TYA                             ; (Nb Line-1) * 3
                    DEC
                    STA        GCS_GetRecvAreaY_3+1
                    ASL
                    CLC
GCS_GetRecvAreaY_3  ADC        #$AAAA
                    CLC
                    ADC        GCS_RecvAreaAddr
                    TAX
GCS_GetRecvAreaY_4  LDAL       $BB0002,X            ; Last line Y
                    AND        #$00FF
                    STA        GCS_RecvAreaLastY                    
                    CLC
                    ADC        GCS_SpriteRecv_Y0
                    STA        GCS_RecvAreaY1
                    INX
                    INX
                    INX
                    INX
                    STX        GCS_RecvAreaAddrEnd  ; @ End of Recv data
*--
GCS_QuickWin        LDA        GCS_RecvAreaY1       ; Case #1 : Recv is above Send
                    CMP        GCS_SendAreaY0
                    BLT        GCS_QuickWinEnd
*--
                    LDA        GCS_SendAreaY1       ; Case #2 : Recv is below Send
                    CMP        GCS_RecvAreaY0
                    BLT        GCS_QuickWinEnd
                    BRA        GCS_Case3
GCS_QuickWinEnd     JMP        GCS_End
*--
GCS_Case3           LDA        GCS_SendAreaY0       ; Case #3 : Send is inside Recv
                    CMP        GCS_RecvAreaY0
                    BLT        GCS_Case4
                    LDA        GCS_RecvAreaY1
                    CMP        GCS_SendAreaY1
                    BLT        GCS_Case4
                    LDA        GCS_SendAreaY0       ; First Screen line where the comparison takes place
                    STA        GCS_CompareScrY0
                    LDA        GCS_SendAreaY1       ; Number of Screen line where comparison takes place
                    SEC
                    SBC        GCS_SendAreaY0
                    INC
                    STA        GCS_CompareScrLine
                    BRA        GCS_CaseEnd
*--     
GCS_Case4           LDA        GCS_RecvAreaY0       ; Case #4 : Recv is inside Send
                    CMP        GCS_SendAreaY0
                    BLT        GCS_Case5
                    LDA        GCS_SendAreaY1
                    CMP        GCS_RecvAreaY1
                    BLT        GCS_Case5
                    LDA        GCS_RecvAreaY0       ; First Screen line where the comparison takes place
                    STA        GCS_CompareScrY0                    
                    LDA        GCS_RecvAreaY1       ; Number of Screen line where comparison takes place
                    SEC
                    SBC        GCS_RecvAreaY0
                    INC
                    STA        GCS_CompareScrLine
                    BRA        GCS_CaseEnd                                
*--
GCS_Case5           LDA        GCS_RecvAreaY0       ; Case #5 : Lower Recv collide with Upper Send 
                    CMP        GCS_SendAreaY0
                    BGE        GCS_Case6
                    LDA        GCS_SendAreaY0       ; First Screen line where the comparison takes place
                    STA        GCS_CompareScrY0                           
                    LDA        GCS_RecvAreaY1       ; Number of Screen line where comparison takes place
                    SEC
                    SBC        GCS_SendAreaY0
                    INC
                    STA        GCS_CompareScrLine
                    BRA        GCS_CaseEnd                         
*--                                                 ; Case #6 : Lower Send collide with Upper Recv
GCS_Case6           LDA        GCS_RecvAreaY0       ; First Screen line where the comparison takes place
                    STA        GCS_CompareScrY0  
                    LDA        GCS_SendAreaY1       ; Number of Screen line where comparison takes place
                    SEC                             
                    SBC        GCS_RecvAreaY0
                    INC
                    STA        GCS_CompareScrLine
*--
GCS_CaseEnd         LDA        GCS_CompareScrY0     ; Last Screen line where the comparison takes place
                    CLC
                    ADC        GCS_CompareScrLine
                    DEC
                    STA        GCS_CompareScrY1
*-- 
GCS_SearchBegin     LDX        GCS_SendAreaAddr     ; Search Begin Address is Collision Data
                    INX
GCS_SearchBegin_1   LDA:       $0001,X              ; Line Y
                    AND        #$00FF
                    CLC
                    ADC        GCS_SpriteSend_Y0    ; + Sprite Send Y0 = First Screen Line for Data 
                    CMP        GCS_CompareScrY0     ; First Screen line where comparison takes place 0-199
                    BEQ        GCS_SearchBegin_2
                    INX
                    INX
                    INX
                    BRA        GCS_SearchBegin_1
GCS_SearchBegin_2   STX        GCS_SendAreaAddrY0   ; Collision Data Address for first Send comparison line
                    LDX        GCS_RecvAreaAddr
                    INX                    
GCS_SearchBegin_3   LDAL       $BB0001,X            ; Line Y
                    AND        #$00FF
                    CLC
                    ADC        GCS_SpriteRecv_Y0    ; + Sprite Recv Y0 = First Screen Line for Data 
                    CMP        GCS_CompareScrY0     ; First Screen line where comparison takes place 0-199
                    BEQ        GCS_SearchBegin_4
                    INX
                    INX
                    INX
                    BRA        GCS_SearchBegin_3
GCS_SearchBegin_4   STX        GCS_RecvAreaAddrY0   ; Collision Data Address for first Receive comparison line
*----
GCS_ComputeImpact   LDY        GCS_SendAreaAddrY0   ; Compte Impact Address X0|Y0
                    LDA:       $0000,Y              ; Line Data XXYY
                    AND        #$00FF               ; 00XX
                    CLC
                    ADC        GCS_SpriteSend_X0
                    LSR
                    XBA
                    ORA        GCS_CompareScrY0
                    STA        GCS_CollisionScrAddr
*---
GCS_ComputeInit     LDA        GCS_CompareScrY0     ; Current Compare Screen Address = First Compare Screen Address
                    STA        GCS_CurrentScrY
*--               
GCS_ComputeScrLine  LDY        GCS_SendAreaAddrY0   ; Loop over all Data Line matching this Screen Line
GCS_ComputeDataLine LDA:       $0000,Y              ; Line Data XXYY
                    XBA
                    AND        #$00FF               ; 00YY
                    CLC
                    ADC        GCS_SpriteSend_Y0    ; Current Screen Line
                    CMP        GCS_CurrentScrY
                    BLT        GCS_ComputeNextData  ; < : No yet = next Data Line
                    BNE        GCS_ComputeNextScr   ; > : Too far = next Screen Line
                    PHY
                    JSR        ComputeRcvLineScore  ; = : Compare this Data Send line segment with all Recv line segments               
                    PLY
GCS_ComputeNextData INY                             ; Next Data Line
                    INY
                    INY
                    CPY        GCS_SendAreaAddrEnd  ; Check end of Send Data lines
                    BLT        GCS_ComputeDataLine                    
*-
GCS_ComputeNextScr  INC        GCS_CurrentScrY      ; Compare Next Screen Line
                    LDA        GCS_CurrentScrY
                    CMP        GCS_CompareScrY1     ; Last Compare Screen Address
                    BLT        GCS_ComputeScrLine   ; <=
                    BEQ        GCS_ComputeScrLine
                    LDA        GCS_CollisionScore
                    BEQ        GCS_End
*----
GCS_End             LDY        GCS_MaximumScore     ; Maximal Collision Score  
                    LDX        GCS_CollisionScrAddr ; Collision Screen Address  X0|Y0
                    LDA        GCS_CollisionScore   ; Collision Score (Last returned)
                    RTS
                    
*-- Internal Sub Function #1                    
ComputeRcvLineScore LDX        GCS_RecvAreaAddrY0   ; Compute Collision Score for 1 Send Data Line 
GCS_CRLS_Loop       LDAL       $CC0000,X            ; Line Data XXYY
                    XBA
                    AND        #$00FF               ; 00YY
                    CLC
                    ADC        GCS_SpriteRecv_Y0    ; Recv Screen Line
                    CMP        GCS_CurrentScrY      ; Current Send Screen Line
                    BEQ        GCS_CRLS_Process
                    BGE        GCS_CRLS_End
GCS_CRLS_Next       INX                             ; Next Recv Data Line
                    INX
                    INX
                    CPX        GCS_RecvAreaAddrEnd  ; Check end of Recv Data
                    BLT        GCS_CRLS_Loop
GCS_CRLS_End        RTS
*--
GCS_CRLS_Process    LDA:       $0000,Y              ; Send Data X0
                    AND        #$00FF
                    CLC
                    ADC        GCS_SpriteSend_X0
                    STA        GCS_SendLineX0       ; Send Data Screen X0
                    LDA:       $0002,Y              ; Send Data Length
                    AND        #$00FF
                    DEC
                    CLC
                    ADC        GCS_SendLineX0
                    STA        GCS_SendLineX1       ; Send Data Screen X1
GCS_CRLS_Process_1  LDAL       $CC0000,X            ; Recv Data X0
                    AND        #$00FF
                    CLC
                    ADC        GCS_SpriteRecv_X0
                    STA        GCS_RecvLineX0       ; Receive Data Screen X0
GCS_CRLS_Process_2  LDAL       $CC0002,X            ; Recv Data Length
                    AND        #$00FF
                    DEC
                    CLC
                    ADC        GCS_RecvLineX0
                    STA        GCS_RecvLineX1       ; Receive Data Screen X1
*--
GCS_CRLS_Case1      LDA        GCS_SendLineX1       ; Case #1 : Send is left from Recv
                    CMP        GCS_RecvLineX0
                    BLT        GCS_CRLS_Next        ; Next Recv Data Line
*-                                             
GCS_CRLS_Case2      LDA        GCS_RecvLineX1       ; Case #2 : Send is right from Recv
                    CMP        GCS_SendLineX0
                    BLT        GCS_CRLS_Next        ; Next Recv Data Line
*-
GCS_CRLS_Case3      LDA        GCS_SendLineX0       ; Case #3 : Send is inside Recv
                    CMP        GCS_RecvLineX0
                    BLT        GCS_CRLS_Case4
                    LDA        GCS_RecvLineX1
                    CMP        GCS_SendLineX1
                    BLT        GCS_CRLS_Case4
                    LDA        GCS_SendLineX1       ; Score += Send length
                    SEC
                    SBC        GCS_SendLineX0
                    INC
                    CLC
                    ADC        GCS_CollisionScore
                    STA        GCS_CollisionScore
                    JMP        GCS_CRLS_Next        ; Next Recv Data Line
*-                    
GCS_CRLS_Case4      LDA        GCS_RecvLineX0       ; Case #4 : Recv is inside Send
                    CMP        GCS_SendLineX0
                    BLT        GCS_CRLS_Case5
                    LDA        GCS_SendLineX1
                    CMP        GCS_RecvLineX1
                    BLT        GCS_CRLS_Case5                    
                    LDA        GCS_RecvLineX1       ; Score += Recv length
                    SEC
                    SBC        GCS_RecvLineX0
                    INC
                    CLC
                    ADC        GCS_CollisionScore
                    STA        GCS_CollisionScore
                    JMP        GCS_CRLS_Next        ; Next Recv Data Line
*-
GCS_CRLS_Case5      LDA        GCS_SendLineX0       ; Case #5 : Send intersect right with Recv
                    CMP        GCS_RecvLineX0
                    BGE        GCS_CRLS_Case6
                    LDA        GCS_SendLineX1       ; Score += intersect length
                    SEC
                    SBC        GCS_RecvLineX0
                    INC
                    CLC
                    ADC        GCS_CollisionScore
                    STA        GCS_CollisionScore
                    JMP        GCS_CRLS_Next        ; Next Recv Data Line
*-
GCS_CRLS_Case6      LDA        GCS_RecvLineX1       ; Case #6 : Send intersect left with Recv 
                    SEC                             ; Score += intersect length
                    SBC        GCS_SendLineX0
                    INC
                    CLC
                    ADC        GCS_CollisionScore
                    STA        GCS_CollisionScore
                    JMP        GCS_CRLS_Next        ; Next Recv Data Line             
                    
*----------   GetCollisionScore Variables   ---------                    
GCS_SpriteSend_ID   HEX        0000                 ; Sprite Send ID   
GCS_SpriteSend_X0   HEX        0000                 ; 0-319 (Pixel)
GCS_SpriteSend_Y0   HEX        0000                 ; 0-199 (Line)
GCS_SendAreaAddr    HEX        0000                 ; Address of Send Area data
GCS_SendAreaAddrEnd HEX        0000                 ; End of Send Area data + 1
GCS_SendAreaNbLine  HEX        0000                 ; Number of Data Line in Send Area
GCS_SendAreaFirstY  HEX        0000                 ; Send Area First Y (relative to Sprite)
GCS_SendAreaLastY   HEX        0000                 ; Send Area Last Y (relative to Sprite)
GCS_SendAreaY0      HEX        0000                 ; Send Area Y0 = First Line (In Picture)
GCS_SendAreaY1      HEX        0000                 ; Send Area Y1 = Last Line (In Picture)
*--
GCS_SpriteRecv_ID   HEX        0000                 ; Sprite Receive ID   
GCS_SpriteRecv_X0   HEX        0000                 ; 0-319 (Pixel)
GCS_SpriteRecv_Y0   HEX        0000                 ; 0-199 (Line)
GCS_RecvAreaAddr    HEX        0000                 ; Address of Receive Area data
GCS_RecvAreaAddrEnd HEX        0000                 ; End of Recv Area data +1
GCS_RecvAreaNbLine  HEX        0000                 ; Number of Data Line in Receive Area
GCS_RecvAreaFirstY  HEX        0000                 ; Receive Area First Y (relative to Sprite)
GCS_RecvAreaLastY   HEX        0000                 ; Receive Area Last Y (relative to Sprite)
GCS_RecvAreaY0      HEX        0000                 ; Receive Area Y0 = First Line (In Picture)
GCS_RecvAreaY1      HEX        0000                 ; Receive Area Y1 = Last Line (In Picture)
*--
GCS_CompareScrY0    HEX        0000                 ; First Screen line where comparison takes place 0-199
GCS_CompareScrY1    HEX        0000                 ; Last Screen line where comparison takes place 0-199
GCS_CompareScrLine  HEX        0000                 ; Number of Screen line where comparison takes place
GCS_SendAreaAddrY0  HEX        0000                 ; Collision Data Address for first Send comparison line
GCS_RecvAreaAddrY0  HEX        0000                 ; Collision Data Address for first Receive comparison line
*--
GCS_CurrentScrY     HEX        0000                 ; Current Screen Line [GCS_CompareScrY0,GCS_CompareScrY0+GCS_CompareScrLine-1]
GCS_SendLineX0      HEX        0000                 ; Send Data Line X0 0-319
GCS_SendLineX1      HEX        0000                 ; Send Data Line X1 0-319
GCS_RecvLineX0      HEX        0000                 ; Receive Data Line X0 0-319
GCS_RecvLineX1      HEX        0000                 ; Receive Data Line X1 0-319
*--
GCS_CollisionScore  HEX        0000                 ; Number of Collision Pixels
GCS_MaximumScore    HEX        0000                 ; Number of SendArea Pixels    
GCS_CollisionScrAddr HEX       0000                 ; Collision Screen Address


**********************************************************************************************
*  GetAvailableOsc() :  Search for a free oscillator couple to play the Sound (OUT:X=Osc*2). *
**********************************************************************************************
GetAvailableOsc    LDX         #$0000               ; First Search : Search for a Free Oscillator
GAO_Loop_1         LDA         SE_SOUND_OSCILLATOR,X
                   BEQ         GAO_END              ; 0000 = Free
                   INX
                   INX
                   CPX         #$001C
                   BNE         GAO_Loop_1       
*--
                   LDX         #$0000               ; Second Search : Search for a nearly-Free Oscillator
GAO_Loop_2         LDA         SE_SOUND_OSCILLATOR,X
                   CMP         #$FFFF               ; FFFF = No available for sounds
                   BEQ         GAO_Next
                   CMP         #$0004               ; The sound is ending soon
                   BLT         GAO_END
GAO_Next           INX
                   INX
                   CPX         #$001C
                   BNE         GAO_Loop_2       
*--
                   LDX         #$FFFF               ; No Oscillator available
*--
GAO_END            RTS                   


*****************************************************************
*  UpdateOscStatusVBL() :  Update Oscillators Status every VBL. *
*****************************************************************
UpdateOscStatusVBL  LDX        #$0000               ; Decrement Sound remaining length
UOS_Loop            LDA        SE_SOUND_OSCILLATOR,X
                    BEQ        UOS_Next
                    CMP        #$FFFF
                    BEQ        UOS_Next
                    DEC
                    STA        SE_SOUND_OSCILLATOR,X
UOS_Next            INX
                    INX
                    CPX        #$0020
                    BNE        UOS_Loop
*--
UOS_End             RTS


***********************************************************************************
*  AppendToSoundList() :  Append a Sound ID to Sound Pending List (IN:A=SoundID). *
***********************************************************************************
AppendToSoundList   PHX
                    TAY                             ; Sound ID
                    LDA        SE_SOUND_PENDING_NB
                    CMP        #$000E               ; No more than 14 sounds 
                    BEQ        ATSL_End
                    ASL                             ; Insert this new Sound ID at the end of the Table
                    TAX
                    TYA
                    STA        SE_SOUND_PENDING_ID,X
                    INC        SE_SOUND_PENDING_NB
*--                    
ATSL_End            PLX
                    RTS
                     
                    
*******************************************************************
*  RandomInitialize() :  Initialize Random Generator for Sprites. *
******************************************************************* 
RandomInitialize    LDA        SE_MAX_SPRITE_NB
                    DEC
                    ASL
                    TAX
                    LDAL       HORIZCNT             ; Use beam Horizontal position to initialize Random
                    AND        #$003F               ; [00-3F]
RandomInitialize_1  STA        SE_SPRITE_RANDOM,X
                    INC
                    CMP        #$0063
                    BLT        RandomInitialize_2
                    SEC
                    SBC        #$063 
RandomInitialize_2  DEX
                    DEX
                    BPL        RandomInitialize_1
                    RTS
                                         
*****************************************************************************************
