******************************************************************************************
*                                                                                        *
*  AppleIIgsKarate.Screen.s :  Library for Screens handling functions.                   *
*                                                                                        *
******************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  April 2022  *
******************************************************************************************

***********************************************************************************************
*  BootScreen() :  Display SHR + Display Logo + Text + Sound + Load next files (OUT:A=Error). *
***********************************************************************************************
BootScreen          SEP        #$20
                    LDA        #$00                 ; Shadowing ON
                    STAL       SHADOW
                    REP        #$30
*--
                    LDX        #$1FFE               ; RAZ Graphic Page
                    LDA        #$000
BS_RAZ_GP           STAL       $012000,X
                    STAL       $014000,X
                    STAL       $016000,X
                    STAL       $018000,X
                    DEX
                    DEX
                    BPL        BS_RAZ_GP
*--
                    SEP        #$20                          
                    LDA        #$C1                 ; Display SHR + Linarize Graphic Page
                    STAL       NEWVIDEO
                    REP        #$30
                    LDA        #$0000               ; Black Border
                    JSL        SetBorderColor
*--
BS_Logo             LDA        #$0F3C               ; Copy Bitmap Logo Width4|Heigh
                    LDX        #BOOT_LOGO           ; @Src
                    LDY        #$4A20               ; @Dst     
                    JSL        CopyOneBitmap
*--
BS_Text             LDX        #BS_BDS              ; Copy Text
                    LDY        #$7662               ; @Dst
                    JSL        DisplayString
*--
BS_SCB              LDX        #$00FE               ; Install SCB
                    LDA        #$0000
BS_SCB_1            STAL       $019D00,X
                    DEX
                    DEX
                    BPL        BS_SCB_1
*--
BS_PALETTE          LDX        #$001E               ; Install Palettes
BS_PALETTE_1        LDA        BOOT_PALETTE,X
                    STA        Buffer_Palette,X
                    DEX
                    DEX
                    BPL        BS_PALETTE_1
*--
BS_Sound_Install    LDX        #BOOT_SOUND          ; Copy Sound Data to DOC Ram
                    LDY        #$3FF6               ; Sound Size = 16 KB - 6
                    LDA        CurrentBank          ; XX00
                    ORA        #$0080               ; Copy Sound To $8000
                    CLC                             ; Stop Sound + Enable Interrupt on Exit
                    JSL        CopySoundDataToRAM
*--
BS_Sound_Play       LDX        #$8000               ; Play Sound located at $8000
                    LDY        #$0004               ; Oscillators 4 & 5
                    LDA        #$36D6               ; Size ($36=16KB)|Freq($D6=11Khz)
                    JSL        PlayOneShotSound
*--
BS_FadeIn           LDA        #Buffer_Palette      ; Fade In
                    JSL        FadeIN
*--
                    JSR        ResetTimerSec        ; Start Timer
*--
BS_LoadFile_Music1  LDX        #BS_FileData002      ; IntroMusic1.bin
                    LDA        Data1Bank
                    JSL        LoadFile
                    BCC        BS_LoadFile_Music2
                    LDA        #BS_FileData002      ; Error
                    BRA        BS_End
*--                    
BS_LoadFile_Music2  LDX        #BS_FileData003      ; IntroMusic2.bin
                    LDA        Data2Bank
                    JSL        LoadFile
                    BCC        BS_LoadFile_Picture
                    LDA        #BS_FileData003      ; Error
                    BRA        BS_End                    
*--
BS_LoadFile_Picture LDX        #BS_FileData001      ; IntroBackg.bin
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    STA        BS_Uncompress_Size+1
                    BCC        BS_Uncompress
                    LDA        #BS_FileData001      ; Error
                    BRA        BS_End
*--
BS_Uncompress       SEP        #$20                 ; LZ4 Uncompress
                    LDA        LoadUnpackBank+1
                    XBA
                    LDA        LoadUnpackBank+1     ; Src Bank | Dst Bank
                    REP        #$30
                    LDX        #$0080               ; Src Offset | Dst Offset ($80=$8000)
BS_Uncompress_Size  LDY        #$0000               ; PackDataSize
                    JSL        UnpackLZ4
                    CMP        #$8000               ; => Return in A the length of unpacked Data
*--
                    JSR        LoadHighScoreTable   ; Load High Score Table
*--
BS_LoadFile_Erase   LDX        #BS_FileData004      ; EraseCode1.bin
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    STA        BS_UncompressE_Size+1
                    BCC        BS_UncompressErase
                    LDA        #BS_FileData004      ; Error
                    BRA        BS_End
BS_UncompressErase  SEP        #$20                 ; LZ4 Uncompress
                    LDA        LoadUnpackBank+1
                    XBA
                    LDA        EraseCode1Bank+1     ; Src Bank | Dst Bank
                    REP        #$30
                    LDX        #$0000               ; Src Offset | Dst Offset ($80=$8000)
BS_UncompressE_Size LDY        #$0000               ; PackDataSize
                    JSL        UnpackLZ4
*--
BS_Credit1_3Sec     JSR        GetTimerSec          ; We need to wait at least 3 sec
                    CMP        #$0003
                    BMI        BS_Credit1_3Sec
*--
                    LDA        #$0000               ; No Error
BS_End              RTS
                    
*----------   BootScreen Variables   ---------                    
BS_BDS              ASC        'BRUTAL DELUXE SOFTWARE',00
*--
BS_FileData001      STRL       '1/Data/IntroBackg.bin'  ; Intro Background Picture LZ4
BS_FileData002      STRL       '1/Data/IntroMusic1.bin' ; Intro Music 1/3
BS_FileData003      STRL       '1/Data/IntroMusic2.bin' ; Intro Music 2/3
BS_FileData004      STRL       '1/Data/EraseCode1.bin'  ; Erase Code 1/3


******************************************************************************************
*  CreditScreen() :  Display Credit Screen + Play Music + Load next files (OUT:A=Error). *
******************************************************************************************
CreditScreen        JSL        ClearKDBStrobe       ; Clear Keyboard
                    JSL        FadeOut              ; Fadout to Black
                    LDA        #$0000
                    JSL        SetBorderColor                    
*------
                    LDA        LoadUnpackBank       ; Copy Pixel + SCB
                    ORA        #$0080
                    STA        CS_CopyPicture+2
                    LDX        #$7DFE               
CS_CopyPicture      LDAL       $AA8000,X
                    STAL       $012000,X
                    DEX
                    DEX
                    BPL        CS_CopyPicture
*------
                    LDA        LoadUnpackBank       ; Copy Palettes
                    ORA        #$00FE
                    STA        CS_CopyPalette+2
                    LDX        #$01FE               
CS_CopyPalette      LDAL       $AAFE00,X
                    STA        Buffer_Palette,X
                    DEX
                    DEX
                    BPL        CS_CopyPalette
*------                                          
CS_FadeIn           LDA        #Buffer_Palette      ; Fade In
                    JSL        FadeIN
*-----
CS_Sound_Install    LDX        #$0000               ; Copy Sound 16KB Data #0 to DOC Ram
                    LDY        #$3FF6               ; Sound Size = 16 KB - 6
                    LDA        Data1Bank            ; XX00
                    ORA        #$0000               ; Copy Sound To $0000
                    CLC                             ; Stop Sound + Enable Interrupt on Exit
                    JSL        CopySoundDataToRAM                
                    LDX        #$4000               ; Copy Sound 16KB Data #1 to DOC Ram
                    LDY        #$3FF6               ; Sound Size = 16 KB - 6
                    LDA        Data1Bank            ; XX00
                    ORA        #$0040               ; Copy Sound To $4000
                    CLC                             ; Stop Sound + Enable Interrupt on Exit
                    JSL        CopySoundDataToRAM
*------
CS_Sound_Play       LDX        #CreditScreenMusic   ; Install local Sound Interrupt Function
                    LDY        #>CreditScreenMusic
                    LDA        #$4036               ; Sound Address ($4000) | Sound Size (16 KB)
                    JSL        PlayLoopSound        ; Play Sound located at $0000
*------
CS_LoadFile_Music3  LDX        #CS_FileData001      ; IntroMusic3.bin
                    LDA        Data3Bank
                    JSL        LoadFile
                    BCC        CS_LoadFile_Credit
                    LDA        #CS_FileData001      ; Error
                    JMP        CS_End
*------
CS_LoadFile_Credit  LDX        #CS_FileData002      ; IntroCredit.bin
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    STA        CS_Uncompress_Size+1
                    BCC        CS_Uncompress
                    LDA        #CS_FileData002      ; Error
                    JMP        CS_End 
*--
CS_Uncompress       SEP        #$20                 ; LZ4 Uncompress
                    LDA        LoadUnpackBank+1
                    XBA
                    LDA        LoadUnpackBank+1     ; Src Bank | Dst Bank
                    REP        #$30
                    LDX        #$0080               ; Src Offset | Dst Offset ($80=$8000)
CS_Uncompress_Size  LDY        #$0000               ; PackDataSize
                    JSL        UnpackLZ4
                    CMP        #$8000               ; => Return in A the length of unpacked Data
*------
CS_BackupLogo       LDA        CS_BackupLogo_3+2    ; Backup Paradox Logo
                    AND        #$00FF
                    ORA        LoadUnpackBank       ; XX00
                    STA        CS_BackupLogo_3+2
                    LDY        #$0000               ; 39(4)*76
CS_BackupLogo_1     LDX        #$004C               ; 39x4
CS_BackupLogo_2     LDAL       $016BCA,X            ; 084,121=6BCA
CS_BackupLogo_3     STAL       $AA8141,X            ; 002,002=0141
                    DEX
                    DEX
                    BPL        CS_BackupLogo_2
                    LDA        CS_BackupLogo_2+1
                    CLC
                    ADC        #$00A0
                    STA        CS_BackupLogo_2+1
                    LDA        CS_BackupLogo_3+1
                    CLC
                    ADC        #$00A0
                    STA        CS_BackupLogo_3+1
                    INY
                    CPY        #$004C               ; 76 lines
                    BNE        CS_BackupLogo_1
*------
CS_Fade             LDA        ESV_Counter          ; Music counter : updated every 1.48 sec
                    CMP        #$0025
                    BGE        CS_Exit
                    STA        CS_Fade_2+1
*--
                    LDX        #$000E               ; Search for an action to perform
CS_Fade_1           LDA        CS_TabCounter,X
CS_Fade_2           CMP        #$AAAA 
                    BEQ        CS_Fade_Action
                    DEX
                    DEX
                    BPL        CS_Fade_1
                    BRA        CS_Fade_Keyboard
*-- 
CS_Fade_Action      LDA        CS_TabScreen,X       ; Display Company / Program / Graphic / Sound
                    TAX
                    CPX        CS_PreviousScreen    ; Check we don't run twice the same Screen
                    BEQ        CS_Fade_Keyboard  
                    STX        CS_PreviousScreen
                    JSR        CreditScreenFade     
*--
CS_Fade_Keyboard    JSR        IsKeyPressed         ; Exit on Key Pressed
                    BEQ        CS_Fade
*------
BS_LoadFile_Erase2  LDX        #CS_FileData003      ; EraseCode2.bin
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    STA        BS_UncompressE2_Sz+1
                    BCC        BS_UncompressErase2
                    LDA        #CS_FileData003      ; Error
                    BRA        CS_End
BS_UncompressErase2 SEP        #$20                 ; LZ4 Uncompress
                    LDA        LoadUnpackBank+1
                    XBA
                    LDA        EraseCode2Bank+1     ; Src Bank | Dst Bank
                    REP        #$30
                    LDX        #$0000               ; Src Offset | Dst Offset ($80=$8000)
BS_UncompressE2_Sz  LDY        #$0000               ; PackDataSize
                    JSL        UnpackLZ4
*------
CS_Exit             JSL        FadeOutSound         ; Decrease Sound Volume
                    JSL        StopSound            ; Stop Sound
*--
                    LDA        #$0000               ; No Error
CS_End              RTS
                    
*----------   CreditScreen Variables   ---------                    
CS_FileData001      STRL       '1/Data/IntroMusic3.bin'  ; Intro Music 3/3
CS_FileData002      STRL       '1/Data/IntroCredit.bin'  ; Intro Credit Picture LZ4
CS_FileData003      STRL       '1/Data/EraseCode2.bin'   ; Erase Code 2/3 LZ4
CS_PreviousScreen   HEX        0000                                      ; Keep latest Screen ID
CS_TabCounter       HEX        0400,0800,0C00,1000,1400,1800,1C00,2000   ; Timing
CS_TabScreen        HEX        0100,0200,0300,0000,0100,0200,0300,0000   ; Screen ID (Company / Program / Graphic / Sound)


***************************************************************
*  CreditScreenMusic() :  Feed DOC Ram with 16 KB Sound Data. *
***************************************************************
CreditScreenMusic   PHB
                    PHK
                    PLB
                    LDY        CSM_Offset           ; Copy Sound 16KB Data to DOC Ram
                    INY
                    INY
                    CPY        #$0016
                    BNE        CreditScreenMusic_1
                    LDY        #$0000
CreditScreenMusic_1 STY        CSM_Offset
*--
                    LDA        CSM_Tab_SoundDst     ; Flip/Flop CSM_Tab_SoundDst
                    CLC
                    ADC        #$0040
                    CMP        #$0040
                    BEQ        CreditScreenMusic_2
                    LDA        #$0000
CreditScreenMusic_2 STA        CSM_Tab_SoundDst
*--
                    LDA        CSM_Tab_SoundSrc,Y   ; X=Sound Src Address
                    TAX 
                    LDA        CSM_Tab_SoundBank,Y  ; Sound Src Bank
                    TAY
                    LDA        Data1Bank,Y          ; XX00
                    ORA        CSM_Tab_SoundDst     ; Sound DOC Address : $0000 or $4000
                    LDY        #$3FF6               ; Sound Size = 16 KB - 6
                    SEC                             ; Do not modify Interrupt bit
                    JSL        CopySoundDataToRAM
*--
                    PLB
                    RTL

*----------   CreditScreenMusic Variables   ---------                    
CSM_Offset          HEX        0200                 ; 16 KB * 11
CSM_Tab_SoundBank   HEX        0000,0000,0000,0000,0200,0200,0200,0200,0400,0400,0400
CSM_Tab_SoundSrc    HEX        0000,0040,0080,00C0,0000,0040,0080,00C0,0000,0040,0080
CSM_Tab_SoundDst    HEX        4000                 ; 0000 or 0040


*********************************************************************
*  CreditScreenFade() :  Display Credit Screen Text (IN:X=Screen#). *
*********************************************************************
CreditScreenFade    TXA                             ; Bitmap Src Address
                    ASL
                    TAX
                    LDA        CSF_TabAddrSrc,X
                    STA        CSF_Copy_Bitmap_2+1
                    LDA        CSF_Copy_Bitmap_2+2
                    AND        #$00FF
                    ORA        LoadUnpackBank       ; XX00
                    STA        CSF_Copy_Bitmap_2+2
*--
                    LDA        CGSS_SpeedCode       ; 0-3
                    ASL
                    TAX
                    LDA        CSF_TabTiming,X
                    STA        CSF_Copy_Bitmap_3+1  ; Waiting duration between each draw
*--
CSF_Sound_Play      LDX        #$8000               ; Play Sound located at $8000
                    LDY        #$0004               ; Oscillators 4 & 5
                    LDA        #$36D6               ; Size ($36=16KB)|Freq($D6=11Khz)
                    JSL        PlayOneShotSound                    
*--
                    LDY        #$0000               ; Random Transition
CSF_Copy_Bitmap_1   LDA        CSF_TabAddrDst,Y
                    AND        #$7FFF 
                    TAX
CSF_Copy_Bitmap_2   LDAL       $AABBBB,X            ; Copy Bytes
                    STAL       $016BCA,X
*--
CSF_Copy_Bitmap_3   LDX        #$AAAA               ; Pause
CSF_Copy_Bitmap_4   DEX
                    BNE        CSF_Copy_Bitmap_4
*--                    
                    INY
                    INY
                    CPY        #$1728               ; 39*76 4 pixels 
                    BNE        CSF_Copy_Bitmap_1     
*--
                    RTS

*----------   CreditScreenFade Variables   ---------                    
CSF_TabTiming       HEX        4000,D000,0003,0040
CSF_TabAddrSrc      HEX        4181,A1B2,9181,F1B2
CSF_TabAddrDst      HEX        8C84,609D,CAA5,BC9B,74A2,7281,A891,0A9E,CA87,6CA6,B480,2891,00AA,BC8F,E08F,FAA4
                    HEX        3085,A8A8,9E96,6C86,C6AD,2699,86A2,A89D,B48A,8A90,0A8A,6A8E,A4AA,46AE,42A3,8287
                    HEX        BC82,C691,D09C,9E8C,8498,8E93,C291,948E,1880,2AAF,8891,EEAE,7290,8095,5CA6,68A6
                    HEX        C482,C88C,0098,0682,0696,8EA2,DE8A,F69C,DC97,2CA5,1EA5,14A5,8C87,E6A8,3488,4095
                    HEX        4280,EE95,1AA5,72A9,60A7,92AC,06A3,1682,C480,8C9A,24AB,709F,6AAB,E89C,AC99,9E89
                    HEX        E690,2E8A,80AC,20A6,B29B,409A,CE88,46A9,BEAD,6C88,AA91,849B,CC9B,AE8F,C0A3,6A97
                    HEX        C2A5,6E93,5C88,EA97,2492,249E,66AC,7884,EA80,4E81,6CAB,069B,009E,B882,FE8D,CE85
                    HEX        00A8,4281,7086,04A5,549F,9484,62AB,34A1,889A,1AA0,96AA,F497,D697,3E94,EC90,0AAA
                    HEX        8C91,0A85,A489,54A6,ECA4,BAA8,9C82,F097,26AF,74A9,42AE,D494,6A83,609A,6E9D,7C89
                    HEX        5E90,FC97,A8A5,C899,A49B,1C8F,288A,2094,7E9F,D0A1,7C84,1082,1CA5,628E,CC87,3A94
                    HEX        448B,A682,5895,8681,9CA7,1AAA,60AE,9AA5,82AE,A889,468D,7E89,1E96,D099,9EA0,EA8D
                    HEX        5E8B,DEA3,EC86,448A,0E91,448F,C687,4E92,3A97,DA8D,EEA9,0887,C48C,409C,4885,CA9B
                    HEX        DEA6,0CAA,CAA0,9AA0,1099,C2AD,3094,E28D,1C85,1899,86A7,2E8F,4EA9,4485,D285,BC8A
                    HEX        769D,8490,DEAB,268C,CE92,6893,6097,A4A0,9A84,A482,6AA1,A089,2883,F6A9,7E9D,96A7
                    HEX        34A3,BE91,8290,4088,14AA,929D,528B,FA83,78A9,8896,4A9A,7E8B,EAAD,D48F,C68F,AC87
                    HEX        04AA,82A4,C299,068C,AA93,4C88,7889,C488,F4A1,208C,2880,A48A,7C81,1AAD,8C82,A69D
                    HEX        7084,10A0,8281,B0A5,5081,AC84,5090,DEAD,02A5,6C90,748B,4A99,AEA5,5292,AAA8,A096
                    HEX        A8A2,F881,7690,2C94,B696,5890,B09B,289C,089B,5EAE,BAA0,E481,CA8F,449C,BE80,B08C
                    HEX        EC97,EA90,E4AD,8084,88A5,1699,6288,F4A9,DC80,8E82,C0A8,9C84,4AAD,06AF,1687,6297
                    HEX        C499,E0AD,68A7,5C95,CEA8,C699,9E91,B6A3,608E,2882,E48B,1E85,44AB,789D,22A0,44A9
                    HEX        5E97,7AAC,EAAB,EA9A,928C,C8AD,28AA,A085,04A0,288C,1694,2C8D,72A7,F8A1,A29E,9091
                    HEX        7E8E,1887,3088,3C8F,D4A6,8887,F683,A493,86AE,1685,B880,F4AB,D283,D08D,208A,CA9E
                    HEX        4C99,CAAD,049E,40A9,D6AB,08A3,6C95,1E94,8CA0,E2AB,44A1,A082,A899,5A90,E895,6CAC
                    HEX        0AAC,DE92,7EA4,8A8B,C09B,CE8D,8C90,1CAF,4099,D8A1,8A82,E49E,828B,4E86,26AB,DC92
                    HEX        9687,1E8A,2692,3AA1,328D,2294,32A6,BCAA,1482,76AE,4692,3A80,CA92,9E8E,8491,F681
                    HEX        0C96,0493,369C,6E8B,C8A1,CAAA,0A8C,38A8,9693,B4A0,E8A8,B08F,0C93,D897,A6A8,C897
                    HEX        669F,0091,0894,7893,5E8D,9EAC,188F,E286,A098,3680,E880,48A1,129B,5EA9,5C9A,ACAA
                    HEX        A68C,0AA7,C491,4A8F,A8AC,688B,1091,569C,C6A8,7A84,A49E,D492,8884,C492,0A98,16AD
                    HEX        0E94,AEA0,9089,769A,0485,008E,4EA1,248C,4086,669C,869F,849F,06A2,7C9F,4AA4,AEAA
                    HEX        4894,C8A5,A0A8,4680,F692,AAAC,E697,5A92,C692,0CA2,3CA8,2AA1,64A7,82A7,FAAB,9093
                    HEX        B68C,A4A3,4C80,EC81,66A9,E099,AA82,BE94,3C97,0C9D,1AA3,009D,B8A8,D483,329E,069D
                    HEX        9E84,908E,AA8E,5C8B,02A8,A087,B28A,F092,A89B,8AAA,70AC,FCAB,6A86,58A6,A2A7,0E8A
                    HEX        988C,409E,08A7,CEA6,F09F,128A,3C83,F28B,C89E,B499,C094,D28D,54A4,5086,949B,4886
                    HEX        18A0,A893,909B,8E96,18A5,2494,96AC,E2AD,F492,529A,EC94,7C9D,48AB,609F,6086,A289
                    HEX        00A7,CA85,2687,D69C,D497,248F,3CAD,8A9D,80A9,FA86,D0AD,8A81,0A84,0299,0498,0085
                    HEX        BAAD,B299,C8AB,0AAF,4E8B,DA9E,508B,709A,E4A8,BE8F,3285,E683,1891,26A1,E4A9,E8A4
                    HEX        E48D,C49E,ECA1,009B,2E83,5EA4,5497,6883,B48F,7681,7486,028F,8C89,6AAC,3E88,58AE
                    HEX        D092,EC9F,7093,DC88,4695,9684,58A1,AA85,7E98,B0A3,888B,6C98,3697,8889,50A4,34AB
                    HEX        BC80,E6A9,E69E,9E9B,4288,AE94,C4A5,8489,4888,04A7,C89B,D097,98A0,E881,EA9C,5A8D
                    HEX        CC9C,CA9C,D699,BA99,B4A3,E09E,A694,EE83,8081,4E97,BC96,8289,2691,2083,5495,FAA9
                    HEX        A2A5,DC9C,849A,E097,36AB,0E9E,2885,F695,36A6,EA92,52A1,CC99,C48D,0291,1691,3497
                    HEX        C09C,8A9F,A4AC,BC9E,02A2,78A2,788E,9CAA,A2A0,EA81,1CAA,E0A3,BE82,0284,8093,648E
                    HEX        78A4,E4A6,E68D,889F,2299,A287,92A7,4492,1A8A,A880,0087,608D,B69E,E488,F6AE,62A9
                    HEX        BC8C,0C99,A0AD,268A,FE8B,7C8E,9884,9084,44A4,AC98,589C,4EAB,6281,4697,0089,5881
                    HEX        48A6,AE82,2C96,8EAA,DCA6,DE94,3C9E,D297,12AF,6698,6AA4,A2A3,EE86,408D,FCA4,B491
                    HEX        FA88,02AF,DE8F,3CAB,AC9B,3E8A,D0A3,E292,4690,9482,D49E,5683,BE8A,F295,A680,7289
                    HEX        6690,888C,C683,8481,CCA8,0E99,D6A8,BA8A,6295,B096,8E9D,A0AA,4A88,24A0,D4A8,8C93
                    HEX        E680,3C99,A882,348A,148A,ACAD,26A3,5092,BC94,24AA,268F,84AA,B294,8284,C297,A09B
                    HEX        AE99,9896,26A5,568D,2AA3,4895,D2A6,EE90,349E,D885,3A9E,7081,02AA,AC9E,0C8C,0487
                    HEX        2A99,4685,CCA1,A6AA,5695,C2A0,C2A8,EA9F,CEA1,1CA0,46AB,E28B,0496,E09C,489A,269B
                    HEX        4CA4,24A3,CA99,C0AB,9A98,70A4,BA9E,068A,4A97,ACAC,0C82,3299,0C89,7689,9C9B,F2A1
                    HEX        8686,5CAB,5AA6,F090,1EA3,4085,C4AD,AC80,4681,20A3,44A3,52A6,FA90,A88E,2C9B,6C93
                    HEX        B691,A4A5,1E8F,00AC,CA91,C48F,8C86,6E95,AEA8,E281,7286,3E9E,A69E,0E80,348F,94A0
                    HEX        5486,76A7,FC81,5A95,1C8A,4081,C883,4092,7A90,3A88,46A8,CE99,448D,DE9E,7E9A,F495
                    HEX        C685,A496,888E,46A3,5CA1,449E,4C81,9A93,EC92,2C8C,4AA1,38AB,A685,0289,669A,8CA7
                    HEX        488A,E285,0EA3,B282,D0A8,38A3,80AA,90A2,16AA,24A8,7695,F890,D6A6,3880,3494,FC83
                    HEX        C497,EAA4,4890,26A8,249B,4E9F,2097,388A,6A95,5686,9A89,48AE,0C8A,4AA9,78A7,0A96
                    HEX        9E98,4297,3297,76AC,EC80,F0A1,B899,B487,A68E,949D,A284,3A92,608B,E088,2894,E299
                    HEX        4683,4EA6,368D,0698,A68A,828C,7895,6C81,C891,689C,4A80,AA80,16A0,2E9E,2A80,1A99
                    HEX        B2A5,EC85,B8A0,E69C,3CA3,988E,C2A3,F490,76A9,DE83,B49B,8EA5,34AD,B085,56AE,528D
                    HEX        2A94,0C80,2C8A,AE9B,D280,3883,2A9B,2697,809B,B28F,089D,5C9C,F488,DAAD,1485,08AD
                    HEX        0889,3A99,8EA7,AEAD,4C83,3294,7AA4,749F,C6A5,3280,FA8B,E68F,50A9,D88F,9298,00A0
                    HEX        DE88,52AB,80AE,2297,D4A3,FE81,DC99,F09C,0280,50AE,42A4,5CAE,288F,D4AB,BA87,908C
                    HEX        D485,C6AB,1E8C,E483,2291,A485,2A92,028A,4E95,FE95,9082,98A7,9296,9C96,8086,0491
                    HEX        3485,B8A3,20A8,C6A6,5CA9,DAA3,9EA7,3897,E29A,86AA,26AA,2899,E085,E890,DAAB,AE96
                    HEX        6CA2,68A2,AAA7,8296,F88D,0494,4E8D,2A9E,028C,8A87,C092,3E9C,3E97,9A82,8898,CCAA
                    HEX        EA8B,64A2,EC95,3885,A08E,CCA3,388D,74AE,E48A,0C84,2C97,229C,E495,589F,2E97,6681
                    HEX        8A86,1E82,C087,2C82,FE9F,0489,8695,9698,2296,1096,4892,60AB,9A96,2A8A,2C91,66A1
                    HEX        808E,DAA8,0CAD,EE97,96A0,468B,AA89,70A2,408B,D894,D692,7886,5281,828E,BA9B,FC9C
                    HEX        8A9A,E6A1,A2AA,989B,B69B,5095,A4A2,0AA3,CE9E,28A1,EEA6,E6A3,6081,D288,4CAD,2A87
                    HEX        2C9C,C4A1,8E98,A484,8C9F,4A9F,A8A7,68A1,AC89,BC91,B89B,DA94,8286,BC99,3A85,2EA3
                    HEX        94A5,F086,C0AA,EA95,689A,B0AA,D8A3,4286,7A86,9C93,DCA3,C69C,0AAD,C2AB,E6AB,2C88
                    HEX        5E86,CAAB,7498,E08D,1499,A08C,F283,E083,84A2,56A1,E28F,488D,8295,4686,049D,3685
                    HEX        ACA0,0CA5,EEA1,82A5,6689,C887,16A5,D892,E0A6,62A1,5C92,F6AB,9E82,3C9C,80A7,8487
                    HEX        C680,5897,AC93,E68A,14A8,A88A,ACA5,20AF,FC8B,088A,1C82,2087,848C,5888,6283,BCA3
                    HEX        8AA9,5492,FCA9,C4AB,4897,7489,E2A1,9A87,5E9C,889B,6881,1491,229E,04A3,EAAE,4488
                    HEX        EE88,1C9B,A494,96A2,A09E,20AA,1EA0,2282,FCA6,48A8,D2A8,469A,B496,4A81,2892,EC8A
                    HEX        449F,B49E,C0A1,548D,069E,3C85,40AE,228F,FA8D,24A6,88AA,6293,4C94,6A9A,A6AD,FA92
                    HEX        8E89,8691,2EAD,9E87,0E9B,0880,0A99,2092,8687,6483,829D,B694,02AC,18A8,8CA5,94A2
                    HEX        DC8D,72AC,B6A0,2280,D0AB,2E99,9A8C,18A3,6889,408A,6EA4,5097,6A98,189B,6E89,F0A4
                    HEX        0684,A0AC,2497,208F,EE8D,9C8C,C28D,8E8E,149E,1487,108F,5A81,2080,808B,0082,BE96
                    HEX        9AA2,1AA8,A498,6089,8089,A49D,289B,7CA7,E2A3,809A,D28A,0899,20A5,1CAD,C097,6286
                    HEX        AA9B,D8AD,0AA0,2CA8,7284,2C8F,66A2,A480,6EAE,6E8E,00A2,E28A,4694,86A9,2487,D68A
                    HEX        30AB,F883,5C97,8A91,649C,00A5,CC97,D480,2E8D,BE85,8C98,1E87,74AC,4AA3,CEAD,2292
                    HEX        829B,F290,648B,D683,94AA,7CAE,E2A9,0099,689F,E485,8291,1C8C,6CA4,2CAB,E0A9,C8AA
                    HEX        B296,E29F,309E,4285,E092,1C80,6AA9,209C,2085,B68A,E09A,FC90,A499,428A,D29C,CAA3
                    HEX        3AAD,EE9F,7A9A,109E,FE90,C49B,509A,EAA6,6E84,48A9,688E,5688,C696,D29E,4480,4CA1
                    HEX        5EA1,BE9B,4E88,169E,6888,E0AE,CA80,10A5,D685,4A8D,2488,4C8F,6697,08AF,24AF,FC86
                    HEX        F895,DA99,FE88,1A91,CAA6,36A3,469F,EC8F,E883,5C81,7495,C694,2AA5,60A9,8C81,0AA2
                    HEX        AC8E,BEA8,C894,3683,3097,82A0,629C,EA85,38A6,CE94,E0A4,0C98,8AA0,5692,3C92,B88C
                    HEX        54AE,1A9B,9498,E4A3,52AE,4483,929B,F4A6,1894,329C,C09E,0C94,7095,AC85,9291,B0A8
                    HEX        44AD,A296,E686,2483,04A2,C8A0,CC88,5083,2EA1,469C,489E,EA8F,3EA3,129E,7CA2,3CA1
                    HEX        3899,32AD,8AA4,62A4,189E,389C,5690,C496,C280,1296,AA8C,BCA0,0EA8,E89E,0A89,629F
                    HEX        B680,7EA2,308D,F2A9,7E84,8A9B,8A89,B8A5,6AA7,128C,F0A9,EC9E,C8A3,6684,6084,CA88
                    HEX        14A0,8CA4,EA94,F697,6E86,2AAA,E6AE,F688,3E80,4AAE,0293,208D,C0A6,048F,12AA,5286
                    HEX        8CAA,2EAB,369E,1087,3892,2A91,328F,C483,3083,1A8F,02A7,0893,C896,468F,0296,808C
                    HEX        848B,F68B,A4A7,0CA8,AAAD,82A9,CC92,F29F,C882,84A5,08A0,AC9D,0694,32A8,F288,A29B
                    HEX        2688,8881,588D,5892,429E,6895,6A9C,048E,729A,BEA5,2CAA,F8AE,2C92,FEA1,9682,8890
                    HEX        BE9E,0882,80A0,868B,5E83,0282,C080,5CA4,0093,F486,9E93,A48C,3480,B682,4AA8,D89C
                    HEX        C88D,F0AE,4A8B,048C,80A4,E29E,CE83,E283,628D,2AA0,C082,8098,1696,348D,8087,B687
                    HEX        D299,F483,84A0,068F,28A0,7A9D,728B,5A9F,3C88,9A91,D89E,488F,BE99,B287,C892,AAA0
                    HEX        0E82,34A8,A896,5481,F48B,FA9C,22AF,F081,BA8C,DA97,CC83,6095,F49C,48A3,0EA5,1885
                    HEX        0A9D,12A8,3692,4090,3099,E294,E897,9696,008F,A8A0,F49A,EAA9,AC8C,E090,1287,EAA1
                    HEX        CC8A,12A0,8493,708E,4299,8893,C296,729F,8293,A4AD,649A,A094,2C83,BC85,C494,BAAA
                    HEX        FC8D,02A0,E290,6EA7,2285,A4A8,4CA6,88A7,8E87,128F,DA85,6C83,20A1,5E81,6892,7890
                    HEX        AAA2,1A9E,1A94,7098,7EA9,5490,6AA6,06A5,84AE,FCAE,6A9D,4899,6481,FA97,9C91,2CAF
                    HEX        4CAE,FEAB,C283,68A9,0689,7295,A28C,F690,66AB,E688,F29C,0CAF,4290,86A0,C285,F6A6
                    HEX        78AC,649F,609C,A0A3,D48A,068E,829A,E88B,5E92,0EAA,82AC,CA96,50A1,028E,E4A1,ECA8
                    HEX        6083,8C95,8C96,D68D,EE9C,3288,6890,849D,A0A2,4883,10A8,70A7,A285,6A90,FEA6,EC88
                    HEX        3A9C,04AD,66A4,84A9,6289,A282,F88B,8AAE,84A4,C091,749A,10A3,EA83,42A9,2C9E,E89A
                    HEX        6495,DC9E,8298,10AD,E297,0687,A298,42A6,C69B,1896,18AD,0096,C083,E497,DE9C,B0AD
                    HEX        A89E,AC8A,0A9B,2287,3A83,9A9B,22AD,5E88,98A2,CCAD,CAA8,42A8,4295,A6A3,B6AD,4E90
                    HEX        FCA1,509C,3CA6,EE92,86AC,2A82,9282,E095,5E9F,7C86,90AA,36A1,A293,D6A1,94AC,46A6
                    HEX        C68C,7AA7,D6A3,CCAB,6492,8096,DC83,E29C,9287,D6AD,92A2,C08A,70A9,32A1,8C8E,8090
                    HEX        168A,928E,BC87,1A80,B087,4499,AA99,9AA7,28A5,6A81,7684,1080,C885,2682,F8AB,58AB
                    HEX        A691,A696,E280,9491,E886,CCA6,4A90,6A89,0AA5,C2A1,4C86,BEA0,2694,4C9C,A080,B080
                    HEX        40AD,308A,2A8F,0094,C085,80A5,64A6,FC9F,3283,DE97,C880,DA92,7EAC,9CA0,AE85,BA8F
                    HEX        0482,36AD,F088,E6A6,6EAC,809F,D488,648D,38AD,A8A3,ACA8,CA97,F29A,1494,10AA,D888
                    HEX        3EA1,D292,1EAD,6897,1294,3694,BA94,E086,2EA8,DEA8,E88F,56A6,7C8B,268D,5AA4,9A9D
                    HEX        0294,20AD,668B,029E,F8A4,5EA6,12AD,76A2,E699,C888,50A6,1291,8A8C,E080,B699,D4A1
                    HEX        8A8E,8682,54AB,E2AE,A28E,4CAB,0AA8,86A4,EC99,74A7,60A2,8E8C,249C,4881,029D,88A0
                    HEX        72AE,6486,108A,7C98,E0A1,68A4,428F,869B,56A4,9489,0693,FE92,02A3,18AF,728E,64A4
                    HEX        A2A2,6C84,2AAB,A6A5,7298,CE97,DCA8,D085,BE8C,EEAB,228A,2887,0C8E,8689,1EAA,269E
                    HEX        7CA9,92AA,B291,F686,E081,2E88,669D,DA9C,3C80,209E,2091,2099,1280,68AE,5C8D,429F
                    HEX        6493,EEA4,EE8B,5886,06A7,A0A7,9691,BA85,A887,B482,6298,C682,7E86,3894,6A8B,6C9D
                    HEX        4EA4,CC8F,2E92,A885,DAA1,B4AA,2A97,7A8E,9891,7881,DC8F,AE91,24A5,D094,90A7,AAA3
                    HEX        2082,5C90,7090,88A4,D2AB,A894,8C9D,40A6,E892,DCAD,5A83,6EA9,0084,148F,8886,3A8D
                    HEX        FE83,C8A6,EE81,9898,5A8B,BCA5,28AD,80A2,349C,42AD,0CAC,F8A9,8482,508D,4880,3080
                    HEX        D294,66A7,B6A8,4E9A,088C,2CA0,868E,04AC,0A82,8E84,668D,12A3,B494,A2AD,F892,7089
                    HEX        6692,1C87,2096,389E,0A93,8698,C282,CC9E,3EA6,A88F,6CA9,90AC,EA9E,5C83,3C8D,4A9C
                    HEX        2E80,C4A6,5288,4C8A,3E92,3EAB,FC95,F2AE,3C8A,2499,FEA9,1A8C,1285,6C8B,489F,2AA6
                    HEX        0A91,368F,48A4,0A8F,CCA5,409F,DA88,8282,C0A5,5283,EA99,FA9A,A6AC,B0A0,FEAE,EAA3
                    HEX        6284,1C91,ECA6,3888,529C,C4A8,BCA8,BA96,768E,4490,08A8,388F,C2A6,E894,0699,6C92
                    HEX        868C,84AC,048A,3E85,4C90,F897,0287,9C8E,6490,B6AA,7C93,DE99,7AA9,2683,CE80,0080
                    HEX        64AB,94A7,8696,0891,9A8E,3AA6,06AC,1094,509F,0A94,98AA,A084,72A2,A487,34A6,22A5
                    HEX        C096,0C91,9C87,4094,FC88,4CA8,0C85,7A89,46A1,F09A,44AE,148C,CC8C,289E,26AD,C487
                    HEX        FA9F,E499,ECAD,2680,54A1,6E81,7898,CC94,BA80,0CA3,EC9A,70AE,6884,B082,7A95,A08A
                    HEX        D09E,C6A0,C29C,52A9,0CA0,D49C,AC94,90A0,CA83,CC8D,5681,7C90,7A81,C29B,708B,64AE
                    HEX        30A3,0A8E,8AA2,CCA0,6A93,4080,A6A0,C88F,809D,AAA5,F68D,7490,108C,1C99,60A6,40A4
                    HEX        E8AB,4494,00AD,66A6,A0A0,2A83,429C,44A8,0E87,A08F,E094,88A2,FC9A,58A9,7A98,A693
                    HEX        488B,588B,E89F,529F,8EA0,CEAB,DE8D,A69B,CA8D,B2AD,B4AD,22AA,568B,DE85,8CAC,948C
                    HEX        0E96,06A0,7A93,1680,8AA5,A687,B485,1EA8,E490,28AF,6686,6098,0685,6C89,9AAA,4699
                    HEX        4A8A,188A,A684,A091,4486,7E90,BEA3,4C9A,A28F,A280,FAAE,0484,5A88,6090,0898,2696
                    HEX        5AA9,E8A6,E888,088E,82A2,8A95,A491,E4A4,0EAF,DA8A,3499,E480,E288,A884,FEA4,3492
                    HEX        7686,6A8D,6EA2,2CA1,8A96,0298,20A0,3E83,309C,0480,2AA8,9893,8690,36A8,EA86,EA8A
                    HEX        AA8A,FAA6,B48C,D68F,A8AA,4AA6,9889,7E95,1C94,9689,ECA9,0E8C,B09E,42AB,3688,5295
                    HEX        5AAB,F886,8AAC,869A,6C8D,5C86,8E91,B29E,B885,368A,6CA7,D88A,2288,F6A1,2685,24A1
                    HEX        B08A,7A8B,6092,E494,68AB,B88A,2088,F2AB,D8A8,F481,8CAE,D088,6A88,EC9C,729D,F281
                    HEX        16A3,74A4,4A9E,9C9D,5AAE,CE8A,AEA3,C08D,2C80,FE9A,B091,B094,7484,06AD,CC85,569A
                    HEX        9289,E8A9,0E85,8CA9,9098,1E9B,6AA2,8693,0C9B,C688,0896,6683,248A,22AB,549C,4CA3
                    HEX        E08A,C48A,7AAE,8A98,8486,468A,C6A3,B2A3,06A8,28A8,C28C,8EAC,CE9C,62AE,96A5,B4A8
                    HEX        60A1,5A9C,2CA6,5488,8091,26A6,848E,F08B,BA82,6088,E685,D28F,4A85,F286,7CAC,F49F
                    HEX        829F,9CAC,C099,AE80,5697,C4AA,3292,188C,08AA,12A5,8A93,AAAA,3A8F,C485,E2A4,449A
                    HEX        9C98,2485,0C9E,AA8F,7CA4,E88D,22A1,168C,6484,ACA2,3A8A,F083,5AA1,2888,A68F,7AA2
                    HEX        04AF,3E8D,1E99,E695,AE8C,22A3,AA87,408F,0CA7,8A84,E88A,4C92,D08A,2A96,6489,C0A0
                    HEX        689D,6695,9284,48AD,A6A7,1C9E,E68B,EC83,C49C,E692,CA8A,A28A,C4A3,5483,D8A6,A689
                    HEX        428B,6898,AA84,109B,1CA8,26A0,2AAD,429A,30A1,C287,8684,6688,7293,E49F,8882,E2A8
                    HEX        6C9F,1282,3AA3,C28F,AA9D,00AF,E69F,C68D,8496,C6AA,C08F,0691,9096,0E8F,8C8C,248D
                    HEX        AE87,AE9E,F89C,28A6,F4A4,EC8B,969D,6C97,CC96,4083,98A5,4CA9,D88D,6E9F,DC8A,C2AA
                    HEX        D080,72A4,76A4,3699,5A97,7698,BE87,6CAE,4A95,E4AB,A291,9487,D694,F2A4,769F,E2A6
                    HEX        CC82,F6A4,909D,4294,6AAE,788B,BEAA,E8AE,C08C,E48F,9887,549A,088F,3483,64A9,22A6
                    HEX        9496,0499,229B,629D,18AA,A698,1882,228D,7E81,EE9A,0285,2CA3,D8AB,6290,F0AB,F48D
                    HEX        62A2,869D,2A8C,668E,82AA,749D,428D,228C,C0AD,14AD,4AAB,52A4,62A6,1A87,0EAD,E8A3
                    HEX        CAA1,1EAF,3E99,569F,4E83,92A0,9293,4C8B,E486,969B,DA83,08AC,008C,A898,4C95,FAA1
                    HEX        6292,3092,4495,589A,2C85,2480,9087,1480,88AE,4A94,F08D,C29E,ACA3,149B,7C9A,FA81
                    HEX        EA88,A48E,A2A8,60AC,049B,9C89,DA8F,28AB,60A4,1496,6A92,628B,B891,2C99,4A83,688D
                    HEX        D2A1,9AAC,3AAB,40A1,E899,B2AA,1A82,B89E,ACA7,6C8E,968E,ECA3,DE80,1E80,D2AD,D48D
                    HEX        20AB,3E8F,C69E,8895,1299,38A1,E6A4,2E94,16AF,A0A5,A294,14AF,1A96,42A1,8484,3EAD
                    HEX        40A3,F0A6,B28C,0884,AA9E,C8A8,08A2,A299,AC96,B2A0,62A7,32A3,B4A5,E49C,2896,08A5
                    HEX        D499,2496,10AF,A2AC,8CA2,40A8,2482,90A5,EC8D,029B,AC8F,0A80,4EAE,F4AE,DAA6,169B
                    HEX        B887,B285,1E91,469E,F69A,88AC,A699,8E9B,7EAE,D08F,CA82,4497,46A4,768B,5E9A,489C
                    HEX        DEA1,4E9C,4C9F,5297,D883,AA94,CC80,86A5,9882,E8AD,AA98,F89F,6498,B896,40AB,1CA3
                    HEX        B280,008A,AC91,88A9,D899,5C9F,9E9D,2E9C,F69F,6A84,5088,968C,9CA2,3AA8,A09D,6693
                    HEX        7493,56A9,C292,A29D,CA94,C697,789A,AC82,B2A8,E492,6E98,7481,C088,E694,FC92,24AD
                    HEX        02AD,7EA7,C89C,0680,FE86,1E9E,0C87,FE97,4C85,6488,6E90,BCAD,98AC,6497,B685,E0A8
                    HEX        46AD,C288,D2A3,6C9C,789F,CE8F,709D,548B,269C,00A3,C4A0,4C97,16A8,EAA8,328A,7E93
                    HEX        CA8C,ECAE,4C9E,4292,2491,748E,AE8A,4097,4C8D,E09F,8C8B,C68A,DCAB,5E95,32AB,A48F
                    HEX        1AAF,BAA3,04A8,D680,84A7,E681,A093,58A4,7C95,0C8F,DA80,E0AB,44A6,089E,4688,C28A
                    HEX        5290,2CAD,1085,E6AD,28A3,308F,B099,54A9,168F,2A88,D880,E885,A099,B88F,629A,8082
                    HEX        E8A1,66AE,7A9F,4481,D4AD,9EA5,9CA5,A8AD,5883,209B,68AC,1C96,B8AA,2A9C,DC94,2EA6
                    HEX        CC91,288D,30A6,7693,3C94,B68F,8495,8AA7,4283,F297,4A86,9EAA,C88A,56AB,B8AD,D688
                    HEX        F292,5EAB,E295,4A92,CEA3,5A9A,64A1,B894,30AD,0EA0,F89A,BAA5,D69E,6E9A,FA95,30A8
                    HEX        64AC,14A3,0A87,F095,06AA,2C87,FE9C,ECAB,5A86,E08B,989D,6C9A,889D,D0A6,22A8,1A85
                    HEX        6886,F2A6,E69A,AA96,50AB,3EA8,E4AE,B6A5,F28D,F888,6A9F,E49A,D083,2A85,DC85,9493
                    HEX        2897,F8A6,92A5,0885,A6A2,9EA2,6CA1,62AC,DCA1,2A8D,A88C,8C9B,C294,BA91,78AE,2283
                    HEX        6093,649D,C6A1,2E85

                    
********************************************************************
*  SettingsScreen() :  Display/Handle Game Settings (OUT:A=Error). *
********************************************************************
SettingsScreen      JSL        ClearKDBStrobe       ; Clear Keyboard
                    JSL        FadeOut              ; Fadout to Black
                    LDA        #$0000
                    JSL        SetBorderColor                    
*----
                    LDX        #$1FFE               ; RAZ Graphic Page
                    LDA        #$000                   
SS_RAZ_GP           STAL       $012000,X
                    STAL       $014000,X
                    STAL       $016000,X
                    STAL       $018000,X
                    DEX
                    DEX
                    BPL        SS_RAZ_GP                    
*----
                    LDY        #$0010               ; Put SCB on Screen
SS_SCB              LDA        SS_TabSCBLine,Y
                    TAX
                    LDA        #$0101
                    STAL       $019D00,X
                    LDA        #$0202
                    STAL       $019D02,X
                    LDA        #$0302
                    STAL       $019D04,X   
                    LDA        #$0303
                    STAL       $019D06,X   
                    DEY
                    DEY
                    BPL        SS_SCB   
*----
SS_TextLine         LDX        #SS_Line_01          ; Build Text Lines
                    LDY        #$2657               ; @Dst 46,10
                    JSL        DisplayString
                    LDX        #SS_Line_02
                    LDY        #$2C97               ; @Dst 46,20
                    JSL        DisplayString
*--
SS_TextLine_START   LDX        #SS_Line_03
                    LDY        #$3CD7               ; @Dst 46,46
                    JSL        DisplayString
*--                    
SS_TextLine_P1      LDA        SS_PLAYER_1          ; Player 1 : 1-4
                    DEC
                    ASL
                    TAY
                    LDX        SS_TAB_PLAYER_1,Y
                    LDY        #$4317               ; @Dst 46,56
                    JSL        DisplayString                    
*--
SS_TextLine_P2      LDA        SS_PLAYER_2          ; Player 2 : 1-4
                    DEC
                    ASL
                    TAY
                    LDX        SS_TAB_PLAYER_2,Y
                    LDY        #$4957               ; @Dst 46,66
                    JSL        DisplayString                    
*--
SS_TextLine_MU      LDA        SS_MUSIC             ; Music : 0,1
                    ASL
                    TAY
                    LDX        SS_TAB_MUSIC,Y
                    LDY        #$4F97               ; @Dst 46,76
                    JSL        DisplayString
*--
SS_TextLine_SD      LDA        SS_SOUND             ; Sound : 0,1
                    ASL
                    TAY
                    LDX        SS_TAB_SOUND,Y
                    LDY        #$55D7               ; @Dst 46,86
                    JSL        DisplayString
*--
SS_TextLine_HS      LDX        #SS_Line_HIGH_SCORE
                    LDY        #$5C17               ; @Dst 46,96
                    JSL        DisplayString
*--
SS_TextLine_EX      LDX        #SS_Line_EXIT_PROG
                    LDY        #$6257               ; @Dst 46,106
                    JSL        DisplayString
*----
SS_Man              LDX        #$000A
SS_Man_1            PHX
                    LDA        SS_TabManAddress,X
                    TAY                             ; Sprite @
                    LDA        SS_TabManID,X        ; Sprite ID
                    JSR        DrawManSprite
                    PLX
                    DEX
                    DEX
                    BPL        SS_Man_1
*--
                    LDX        #$007E
SS_SetPalette_1     LDA        SS_Palette_1,X       ; Copy 4 Palettes
                    STA        Buffer_Palette,X
                    DEX
                    DEX
                    BPL        SS_SetPalette_1
*--
SS_MarkSelection    LDA        #$0001               ; Mark Selected Entry
                    STA        SS_SELECTION
                    INC
                    ASL
                    TAY
                    LDA        SS_TabSCBLine,Y
                    TAX
                    LDA        #$0000
                    STAL       $019D00,X 
                    STAL       $019D02,X
                    STAL       $019D04,X 
                    STAL       $019D06,X
*--
                    LDA        SS_LoadEraseCode3    ; Load EraseCode3
                    BNE        SS_FadeIn
SS_LoadFile_Erase3  LDX        #SS_EraseData003     ; EraseCode3.bin
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    STA        SS_UncompressE3_Sz+1
                    BCC        SS_UncompressErase3
                    LDA        #SS_EraseData003     ; Error
                    JMP        BS_End
SS_UncompressErase3 SEP        #$20                 ; LZ4 Uncompress
                    LDA        LoadUnpackBank+1
                    XBA
                    LDA        EraseCode3Bank+1     ; Src Bank | Dst Bank
                    REP        #$30
                    LDX        #$0000               ; Src Offset | Dst Offset ($80=$8000)
SS_UncompressE3_Sz  LDY        #$0000               ; PackDataSize
                    JSL        UnpackLZ4
                    INC        SS_LoadEraseCode3
*--
SS_FadeIn           LDA        #Buffer_Palette      ; Fade In
                    JSL        FadeIN       
*--
                    LDA        #$0005               ; Dark Grey Border
                    JSL        SetBorderColor
*--------
                    LDA        SS_LoadGameSound     ; Load Game Sound + Copy Sound data into DOC Ram
                    BNE        SS_ManAnimation
                    JSL        LoadGameSound        
*                    BCC        SS_ManAnimation
                    INC        SS_LoadGameSound
*--------
SS_ManAnimation     INC        SS_AnimCounter       ; Draw Men once every 16 VBL
                    LDA        SS_AnimCounter
                    CMP        #$0008
                    BNE        SS_CheckForAction 
                    STZ        SS_AnimCounter
                    LDY        #$000A               ; Man Animation x 6
SS_ManDirection     LDA        SS_TabManDirection,Y
                    BEQ        SS_ManDirUp
*--
SS_ManDirDown       LDA        SS_TabManID,Y        ; Down
                    BEQ        SS_ManDirDownBounce
                    DEC
                    STA        SS_TabManID,Y
                    BRA        SS_ManNext
SS_ManDirDownBounce LDA        #$0001               ; Go Up
                    STA        SS_TabManID,Y
                    LDA        #$0000
                    STA        SS_TabManDirection,Y
                    BRA        SS_ManNext
*--
SS_ManDirUp         LDA        SS_TabManID,Y        ; Up
                    CMP        #$0003
                    BEQ        SS_ManDirUpBounce
                    INC
                    STA        SS_TabManID,Y
                    BRA        SS_ManNext
SS_ManDirUpBounce   LDA        #$0002               ; Go Down
                    STA        SS_TabManID,Y
                    LDA        #$0001
                    STA        SS_TabManDirection,Y
                    BRA        SS_ManNext                    
*--
SS_ManNext          DEY
                    DEY
                    BPL        SS_ManDirection                    
*----
                    LDX        #$000A
SS_Man_Display_1    PHX
                    LDA        SS_TabManAddress,X
                    TAY                             ; Sprite @
                    LDA        SS_TabManID,X        ; Sprite ID
                    JSR        DrawManSprite
                    PLX
                    DEX
                    DEX
                    BPL        SS_Man_Display_1
*--------
SS_CheckForAction   JSR        WaitForNextVBL       ; Wait For next VBL 
                    JSR        CheckForKeyAction
                    BEQ        SS_ManAnimation      ; No Key 
                    CMP        #$0030               ; Action
                    BEQ        SS_Action
SS_CFA_1            CMP        #$0038               ; UP
                    BNE        SS_CFA_2
                    JMP        SS_GoUp
SS_CFA_2            CMP        #$0032               ; DOWN
                    BNE        SS_CFA_3
                    JMP        SS_GoDown
SS_CFA_3            CMP        #$0034               ; LEFT
                    BNE        SS_CFA_4
                    JMP        SS_GoLeft
SS_CFA_4            CMP        #$0036               ; RIGHT
                    BNE        SS_CFA_5
                    JMP        SS_GoRight
SS_CFA_5            JMP        SS_Continue
*----
SS_Action           LDA        SS_SELECTION
                    CMP        #$0001
                    BEQ        SS_ActionPlayGame    ; Start Game
SS_Action_1         CMP        #$0002
                    BNE        SS_Action_2
                    JMP        SS_Player1_Right     ; Player 1
SS_Action_2         CMP        #$0003
                    BNE        SS_Action_3
                    JMP        SS_Player2_Right     ; Player 2
SS_Action_3         CMP        #$0004
                    BNE        SS_Action_4
                    JMP        SS_Music             ; Music
SS_Action_4         CMP        #$0005
                    BNE        SS_Action_5
                    JMP        SS_Sound             ; Sound
SS_Action_5         CMP        #$0006
                    BNE        SS_Action_6
                    JMP        SS_ActionHighScore   ; High Score
SS_Action_6         CMP        #$0007
                    BNE        SS_Action_7
                    JMP        SS_ActionQuitGame    ; Quit Game
SS_Action_7         JMP        SS_Continue
*--
SS_ActionPlayGame   LDA        #$0030               ; '0' = Start Game
                    JMP        SS_End
*---
SS_Player1_Right    LDA        SS_PLAYER_1          ; Player 1 Right 1-4 ++
                    CMP        #$0004
                    BNE        SS_Player1_R1
                    LDA        #$0000
SS_Player1_R1       INC
                    CMP        #$0002               ; Joypad
                    BEQ        SS_Player1_R2
                    CMP        #$0004               ; Computer
                    BEQ        SS_Player1_R2
                    CMP        SS_PLAYER_2          ; 3:Keyboard/Keyboard & 1:Joystick/Joystick forbiden
                    BNE        SS_Player1_R2
                    INC
SS_Player1_R2       STA        SS_PLAYER_1
                    DEC
                    ASL
                    TAY
                    LDX        SS_TAB_PLAYER_1,Y
                    LDY        #$4317               ; @Dst 46,56
                    JSL        DisplayString                                        
                    JMP        SS_Continue
*---
SS_Player1_Left     LDA        SS_PLAYER_1          ; Player 1 Left 1-4 --
                    CMP        #$0001
                    BNE        SS_Player1_L1
                    LDA        #$0005
SS_Player1_L1       DEC
                    CMP        #$0002               ; Joypad
                    BEQ        SS_Player1_L2
                    CMP        #$0004               ; Computer
                    BEQ        SS_Player1_L2
                    CMP        SS_PLAYER_2          ; 3:Keyboard/Keyboard & 1:Joystick/Joystick forbiden
                    BNE        SS_Player1_L2
                    DEC
                    CMP        #$0000
                    BNE        SS_Player1_L2
                    LDA        #$0004
SS_Player1_L2       STA        SS_PLAYER_1
                    DEC
                    ASL
                    TAY
                    LDX        SS_TAB_PLAYER_1,Y
                    LDY        #$4317               ; @Dst 46,56
                    JSL        DisplayString                                        
                    JMP        SS_Continue
*---                    
SS_Player2_Right    LDA        SS_PLAYER_2          ; Player 2 Right 1-4 ++
                    CMP        #$0004
                    BNE        SS_Player2_R1
                    LDA        #$0000
SS_Player2_R1       INC
                    CMP        #$0002               ; Joypad
                    BEQ        SS_Player2_R2
                    CMP        #$0004               ; Computer
                    BEQ        SS_Player2_R2
                    CMP        SS_PLAYER_1          ; 3:Keyboard/Keyboard & 1:Joystick/Joystick forbiden
                    BNE        SS_Player2_R2
                    INC
SS_Player2_R2       STA        SS_PLAYER_2
                    DEC
                    ASL
                    TAY
                    LDX        SS_TAB_PLAYER_2,Y
                    LDY        #$4957               ; @Dst 46,66
                    JSL        DisplayString                                        
                    JMP        SS_Continue
*---
SS_Player2_Left     LDA        SS_PLAYER_2          ; Player 2 Left 1-4 --
                    CMP        #$0001
                    BNE        SS_Player2_L1
                    LDA        #$0005
SS_Player2_L1       DEC
                    CMP        #$0002               ; Joypad
                    BEQ        SS_Player2_L2
                    CMP        #$0004               ; Computer
                    BEQ        SS_Player2_L2
                    CMP        SS_PLAYER_1          ; 3:Keyboard/Keyboard & 1:Joystick/Joystick forbiden
                    BNE        SS_Player2_L2
                    DEC
                    CMP        #$0000
                    BNE        SS_Player2_L2
                    LDA        #$0004
SS_Player2_L2       STA        SS_PLAYER_2
                    DEC
                    ASL
                    TAY
                    LDX        SS_TAB_PLAYER_2,Y
                    LDY        #$4957               ; @Dst 46,66
                    JSL        DisplayString                                        
                    JMP        SS_Continue
*---
SS_Music            LDA        SS_MUSIC
                    BNE        SS_Music_Off
SS_Music_On         INC        SS_MUSIC
                    LDA        #$0000
                    JSL        SetStatusMusic       ; Music Status : ON
                    LDX        #SS_Line_MUSIC_ON
                    BRA        SS_Music_Set
SS_Music_Off        STZ        SS_MUSIC
                    LDA        #$0001
                    JSL        SetStatusMusic       ; Music Status : OFF
                    LDX        #SS_Line_MUSIC_OFF
SS_Music_Set        LDY        #$4F97               ; @Dst 46,76
                    JSL        DisplayString
                    JMP        SS_Continue
*---
SS_Sound            LDA        SS_SOUND
                    BNE        SS_Sound_Off
SS_Sound_On         INC        SS_SOUND
                    LDA        #$0000
                    JSL        SetStatusSound       ; Sound Status : ON
                    LDX        #SS_Line_SOUND_ON
                    BRA        SS_Sound_Set
SS_Sound_Off        STZ        SS_SOUND
                    LDA        #$0001
                    JSL        SetStatusSound       ; Sound Status : OFF
                    LDX        #SS_Line_SOUND_OFF
SS_Sound_Set        LDY        #$55D7               ; @Dst 46,86
                    JSL        DisplayString
                    JMP        SS_Continue
*---
SS_ActionHighScore  LDA        #$0048               ; 'H' = High Score Screen
                    JMP        SS_End
*---
SS_ActionQuitGame   LDA        #$0051               ; 'Q' = Quit Game
                    JMP        SS_End
*----
SS_GoLeft           LDA        SS_SELECTION
                    CMP        #$0002
                    BNE        SS_GoLeft_1
                    JMP        SS_Player1_Left      ; Player 1
SS_GoLeft_1         CMP        #$0003
                    BNE        SS_GoLeft_2
                    JMP        SS_Player2_Left      ; Player 2
SS_GoLeft_2         CMP        #$0004
                    BNE        SS_GoLeft_3
                    JMP        SS_Music             ; Music
SS_GoLeft_3         CMP        #$0005
                    BNE        SS_GoLeft_4
                    JMP        SS_Sound             ; Sound
SS_GoLeft_4         JMP        SS_Continue
*----
SS_GoRight          LDA        SS_SELECTION
                    CMP        #$0002
                    BNE        SS_GoRight_1
                    JMP        SS_Player1_Right     ; Player 1
SS_GoRight_1        CMP        #$0003
                    BNE        SS_GoRight_2
                    JMP        SS_Player2_Right     ; Player 2
SS_GoRight_2        CMP        #$0004
                    BNE        SS_GoRight_3
                    JMP        SS_Music             ; Music
SS_GoRight_3        CMP        #$0005
                    BNE        SS_GoRight_4
                    JMP        SS_Sound             ; Sound
SS_GoRight_4        JMP        SS_Continue
*----
SS_GoUp             LDA        SS_SELECTION         ; Select Up
                    CMP        #$0001
                    BEQ        SS_Continue
                    STA        SS_SELECTION_PREV
                    DEC
                    STA        SS_SELECTION
                    BRA        SS_UpdateSelect
*--
SS_GoDown           LDA        SS_SELECTION         ; Select Down
                    CMP        #$0007
                    BEQ        SS_Continue
                    STA        SS_SELECTION_PREV
                    INC
                    STA        SS_SELECTION
*----
SS_UpdateSelect     LDA        SS_SELECTION_PREV    ; Clear Previous Selected
                    INC
                    ASL
                    TAY
                    LDA        SS_TabSCBLine,Y
                    TAX
                    LDA        #$0101
                    STAL       $019D00,X
                    LDA        #$0202
                    STAL       $019D02,X
                    LDA        #$0302
                    STAL       $019D04,X   
                    LDA        #$0303
                    STAL       $019D06,X   
*--
                    LDA        SS_SELECTION         ; Mark Selected Entry
                    INC
                    ASL
                    TAY
                    LDA        SS_TabSCBLine,Y
                    TAX
                    LDA        #$0000
                    STAL       $019D00,X 
                    STAL       $019D02,X
                    STAL       $019D04,X 
                    STAL       $019D06,X
*--------
SS_Continue         JMP        SS_ManAnimation
*----
SS_End              RTS
                    
*----------   SettingsScreen Variables   ---------
SS_LoadEraseCode3   HEX        0000                 ; Load EraseCode3 status
SS_LoadGameSound    HEX        0000                 ; Load Game Sound status
SS_EraseData003     STRL       '1/Data/EraseCode3.bin'   ; Erase Code 3/3 LZ4
SS_Line_01          ASC        'SELECT OPTION WITH ARROW KEYS',00
SS_Line_02          ASC        'AND CHANGE WITH THE ENTER KEY',00
SS_Line_03          ASC        '        START GAME',00
SS_Line_PLAYER_1_JS ASC        '     PLAYER 1 : JOYSTICK',00
SS_Line_PLAYER_1_JP ASC        '     PLAYER 1 : JOYPAD  ',00
SS_Line_PLAYER_1_KB ASC        '     PLAYER 1 : KEYBOARD',00
SS_Line_PLAYER_1_UC ASC        '     PLAYER 1 : COMPUTER',00
SS_Line_PLAYER_2_JS ASC        '     PLAYER 2 : JOYSTICK',00
SS_Line_PLAYER_2_JP ASC        '     PLAYER 2 : JOYPAD  ',00
SS_Line_PLAYER_2_KB ASC        '     PLAYER 2 : KEYBOARD',00
SS_Line_PLAYER_2_UC ASC        '     PLAYER 2 : COMPUTER',00
SS_Line_MUSIC_ON    ASC        '        MUSIC : ON ',00
SS_Line_MUSIC_OFF   ASC        '        MUSIC : OFF',00
SS_Line_SOUND_ON    ASC        '        SOUND : ON ',00
SS_Line_SOUND_OFF   ASC        '        SOUND : OFF',00
SS_Line_HIGH_SCORE  ASC        '        HIGH SCORE',00
SS_Line_EXIT_PROG   ASC        '       EXIT PROGRAM',00

SS_TAB_PLAYER_1     DA         SS_Line_PLAYER_1_JS,SS_Line_PLAYER_1_JP,SS_Line_PLAYER_1_KB,SS_Line_PLAYER_1_UC
SS_TAB_PLAYER_2     DA         SS_Line_PLAYER_2_JS,SS_Line_PLAYER_2_JP,SS_Line_PLAYER_2_KB,SS_Line_PLAYER_2_UC
SS_TAB_MUSIC        DA         SS_Line_MUSIC_OFF,SS_Line_MUSIC_ON
SS_TAB_SOUND        DA         SS_Line_SOUND_OFF,SS_Line_SOUND_ON 

SS_PLAYER_1         ENT
                    HEX        0100                 ; 1:Joystick, 2:Joypad ,3:Keyboard, 4:Computer
SS_PLAYER_2         ENT
                    HEX        0400                 ; 1:Joystick, :2Joypad, 3:Keyboard, 4:Computer
SS_SOUND            HEX        0100                 ; 0=Off, 1=On, 
SS_MUSIC            HEX        0100                 ; 0=Off, 1=On 
SS_SELECTION        HEX        0100                 ; Current Selection  1-7
SS_SELECTION_PREV   HEX        0100                 ; Previous Selection 1-7

SS_AnimCounter      HEX        0000

SS_Palette_1        HEX        5505,0000,0500,2602,4604,AF07,0006,0008,000E,760D,860E,EE0E,1101,2202,3303,FF0F
SS_Palette_2        HEX        5505,0000,0500,2602,4604,AF07,0006,0008,000E,760D,860E,EE0E,1101,2202,3303,000E
SS_Palette_3        HEX        5505,0000,0500,2602,4604,AF07,0006,0008,000E,760D,860E,EE0E,1101,2202,3303,0008
SS_Palette_4        HEX        5505,0000,0500,2602,4604,AF07,0006,0008,000E,760D,860E,EE0E,1101,2202,3303,0006
SS_TabSCBLine       HEX        0A00,1400,2E00,3800,4200,4C00,5600,6000,6A00
SS_TabManID         HEX        0000,0100,0200,0300,0200,0100
SS_TabManAddress    HEX        1670,8A6F,9E6F,126F,C66F,DA6F                 
SS_TabManDirection  HEX        0000,0000,0000,0100,0100,0100  ; 0:UP, 1:Down         
         
                    
*******************************************************************************
*  DrawManSprite() :  Display Man Sprite on Screen (IN:A=ManID,IN:Y=@Screen). *
*******************************************************************************
DrawManSprite       ASL
                    TAX
                    JSR       (SpriteManTab,X)
                    RTS

*----------   DrawManSprite Variables   ---------
SpriteManTab        DA        SpriteMan0,SpriteMan1,SpriteMan2,SpriteMan3 
*----
SpriteMan0          CLC                             ; 28x68, 1404 bytes, 2732 cycles
                    SEI                             ; Disable Interrupts
                    PHD                             ; Backup Direct Page
                    TSC                             ; Backup Stack
                    STA        StackAddress         
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 01/
                    ORA        #$0030               
                    STAL       $E1C068              
                    TYA                             ; Y = Sprite Target Screen Address (upper left corner)
                    TCS                             ; New Stack address
                    LDX        #$0000               ; Pattern #1 : 317
                    LDY        #$1212               ; Pattern #2 : 27
                    LDA        #$1010               ; Pattern #3 : 23
                    TCD                             
*--                                                 
                    TSC                             
                    ADC        #$000D               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 1
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 2
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 3
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 4
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 5
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 6
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 7
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 8
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 9
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 10
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 11
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 12
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 13
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 14
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 15
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 16
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 17
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0010                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 18
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $3311                
                    PEA        $1111                
                    PEA        $2123                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 19
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4034                
                    PEA        $1311                
                    PEA        $1111                
                    PEA        $1133                
                    PEA        $2203                
                    PHX                             
                    TSC                             ; Line 20
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0040                
                    PEA        $3534                
                    PEA        $1411                
                    PEA        $1111                
                    PEA        $1121                
                    PEA        $2331                
                    PHX                             
                    TSC                             ; Line 21
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0054                
                    PEA        $4343                
                    PEA        $1611                
                    PEA        $1111                
                    PEA        $1136                
                    PEA        $3212                
                    PEA        $0300                
                    TSC                             ; Line 22
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4035                
                    PEA        $3242                
                    PEA        $1611                
                    PEA        $1111                
                    PEA        $1126                
                    PEA        $2323                
                    PEA        $3100                
                    TSC                             ; Line 23
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5024                
                    PEA        $2342                
                    PEA        $1611                
                    PEA        $1111                
                    PEA        $1116                
                    PEA        $3432                
                    PEA        $1203                
                    TSC                             ; Line 24
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4034                
                    PEA        $2142                
                    PEA        $6111                
                    PEA        $1111                
                    PEA        $6114                
                    PEA        $4324                
                    PEA        $2101                
                    TSC                             ; Line 25
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5413                
                    PEA        $1242                
                    PEA        $6466                
                    PEA        $1161                
                    PEA        $6614                
                    PEA        $3234                
                    PEA        $1213                
                    TSC                             ; Line 26
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B24                
                    PEA        $2113                
                    PEA        $146A                
                    PEA        $6666                
                    PEA        $4A21                
                    PEA        $2343                
                    PEA        $2331                
                    TSC                             ; Line 27
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5413                
                    PEA        $1233                
                    PEA        $41A1                
                    PEA        $AAAA                
                    PEA        $1432                
                    PEA        $1234                
                    PHY                             
                    TSC                             ; Line 28
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B24                
                    PEA        $2133                
                    PEA        $4113                
                    PEA        $AA43                
                    PEA        $1422                
                    PEA        $2140                
                    PEA        $2331                
                    TSC                             ; Line 29
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4413                
                    PEA        $0223                
                    PEA        $1334                
                    PEA        $3144                
                    PEA        $2112                
                    PEA        $1240                
                    PHY                             
                    TSC                             ; Line 30
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B24                
                    PEA        $0132                
                    PEA        $3211                
                    PEA        $4314                
                    PEA        $2221                
                    PEA        $2140                
                    PEA        $2331                
                    TSC                             ; Line 31
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5413                
                    PEA        $0223                
                    PEA        $1343                
                    PEA        $1421                
                    PHY                             
                    PEA        $6240                
                    PHY                             
                    TSC                             ; Line 32
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4024                
                    PEA        $0136                
                    PEA        $3144                
                    PEA        $2121                
                    PEA        $2121                
                    PEA        $6140                
                    PEA        $2331                
                    TSC                             ; Line 33
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5013                
                    PEA        $0260                
                    PEA        $6612                
                    PHY                             
                    PEA        $1262                
                    PEA        $1600                
                    PEA        $1403                
                    TSC                             ; Line 34
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4024                
                    PEA        $0121                
                    PEA        $6126                
                    PEA        $2121                
                    PEA        $6166                
                    PEA        $3100                
                    PEA        $2401                
                    TSC                             ; Line 35
                    ADC        #$00AE               
                    TCS                             
                    PEA        $A013                
                    PEA        $0213                
                    PEA        $1266                
                    PEA        $1662                
                    PEA        $6616                
                    PEA        $1203                
                    PEA        $1403                
                    TSC                             ; Line 36
                    ADC        #$00AE               
                    TCS                             
                    PEA        $A09A                
                    PEA        $3021                
                    PEA        $2121                
                    PEA        $6666                
                    PEA        $2621                
                    PEA        $2101                
                    PEA        $A90A                
                    TSC                             ; Line 37
                    ADC        #$00AE               
                    TCS                             
                    PEA        $A09A                
                    PEA        $1012                
                    PEA        $1262                
                    PEA        $1616                
                    PHY                             
                    PEA        $1203                
                    PEA        $A90A                
                    TSC                             ; Line 38
                    ADC        #$00AE               
                    TCS                             
                    PEA        $009A                
                    PEA        $3021                
                    PEA        $2161                
                    PEA        $2126                
                    PEA        $2121                
                    PEA        $2101                
                    PEA        $A900                
                    TSC                             ; Line 39
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0090                
                    PEA        $2012                
                    PEA        $2222                
                    PEA        $2121                
                    PEA        $2221                
                    PEA        $2101                
                    PEA        $0900                
                    TSC                             ; Line 40
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5022                
                    PEA        $1112                
                    PHY                             
                    PHY                             
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 41
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4023                
                    PEA        $2221                
                    PEA        $2121                
                    PEA        $2121                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 42
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5033                
                    PHY                             
                    PEA        $1211                
                    PHY                             
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 43
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4034                
                    PEA        $2321                
                    PEA        $3130                
                    PEA        $2121                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 44
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5034                
                    PEA        $3212                
                    PEA        $1240                
                    PEA        $2312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 45
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4044                
                    PEA        $2321                
                    PEA        $3130                
                    PEA        $2322                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 46
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5034                
                    PEA        $3412                
                    PEA        $1340                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 47
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4045                
                    PEA        $2321                
                    PEA        $0150                
                    PEA        $3423                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 48
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5044                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 49
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 50
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $03B5                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 51
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3B23                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 52
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $00B4                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 53
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 54
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 55
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 56
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0200                
                    PHX                             
                    TSC                             ; Line 57
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 58
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3312                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0200                
                    PHX                             
                    TSC                             ; Line 59
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $3331                
                    PEA        $0140                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 60
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0030                
                    PEA        $2312                
                    PEA        $0040                
                    PEA        $3312                
                    PEA        $0200                
                    PHX                             
                    TSC                             ; Line 61
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0001                
                    PEA        $01A1                
                    PEA        $9A01                
                    PEA        $0101                
                    PEA        $A9AA                
                    PEA        $0101                
                    PHX                             
                    TSC                             ; Line 62
                    ADC        #$00AE               
                    TCS                             
                    PHD                             
                    PEA        $10AA                
                    PEA        $9910                
                    PHD                             
                    PEA        $99A9                
                    PEA        $1A10                
                    PEA        $1000                
                    TSC                             ; Line 63
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0001                
                    PEA        $A199                
                    PEA        $9901                
                    PEA        $0101                
                    PEA        $9999                
                    PEA        $9909                
                    PEA        $0101                
                    TSC                             ; Line 64
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0010                
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PEA        $1000                
                    TSC                             ; Line 65
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHX                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHX                             
                    TSC                             ; Line 66
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0010                
                    PHD                             
                    PHD                             
                    PHD                             
                    PEA        $1000                
                    PHX                             
                    TSC                             ; Line 67
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHX                             
                    PHX                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHX                             
                    PHX                             
*--                                                 
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 00/
                    AND        #$FFCF               
                    STAL       $E1C068              
                    LDA        StackAddress         ; Restore Stack
                    TCS                             
                    PLD                             ; Restore Direct Page
                    CLI                             ; Enable Interrupts
                    RTS                             
                                                    
*-----------------------------------                                

SpriteMan1          CLC                             ; 28x68, 1445 bytes, 2754 cycles
                    SEI                             ; Disable Interrupts
                    PHD                             ; Backup Direct Page
                    TSC                             ; Backup Stack
                    STA        StackAddress         
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 01/
                    ORA        #$0030               
                    STAL       $E1C068              
                    TYA                             ; Y = Sprite Target Screen Address (upper left corner)
                    TCS                             ; New Stack address
                    LDX        #$0000               ; Pattern #1 : 249
                    LDY        #$1212               ; Pattern #2 : 35
                    LDA        #$2121               ; Pattern #3 : 29
                    TCD                             
*--                                                 
                    TSC                             
                    ADC        #$000D               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 1
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 2
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 3
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 4
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 5
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 6
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 7
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 8
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PEA        $1111                
                    PEA        $0100                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 9
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0011                
                    PEA        $1111                
                    PEA        $1101                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 10
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $1011                
                    PEA        $1111                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 11
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $1011                
                    PEA        $1111                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 12
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $6011                
                    PEA        $1111                
                    PEA        $1111                
                    PEA        $0600                
                    PHX                             
                    TSC                             ; Line 13
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $6016                
                    PEA        $1111                
                    PEA        $1166                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 14
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $1066                
                    PEA        $6666                
                    PEA        $6606                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 15
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $34A1                
                    PEA        $AAAA                
                    PEA        $AA0A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 16
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $B054                
                    PEA        $13AA                
                    PEA        $111A                
                    PEA        $111A                
                    PEA        $3101                
                    PHX                             
                    TSC                             ; Line 17
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0040                
                    PEA        $4B45                
                    PEA        $41A1                
                    PEA        $AAA9                
                    PEA        $AA11                
                    PHY                             
                    PEA        $0300                
                    TSC                             ; Line 18
                    ADC        #$00AE               
                    TCS                             
                    PEA        $00B4                
                    PEA        $4444                
                    PEA        $3114                
                    PEA        $1911                
                    PEA        $1914                
                    PHD                             
                    PEA        $3100                
                    TSC                             ; Line 19
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0045                
                    PEA        $4343                
                    PEA        $1343                
                    PEA        $9199                
                    PEA        $3114                
                    PHY                             
                    PEA        $1203                
                    TSC                             ; Line 20
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5034                
                    PEA        $3442                
                    PEA        $1333                
                    PEA        $1419                
                    PEA        $3114                
                    PEA        $2321                
                    PEA        $2101                
                    TSC                             ; Line 21
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4043                
                    PEA        $2321                
                    PEA        $3331                
                    PEA        $1419                
                    PEA        $4314                
                    PEA        $3324                
                    PEA        $1232                
                    TSC                             ; Line 22
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5034                
                    PEA        $1232                
                    PEA        $3213                
                    PEA        $4311                
                    PEA        $4321                
                    PEA        $3243                
                    PEA        $2221                
                    TSC                             ; Line 23
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4543                
                    PEA        $3131                
                    PEA        $3332                
                    PEA        $4131                
                    PEA        $4421                
                    PEA        $2324                
                    PEA        $1312                
                    TSC                             ; Line 24
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4434                
                    PEA        $1223                
                    PEA        $2323                
                    PEA        $1331                
                    PEA        $5421                
                    PEA        $3142                
                    PEA        $2231                
                    TSC                             ; Line 25
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B23                
                    PEA        $3130                
                    PEA        $3232                
                    PEA        $1243                
                    PEA        $1B22                
                    PEA        $1224                
                    PEA        $1312                
                    TSC                             ; Line 26
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4434                
                    PEA        $1220                
                    PEA        $2323                
                    PEA        $31B4                
                    PEA        $1422                
                    PEA        $3141                
                    PEA        $2231                
                    TSC                             ; Line 27
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B23                
                    PEA        $3130                
                    PEA        $3212                
                    PEA        $434B                
                    PEA        $2112                
                    PEA        $1230                
                    PEA        $1312                
                    TSC                             ; Line 28
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4434                
                    PEA        $1220                
                    PEA        $1331                
                    PEA        $4511                
                    PEA        $2221                
                    PEA        $3140                
                    PEA        $2231                
                    TSC                             ; Line 29
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B23                
                    PEA        $3110                
                    PEA        $3145                
                    PEA        $1122                
                    PHY                             
                    PEA        $1240                
                    PEA        $1312                
                    TSC                             ; Line 30
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4434                
                    PEA        $1240                
                    PEA        $4511                
                    PEA        $2222                
                    PHD                             
                    PEA        $3140                
                    PEA        $2431                
                    TSC                             ; Line 31
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4523                
                    PEA        $3110                
                    PEA        $1122                
                    PHY                             
                    PHY                             
                    PEA        $1240                
                    PEA        $1312                
                    TSC                             ; Line 32
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4034                
                    PEA        $1260                
                    PHD                             
                    PHD                             
                    PHD                             
                    PEA        $6100                
                    PEA        $2401                
                    TSC                             ; Line 33
                    ADC        #$00AE               
                    TCS                             
                    PEA        $009A                
                    PEA        $0010                
                    PEA        $6612                
                    PHY                             
                    PEA        $1262                
                    PEA        $1600                
                    PEA        $A00A                
                    TSC                             ; Line 34
                    ADC        #$00AE               
                    TCS                             
                    PEA        $00AA                
                    PEA        $0021                
                    PEA        $6126                
                    PHD                             
                    PEA        $6166                
                    PEA        $3100                
                    PEA        $AA0A                
                    TSC                             ; Line 35
                    ADC        #$00AE               
                    TCS                             
                    PEA        $00AA                
                    PEA        $0013                
                    PEA        $1266                
                    PEA        $2662                
                    PEA        $6616                
                    PEA        $1203                
                    PEA        $AA00                
                    TSC                             ; Line 36
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0099                
                    PEA        $3921                
                    PHD                             
                    PEA        $6666                
                    PEA        $2621                
                    PEA        $2131                
                    PEA        $9900                
                    TSC                             ; Line 37
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $1012                
                    PEA        $1262                
                    PEA        $1616                
                    PHY                             
                    PHY                             
                    PHX                             
                    TSC                             ; Line 38
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $3021                
                    PEA        $2161                
                    PEA        $2126                
                    PHD                             
                    PEA        $2131                
                    PHX                             
                    TSC                             ; Line 39
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHD                             
                    PEA        $2126                
                    PEA        $2116                
                    PHD                             
                    PEA        $2111                
                    PHX                             
                    TSC                             ; Line 40
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $1312                
                    PEA        $1216                
                    PEA        $2221                
                    PHY                             
                    PEA        $1233                
                    PHX                             
                    TSC                             ; Line 41
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $2021                
                    PHD                             
                    PEA        $2112                
                    PHD                             
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 42
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5012                
                    PHY                             
                    PEA        $1221                
                    PHY                             
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 43
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $2021                
                    PHD                             
                    PEA        $3110                
                    PEA        $2221                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 44
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5012                
                    PHY                             
                    PEA        $1220                
                    PEA        $1112                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 45
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4022                
                    PHD                             
                    PEA        $3130                
                    PEA        $2221                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 46
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5024                
                    PHY                             
                    PEA        $1340                
                    PEA        $2312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 47
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4045                
                    PEA        $2221                
                    PEA        $0150                
                    PEA        $3422                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 48
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5044                
                    PEA        $2412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 49
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 50
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $03B5                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 51
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3B23                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 52
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $00B4                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 53
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 54
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 55
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 56
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0200                
                    PHX                             
                    TSC                             ; Line 57
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 58
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3312                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0200                
                    PHX                             
                    TSC                             ; Line 59
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $3321                
                    PEA        $0140                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 60
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0030                
                    PEA        $2312                
                    PEA        $0040                
                    PEA        $3312                
                    PEA        $0200                
                    PHX                             
                    TSC                             ; Line 61
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0001                
                    PEA        $01A1                
                    PEA        $9A01                
                    PEA        $0101                
                    PEA        $A9AA                
                    PEA        $0101                
                    PHX                             
                    TSC                             ; Line 62
                    ADC        #$00AE               
                    TCS                             
                    PEA        $1010                
                    PEA        $10AA                
                    PEA        $9910                
                    PEA        $1010                
                    PEA        $99A9                
                    PEA        $1A10                
                    PEA        $1000                
                    TSC                             ; Line 63
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHA                             
                    PEA        $A199                
                    PEA        $9901                
                    PHA                             
                    PEA        $9999                
                    PEA        $9909                
                    PHA                             
                    TSC                             ; Line 64
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$1010               
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PEA        $1000                
                    TSC                             ; Line 65
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHX                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PEA        $0100                
                    TSC                             ; Line 66
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0010                
                    PEA        $1010                
                    PEA        $1010                
                    PEA        $1000                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 67
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
*--                                                 
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 00/
                    AND        #$FFCF               
                    STAL       $E1C068              
                    LDA        StackAddress         ; Restore Stack
                    TCS                             
                    PLD                             ; Restore Direct Page
                    CLI                             ; Enable Interrupts
                    RTS                             
                                                    
*-----------------------------------                                

SpriteMan2          CLC                             ; 28x68, 1512 bytes, 2786 cycles
                    SEI                             ; Disable Interrupts
                    PHD                             ; Backup Direct Page
                    TSC                             ; Backup Stack
                    STA        StackAddress         
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 01/
                    ORA        #$0030               
                    STAL       $E1C068              
                    TYA                             ; Y = Sprite Target Screen Address (upper left corner)
                    TCS                             ; New Stack address
                    LDX        #$0000               ; Pattern #1 : 222
                    LDY        #$1212               ; Pattern #2 : 26
                    LDA        #$1010               ; Pattern #3 : 20
                    TCD                             
*--                                                 
                    TSC                             
                    ADC        #$000D               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 1
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 2
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 3
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PEA        $1011                
                    PEA        $0100                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 4
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0010                
                    PEA        $1111                
                    PEA        $1101                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 5
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0011                
                    PEA        $1111                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 6
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0011                
                    PEA        $1111                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 7
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0016                
                    PEA        $1111                
                    PEA        $1161                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 8
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0066                
                    PEA        $6666                
                    PEA        $6666                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 9
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $00A1                
                    PEA        $6666                
                    PEA        $661A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 10
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $00A1                
                    PEA        $AAAA                
                    PEA        $AA1A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 11
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $00A0                
                    PEA        $111A                
                    PEA        $111A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 12
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $00A0                
                    PEA        $AAAA                
                    PEA        $AA0A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 13
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0021                
                    PEA        $1A11                
                    PEA        $AA00                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 14
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $1314                
                    PEA        $AAAA                
                    PEA        $AA23                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 15
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $3424                
                    PEA        $A1AA                
                    PEA        $1A21                
                    PEA        $2331                
                    PHX                             
                    TSC                             ; Line 16
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $404B                
                    PEA        $4541                
                    PEA        $A1AA                
                    PEA        $1A14                
                    PEA        $3212                
                    PEA        $0300                
                    TSC                             ; Line 17
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $B454                
                    PEA        $5441                
                    PEA        $A1AA                
                    PEA        $1A35                
                    PEA        $2123                
                    PEA        $3100                
                    TSC                             ; Line 18
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0040                
                    PEA        $3534                
                    PEA        $4541                
                    PEA        $13AA                
                    PEA        $1134                
                    PEA        $3132                
                    PEA        $1200                
                    TSC                             ; Line 19
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0054                
                    PEA        $4343                
                    PEA        $4441                
                    PEA        $13AA                
                    PEA        $1135                
                    PEA        $4123                
                    PEA        $3101                
                    TSC                             ; Line 20
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0045                
                    PEA        $3443                
                    PEA        $4414                
                    PEA        $141A                
                    PEA        $5113                
                    PEA        $3334                
                    PEA        $1203                
                    TSC                             ; Line 21
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5044                
                    PEA        $3342                
                    PEA        $4414                
                    PEA        $1411                
                    PEA        $4113                
                    PEA        $4343                
                    PEA        $3332                
                    TSC                             ; Line 22
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4035                
                    PEA        $1323                
                    PEA        $4414                
                    PEA        $3411                
                    PEA        $3B31                
                    PEA        $3234                
                    PEA        $3421                
                    TSC                             ; Line 23
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5034                
                    PEA        $3141                
                    PEA        $4443                
                    PEA        $4113                
                    PEA        $3421                
                    PEA        $2343                
                    PEA        $3432                
                    TSC                             ; Line 24
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B23                
                    PEA        $1334                
                    PEA        $3434                
                    PEA        $41B1                
                    PEA        $1333                
                    PEA        $3234                
                    PEA        $2421                
                    TSC                             ; Line 25
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5434                
                    PEA        $3140                
                    PEA        $4332                
                    PEA        $123B                
                    PEA        $3123                
                    PEA        $2343                
                    PEA        $3432                
                    TSC                             ; Line 26
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B23                
                    PEA        $1230                
                    PEA        $2423                
                    PEA        $B113                
                    PEA        $2323                
                    PEA        $1234                
                    PEA        $2421                
                    TSC                             ; Line 27
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5434                
                    PEA        $3140                
                    PEA        $3212                
                    PEA        $3B31                
                    PEA        $3222                
                    PEA        $3141                
                    PEA        $3432                
                    TSC                             ; Line 28
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4B23                
                    PEA        $1220                
                    PEA        $2351                
                    PEA        $1323                
                    PEA        $2212                
                    PEA        $1230                
                    PEA        $4421                
                    TSC                             ; Line 29
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5434                
                    PEA        $3130                
                    PEA        $1235                
                    PEA        $2122                
                    PEA        $2221                
                    PEA        $3140                
                    PEA        $3432                
                    TSC                             ; Line 30
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4523                
                    PEA        $1220                
                    PEA        $1234                
                    PEA        $2112                
                    PHY                             
                    PEA        $1230                
                    PEA        $4421                
                    TSC                             ; Line 31
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5434                
                    PEA        $3120                
                    PEA        $5113                
                    PEA        $2221                
                    PEA        $2121                
                    PEA        $3100                
                    PEA        $3432                
                    TSC                             ; Line 32
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4523                
                    PEA        $1220                
                    PEA        $3521                
                    PHY                             
                    PHY                             
                    PEA        $1300                
                    PEA        $4421                
                    TSC                             ; Line 33
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4034                
                    PEA        $3160                
                    PEA        $3321                
                    PEA        $2121                
                    PEA        $2121                
                    PEA        $6106                
                    PEA        $3432                
                    TSC                             ; Line 34
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4013                
                    PEA        $1230                
                    PEA        $6613                
                    PHY                             
                    PEA        $1262                
                    PEA        $6603                
                    PEA        $4421                
                    TSC                             ; Line 35
                    ADC        #$00AE               
                    TCS                             
                    PEA        $000A                
                    PEA        $0010                
                    PEA        $6266                
                    PEA        $2222                
                    PEA        $6666                
                    PEA        $2601                
                    PEA        $A00A                
                    TSC                             ; Line 36
                    ADC        #$00AE               
                    TCS                             
                    PEA        $009A                
                    PEA        $0031                
                    PEA        $2161                
                    PEA        $6666                
                    PEA        $6612                
                    PEA        $1213                
                    PEA        $A000                
                    TSC                             ; Line 37
                    ADC        #$00AE               
                    TCS                             
                    PEA        $00AA                
                    PEA        $0013                
                    PHY                             
                    PEA        $6621                
                    PEA        $2121                
                    PEA        $2131                
                    PEA        $AA00                
                    TSC                             ; Line 38
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0099                
                    PEA        $0921                
                    PEA        $2161                
                    PEA        $6616                
                    PHY                             
                    PHY                             
                    PEA        $AA00                
                    TSC                             ; Line 39
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $2021                
                    PEA        $2126                
                    PEA        $2116                
                    PEA        $2221                
                    PEA        $2131                
                    PHX                             
                    TSC                             ; Line 40
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $1012                
                    PEA        $1216                
                    PEA        $1222                
                    PEA        $1112                
                    PEA        $1213                
                    PHX                             
                    TSC                             ; Line 41
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4021                
                    PEA        $2126                
                    PEA        $2123                
                    PEA        $2221                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 42
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5014                
                    PHY                             
                    PEA        $1232                
                    PEA        $2212                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 43
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4044                
                    PEA        $2121                
                    PEA        $3130                
                    PEA        $2321                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 44
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5034                
                    PEA        $1412                
                    PEA        $1240                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 45
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4044                
                    PEA        $2321                
                    PEA        $3130                
                    PEA        $2322                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 46
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5034                
                    PEA        $3412                
                    PEA        $1340                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 47
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4045                
                    PEA        $2321                
                    PEA        $0150                
                    PEA        $3423                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 48
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5044                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 49
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 50
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $03B5                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 51
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3B23                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 52
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $00B4                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 53
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 54
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 55
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 56
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 57
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 58
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3312                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 59
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $3321                
                    PEA        $0140                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 60
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0030                
                    PEA        $2312                
                    PEA        $0040                
                    PEA        $3312                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 61
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0001                
                    PEA        $01A1                
                    PEA        $9A01                
                    PEA        $0101                
                    PEA        $A9AA                
                    PEA        $0101                
                    PHX                             
                    TSC                             ; Line 62
                    ADC        #$00AE               
                    TCS                             
                    PHD                             
                    PEA        $10AA                
                    PEA        $9910                
                    PHD                             
                    PEA        $99A9                
                    PEA        $1A10                
                    PEA        $1000                
                    TSC                             ; Line 63
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHA                             
                    PEA        $A199                
                    PEA        $9901                
                    PHA                             
                    PEA        $9999                
                    PEA        $9909                
                    PHA                             
                    TSC                             ; Line 64
                    ADC        #$00AE               
                    TCS                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PEA        $1000                
                    TSC                             ; Line 65
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHX                             
                    PEA        $0001                
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHX                             
                    TSC                             ; Line 66
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0010                
                    PHD                             
                    PEA        $1000                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 67
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
*--                                                 
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 00/
                    AND        #$FFCF               
                    STAL       $E1C068              
                    LDA        StackAddress         ; Restore Stack
                    TCS                             
                    PLD                             ; Restore Direct Page
                    CLI                             ; Enable Interrupts
                    RTS                             
                                                    
*-----------------------------------                                

SpriteMan3          CLC                             ; 28x68, 1534 bytes, 2797 cycles
                    SEI                             ; Disable Interrupts
                    PHD                             ; Backup Direct Page
                    TSC                             ; Backup Stack
                    STA        StackAddress         
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 01/
                    ORA        #$0030               
                    STAL       $E1C068              
                    TYA                             ; Y = Sprite Target Screen Address (upper left corner)
                    TCS                             ; New Stack address
                    LDX        #$0000               ; Pattern #1 : 210
                    LDY        #$1212               ; Pattern #2 : 20
                    LDA        #$1010               ; Pattern #3 : 17
                    TCD                             
*--                                                 
                    TSC                             
                    ADC        #$000D               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PEA        $1011                
                    PEA        $0100                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 1
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0010                
                    PEA        $1111                
                    PEA        $1101                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 2
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0011                
                    PEA        $1111                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 3
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0011                
                    PEA        $1111                
                    PEA        $1111                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 4
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0066                
                    PEA        $6666                
                    PEA        $6666                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 5
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0066                
                    PEA        $6666                
                    PEA        $6666                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 6
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0011                
                    PEA        $AAAA                
                    PEA        $AA11                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 7
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $00A1                
                    PEA        $111A                
                    PEA        $111A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 8
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $00A1                
                    PEA        $AA9A                
                    PEA        $AA1A                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 9
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0010                
                    PEA        $AA9A                
                    PEA        $AA01                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 10
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PEA        $AAAA                
                    PEA        $AA00                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 11
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PEA        $1A11                
                    PEA        $AA00                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 12
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PEA        $AAAA                
                    PEA        $AA00                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 13
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PEA        $0033                
                    PEA        $A1AA                
                    PEA        $1A04                
                    PHX                             
                    PHX                             
                    TSC                             ; Line 14
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0050                
                    PEA        $1435                
                    PEA        $A1AA                
                    PEA        $1A24                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 15
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $404B                
                    PEA        $1434                
                    PEA        $A1AA                
                    PEA        $3124                
                    PEA        $2103                
                    PHX                             
                    TSC                             ; Line 16
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $B444                
                    PEA        $1535                
                    PEA        $A1AA                
                    PEA        $3124                
                    PEA        $3131                
                    PHX                             
                    TSC                             ; Line 17
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0040                
                    PEA        $4553                
                    PEA        $1434                
                    PEA        $A1AA                
                    PEA        $B124                
                    PEA        $3112                
                    PEA        $0300                
                    TSC                             ; Line 18
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0050                
                    PEA        $3444                
                    PEA        $4451                
                    PEA        $131A                
                    PEA        $4B13                
                    PEA        $4333                
                    PEA        $3100                
                    TSC                             ; Line 19
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0044                
                    PEA        $3342                
                    PEA        $4441                
                    PEA        $131A                
                    PEA        $4412                
                    PEA        $3334                
                    PEA        $1203                
                    TSC                             ; Line 20
                    ADC        #$00AE               
                    TCS                             
                    PEA        $3034                
                    PEA        $2441                
                    PEA        $4413                
                    PEA        $35BA                
                    PEA        $2431                
                    PEA        $4333                
                    PEA        $2301                
                    TSC                             ; Line 21
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4044                
                    PEA        $1243                
                    PEA        $3414                
                    PEA        $3441                
                    PEA        $2431                
                    PEA        $3234                
                    PEA        $3303                
                    TSC                             ; Line 22
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4034                
                    PEA        $2241                
                    PEA        $4433                
                    PEA        $114B                
                    PEA        $1223                
                    PEA        $3243                
                    PEA        $2301                
                    TSC                             ; Line 23
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4043                
                    PEA        $1242                
                    PEA        $3434                
                    PEA        $B124                
                    PEA        $3123                
                    PEA        $3144                
                    PEA        $3403                
                    TSC                             ; Line 24
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5034                
                    PEA        $2141                
                    PEA        $4313                
                    PEA        $4512                
                    PEA        $3312                
                    PEA        $3243                
                    PEA        $2331                
                    TSC                             ; Line 25
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4023                
                    PEA        $1233                
                    PEA        $3413                
                    PEA        $4512                
                    PEA        $2323                
                    PEA        $2143                
                    PEA        $3412                
                    TSC                             ; Line 26
                    ADC        #$00AE               
                    TCS                             
                    PEA        $B034                
                    PEA        $3140                
                    PEA        $3351                
                    PEA        $2431                
                    PEA        $3212                
                    PEA        $1243                
                    PEA        $2331                
                    TSC                             ; Line 27
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4023                
                    PEA        $1230                
                    PEA        $3451                
                    PEA        $2431                
                    PEA        $2321                
                    PEA        $3150                
                    PEA        $3412                
                    TSC                             ; Line 28
                    ADC        #$00AE               
                    TCS                             
                    PEA        $B034                
                    PEA        $3140                
                    PEA        $1345                
                    PEA        $1233                
                    PEA        $3212                
                    PEA        $1240                
                    PEA        $2331                
                    TSC                             ; Line 29
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4023                
                    PEA        $1230                
                    PEA        $1345                
                    PEA        $1223                
                    PEA        $2321                
                    PEA        $3150                
                    PEA        $3412                
                    TSC                             ; Line 30
                    ADC        #$00AE               
                    TCS                             
                    PEA        $5034                
                    PEA        $3140                
                    PEA        $5124                
                    PEA        $3133                
                    PHY                             
                    PEA        $1240                
                    PEA        $2331                
                    TSC                             ; Line 31
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4023                
                    PEA        $1230                
                    PEA        $5124                
                    PEA        $3123                
                    PEA        $2321                
                    PEA        $3150                
                    PEA        $3413                
                    TSC                             ; Line 32
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4034                
                    PEA        $3140                
                    PEA        $5124                
                    PEA        $3132                
                    PHY                             
                    PEA        $1240                
                    PEA        $2301                
                    TSC                             ; Line 33
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4023                
                    PEA        $1260                
                    PEA        $4411                
                    PEA        $2323                
                    PEA        $2121                
                    PEA        $6140                
                    PEA        $3403                
                    TSC                             ; Line 34
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4034                
                    PEA        $3160                
                    PEA        $6633                
                    PEA        $3212                
                    PHY                             
                    PEA        $6646                
                    PEA        $2301                
                    TSC                             ; Line 35
                    ADC        #$00AE               
                    TCS                             
                    PEA        $4023                
                    PEA        $1240                
                    PEA        $6666                
                    PEA        $3266                
                    PEA        $6666                
                    PEA        $3640                
                    PEA        $3403                
                    TSC                             ; Line 36
                    ADC        #$00AE               
                    TCS                             
                    PEA        $A0A9                
                    PEA        $0044                
                    PEA        $2366                
                    PEA        $6666                
                    PEA        $6626                
                    PEA        $2103                
                    PEA        $A009                
                    TSC                             ; Line 37
                    ADC        #$00AE               
                    TCS                             
                    PEA        $A0A9                
                    PEA        $0054                
                    PEA        $3462                
                    PEA        $6236                
                    PHY                             
                    PEA        $1201                
                    PEA        $A009                
                    TSC                             ; Line 38
                    ADC        #$00AE               
                    TCS                             
                    PEA        $A09A                
                    PEA        $4A45                
                    PEA        $2362                
                    PEA        $6236                
                    PEA        $2221                
                    PEA        $2103                
                    PEA        $AA09                
                    TSC                             ; Line 39
                    ADC        #$00AE               
                    TCS                             
                    PEA        $00AA                
                    PEA        $4A44                
                    PEA        $2363                
                    PEA        $2136                
                    PEA        $2321                
                    PEA        $2101                
                    PEA        $9A0A                
                    TSC                             ; Line 40
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4044                
                    PEA        $1216                
                    PEA        $1222                
                    PEA        $1112                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 41
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4035                
                    PEA        $2126                
                    PEA        $2123                
                    PEA        $2221                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 42
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5054                
                    PEA        $1412                
                    PEA        $1232                
                    PEA        $2212                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 43
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4044                
                    PEA        $2121                
                    PEA        $3130                
                    PEA        $2321                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 44
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5034                
                    PEA        $1412                
                    PEA        $1240                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 45
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4044                
                    PEA        $2321                
                    PEA        $3130                
                    PEA        $2322                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 46
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5034                
                    PEA        $3412                
                    PEA        $1340                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 47
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $4045                
                    PEA        $2321                
                    PEA        $0150                
                    PEA        $3423                
                    PEA        $2101                
                    PHX                             
                    TSC                             ; Line 48
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $5044                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1203                
                    PHX                             
                    TSC                             ; Line 49
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 50
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $03B5                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 51
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $004B                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3B23                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 52
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $00B4                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 53
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 54
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $1200                
                    PHX                             
                    TSC                             ; Line 55
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $3100                
                    PHX                             
                    TSC                             ; Line 56
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3412                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 57
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0044                
                    PEA        $2321                
                    PEA        $0144                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 58
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0034                
                    PEA        $3312                
                    PEA        $0345                
                    PEA        $3312                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 59
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0040                
                    PEA        $3321                
                    PEA        $0140                
                    PEA        $3423                
                    PEA        $0100                
                    PHX                             
                    TSC                             ; Line 60
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PEA        $0030                
                    PEA        $2312                
                    PEA        $0040                
                    PEA        $3312                
                    PEA        $0300                
                    PHX                             
                    TSC                             ; Line 61
                    ADC        #$00AE               
                    TCS                             
                    PEA        $0001                
                    PEA        $01A1                
                    PEA        $9A01                
                    PEA        $0101                
                    PEA        $A9AA                
                    PEA        $0101                
                    PHX                             
                    TSC                             ; Line 62
                    ADC        #$00AE               
                    TCS                             
                    PHD                             
                    PEA        $10AA                
                    PEA        $9910                
                    PHD                             
                    PEA        $99A9                
                    PEA        $1A10                
                    PEA        $1000                
                    TSC                             ; Line 63
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHA                             
                    PEA        $A199                
                    PEA        $9901                
                    PHA                             
                    PEA        $9999                
                    PEA        $9909                
                    PHA                             
                    TSC                             ; Line 64
                    ADC        #$00AE               
                    TCS                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PHD                             
                    PEA        $1000                
                    TSC                             ; Line 65
                    ADC        #$00AE               
                    TCS                             
                    LDA        #$0101               
                    PHX                             
                    PEA        $0001                
                    PHA                             
                    PHA                             
                    PHA                             
                    PHA                             
                    PHX                             
                    TSC                             ; Line 66
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    TSC                             ; Line 67
                    ADC        #$00AE               
                    TCS                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
                    PHX                             
*--                                                 
                    LDAL       $E1C068              ; Direct Page and Stack in Bank 00/
                    AND        #$FFCF               
                    STAL       $E1C068              
                    LDA        StackAddress         ; Restore Stack
                    TCS                             
                    PLD                             ; Restore Direct Page
                    CLI                             ; Enable Interrupts
                    RTS                       


***************************************************************
*  HighScoreScreen() :  Display High Score Table (IN:A=Mode). *
***************************************************************
HighScoreScreen     STA        HSS_Mode
                    JSL        ClearKDBStrobe       ; Clear Keyboard
                    JSL        FadeOut              ; Fadout to Black
                    LDA        #$0000
                    JSL        SetBorderColor
*----
                    LDX        #$1FFE               ; RAZ Graphic Page
                    LDA        #$000
HSS_RAZ_GP          STAL       $012000,X
                    STAL       $014000,X
                    STAL       $016000,X
                    STAL       $018000,X
                    DEX
                    DEX
                    BPL        HSS_RAZ_GP                    
*--
HSS_SetPalette      LDX        #$005E               ; Copy Palettes
HSS_SetPalette_1    LDA        HSS_Palette_1,X
                    STA        Buffer_Palette,X        
                    DEX
                    DEX
                    BPL        HSS_SetPalette_1
*--
                    LDY        #$0018               ; Install SCB
HSS_SCB             LDA        HSS_TabSCBLine,Y
                    TAX
                    LDA        #$0000
                    STAL       $019D00,X
                    LDA        #$0101
                    STAL       $019D02,X
                    LDA        #$0201
                    STAL       $019D04,X   
                    LDA        #$0202
                    STAL       $019D06,X   
                    DEY
                    DEY
                    BPL        HSS_SCB
*----
HSS_Install         LDA        #HSS_FileScore      ; Install File Score information
                    STA        HSS_Install_2+1
                    LDA        #HSS_Line_01
                    STA        HSS_Install_3+1
                    LDX        #$0000
*-
HSS_Install_1       SEP        #$20
                    LDY        #$001A
HSS_Install_2       LDA        HSS_FileScore,Y     ; File Score Table
HSS_Install_3       STA        HSS_Line_01,Y       ; Line Score Table
                    DEY
                    BPL        HSS_Install_2
                    REP        #$30
*-
                    LDA        HSS_Install_2+1
                    CLC
                    ADC        #$001B              ; 27
                    STA        HSS_Install_2+1
                    LDA        HSS_Install_3+1
                    CLC
                    ADC        #$001C              ; 28
                    STA        HSS_Install_3+1
                    INX
                    CPX        #$000A
                    BNE        HSS_Install_1 
*----
                    LDX        #$0016              ; Display Strings
HSS_LineLoop        PHX
                    LDA        HSS_Tab_Line,X
                    STA        HSS_LineLoop_1+1
                    LDA        HSS_Tab_Address,X
                    STA        HSS_LineLoop_2+1
HSS_LineLoop_1      LDX        #$AAAA              ; @String
HSS_LineLoop_2      LDY        #$BBBB              ; @Screen
                    JSL        DisplayString
                    PLX
                    DEX
                    DEX
                    BPL        HSS_LineLoop
*--
                    LDA        HSS_Mode            ; Display Enter Name String
                    BEQ        HSS_FadeIn
                    LDX        #HSS_Enter_Name
                    LDY        #$8EAD
                    JSL        DisplayString
*--
HSS_FadeIn          LDA        #Buffer_Palette
                    JSL        FadeIN
*--
                    LDA        HSS_Mode
                    BNE        HSS_LineLoop_End
                    JSL        WaitForKey          ; Wait for Key Pressed
*--
HSS_LineLoop_End    RTS

*----------   HighScoreScreen Variables   ---------
HSS_Mode            HEX        0000                ; 0=Display, 1=Edit 
HSS_Palette_1       HEX        0000,5107,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,3004
HSS_Palette_2       HEX        0000,5107,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,4005
HSS_Palette_3       HEX        0000,5107,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,0000,5107
HSS_Tab_Line        DA         HSS_Title,HSS_Column,HSS_Line_01,HSS_Line_02,HSS_Line_03,HSS_Line_04,HSS_Line_05
                    DA         HSS_Line_06,HSS_Line_07,HSS_Line_08,HSS_Line_09,HSS_Line_10
HSS_Tab_Address     HEX        7521,952C,5535,753D,9545,B54D,D555,F55D,1566,356E,5576,757E
HSS_TabSCBLine      HEX        0200,1400,2200,2F00,3C00,4900,5600,6300,7000,7D00,8A00,9700,B100
HSS_Title           ASC        'MASTER WARRIORS',00
HSS_Column          ASC        '  NAME     LEVEL     SCORE ',00
HSS_Line_01         ASC        '                           ',00
HSS_Line_02         ASC        '                           ',00
HSS_Line_03         ASC        '                           ',00
HSS_Line_04         ASC        '                           ',00
HSS_Line_05         ASC        '                           ',00
HSS_Line_06         ASC        '                           ',00
HSS_Line_07         ASC        '                           ',00
HSS_Line_08         ASC        '                           ',00
HSS_Line_09         ASC        '                           ',00
HSS_Line_10         ASC        '                           ',00
HSS_Enter_Name      ASC        'ENTER YOUR NAME AND PRESS ENTER',00
HSS_HighScoreTab    HEX        0000,0000,0000,0000,0000,0000,0000,0000,0000,0000
HSS_FileScoreOffset HEX        0000,1B00,3600,5100,6C00,8700,A200,BD00,D800,F300
HSS_FileScore       DS         270                 ; High Score File


*****************************************************************************************************
*  NewHighScoreScreen() :  Check/Insert a new High Score in the Table (IN:A=@Score,IN:X=BankScore). *
*****************************************************************************************************
NewHighScoreScreen  ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    JSR        CompactHexaScore
                    CMP        HSS_HighScoreTab+18  ; Score #10
                    BGE        NHSS_Begin
                    JMP        NHSS_End
*----
NHSS_Begin          STA        NHSS_ScoreCompact
*----
                    LDX        #$0000               ; Search Insert Point
NHSS_Insert_1       CMP        HSS_HighScoreTab,X
                    BGE        NHSS_Insert_2
                    INX
                    INX
                    CPX        #$0014
                    BNE        NHSS_Insert_1
                    LDX        #$0012               ; Last Position
NHSS_Insert_2       STX        NHSS_InsertPoint
                    CPX        #$0012
                    BEQ        NHSS_Clear
*----
NHSS_Move_Score     LDY        #$0010               ; Move Score Down
NHSS_Move_Score_1   LDA        HSS_HighScoreTab,Y
                    STA        HSS_HighScoreTab+2,Y             
                    CPY        NHSS_InsertPoint
                    BEQ        NHSS_Move_Line
                    DEY
                    DEY
                    BRA        NHSS_Move_Score_1
*--
NHSS_Move_Line      LDY        #$0010               ; Mode Line Down
NHSS_Move_Line_1    LDA        HSS_FileScoreOffset,Y
                    TAX 
                    CLC
                    ADC        #HSS_FileScore
                    STA        NHSS_Move_Line_2+1  ; Source
                    CLC
                    ADC        #$001B
                    STA        NHSS_Move_Line_3+1  ; Destination
*-
                    SEP        #$20                ; Move 27 Bytes down
                    LDX        #$001A       
NHSS_Move_Line_2    LDA        $AAAA,X
NHSS_Move_Line_3    STA        $BBBB,X
                    DEX
                    BPL        NHSS_Move_Line_2
                    REP        #$30
*-
                    CPY        NHSS_InsertPoint
                    BEQ        NHSS_Clear
                    DEY
                    DEY
                    BRA        NHSS_Move_Line_1
*----
NHSS_Clear          LDX        NHSS_InsertPoint     ; Clear Line
                    LDA        HSS_FileScoreOffset,X
                    TAY
                    SEP        #$20
                    LDX        #$0000
                    LDA        #$20                 ; Space
NHSS_Clear_1        STA        HSS_FileScore,Y
                    INY
                    INX
                    CPX        #$001B
                    BNE        NHSS_Clear_1
                    REP        #$30
*----
NHSS_SetScore       LDX        NHSS_InsertPoint     ; Set New Score
                    LDA        NHSS_ScoreCompact
                    STA        HSS_HighScoreTab,X   ; Compact
                    LDA        HSS_FileScoreOffset,X ; Ascii
                    TAY
                    SEP        #$20
                    LDAL       PG_Score
                    CLC
                    ADC        #$30
                    STA        HSS_FileScore+21,Y
                    LDAL       PG_Score+2
                    CLC
                    ADC        #$30
                    STA        HSS_FileScore+22,Y
                    LDAL       PG_Score+4
                    CLC
                    ADC        #$30
                    STA        HSS_FileScore+23,Y
                    LDAL       PG_Score+6
                    CLC
                    ADC        #$30
                    STA        HSS_FileScore+24,Y
                    LDA        #$30                 ; Last two are always '0'
                    STA        HSS_FileScore+25,Y
                    LDA        #$30
                    STA        HSS_FileScore+26,Y     
*----
NHSS_SetLevel       STZ        NHSS_Dozen           ; Set New Level
                    LDAL       PG_Level            
NHSS_CheckDozen     CMP        #$0A                 ; Hexa -> Dec
                    BLT        NHSS_StoreUnit
                    SEC
                    SBC        #$0A
                    INC        NHSS_Dozen
                    BRA        NHSS_CheckDozen
NHSS_StoreUnit      STA        NHSS_Unit
                    CLC
                    ADC        #$30                 ; '0'
                    STA        HSS_FileScore+15,Y
                    LDA        NHSS_Dozen
                    CLC  
                    ADC        #$30                 ; '0'
                    CMP        #$30
                    BNE        NHSS_StoreUnit_1
                    LDA        #$20                 ; Space
NHSS_StoreUnit_1    STA        HSS_FileScore+14,Y 
                    REP        #$30 
*----
                    LDA        #$0001               ; Display High Score Page on Screen
                    JSR        HighScoreScreen
*----
                    LDA        #$2020               ; Reset Name
                    STA        NHSS_CurrentName+0
                    STA        NHSS_CurrentName+2
                    STA        NHSS_CurrentName+4
                    STA        NHSS_CurrentName+6
                    STA        NHSS_CurrentName+8
                    STZ        NHSS_InsertColumn
NHSS_EnterName      REP        #$30
                    LDX        #NHSS_CurrentName    ; Display Current Name
                    LDY        NHSS_InsertPoint
                    LDA        HSS_Tab_Address+4,Y
                    TAY
                    JSL        DisplayString
*--                 
                    LDX        NHSS_InsertPoint     ; Display Cursor
                    LDA        HSS_Tab_Address+4,X
                    CLC
                    ADC        NHSS_InsertColumn    ; 8 pixels width
                    ADC        NHSS_InsertColumn
                    ADC        NHSS_InsertColumn
                    ADC        NHSS_InsertColumn                                        
                    TAX
                    LDA        #$1111
                    STAL       $010000,X
                    STAL       $0100A0,X
                    STAL       $010140,X
                    STAL       $0101E0,X
                    STAL       $010280,X
                    STAL       $010320,X
                    STAL       $0103C0,X
                    STAL       $010460,X
                    LDA        #$1011
                    STAL       $010002,X
                    STAL       $0100A2,X
                    STAL       $010142,X
                    STAL       $0101E2,X
                    STAL       $010282,X
                    STAL       $010322,X
                    STAL       $0103C2,X
                    STAL       $010462,X
*----             
NHSS_WaitForKey     REP        #$30                 ; Wait for Key
                    JSL        WaitForKey
                    SEP        #$20
                    CMP        #$0D                 ; Enter
                    BEQ        NHSS_Validate
                    CMP        #$7F                 ; Delete
                    BEQ        NHSS_Delete
                    CMP        #$61                 ; toupper
                    BLT        NHSS_StoreChar
                    CMP        #$7B 
                    BGE        NHSS_StoreChar
                    SEC
                    SBC        #$20                 ; 'a-z' -> 'A-Z'
NHSS_StoreChar      STA        NHSS_CurrentChar
                    JSR        IsValidChar          ; Is Valid Char ?
                    BCS        NHSS_WaitForKey
                    LDX        NHSS_InsertColumn
                    CPX        #$000A               ; 10 Char Max
                    BGE        NHSS_WaitForKey
                    LDA        NHSS_CurrentChar
                    STA        NHSS_CurrentName,X
                    INC        NHSS_InsertColumn
                    JMP        NHSS_EnterName
*----
NHSS_Delete         LDA        NHSS_InsertColumn    ; Delete Previous char
                    BEQ        NHSS_WaitForKey
                    LDX        NHSS_InsertColumn
                    CPX        #$0000
                    BEQ        NHSS_Delete_1
                    DEX
                    STX        NHSS_InsertColumn
                    LDA        #$20                 ; Space
                    STA        NHSS_CurrentName,X
NHSS_Delete_1       JMP        NHSS_EnterName
*----
NHSS_Validate       REP        #$30                 ; Store Name into FileScore
                    LDX        NHSS_InsertPoint
                    LDA        HSS_FileScoreOffset,X
                    TAX
                    LDA        NHSS_CurrentName+0
                    STA        HSS_FileScore+0,X
                    LDA        NHSS_CurrentName+2
                    STA        HSS_FileScore+2,X
                    LDA        NHSS_CurrentName+4
                    STA        HSS_FileScore+4,X
                    LDA        NHSS_CurrentName+6
                    STA        HSS_FileScore+6,X
                    LDA        NHSS_CurrentName+8
                    STA        HSS_FileScore+8,X
*----
SaveFileScore       JSR        SaveHighScoreTable   ; Save Score Table to disk
*----
NHSS_End            PLB
                    RTL
                    
*----------   NewHighScoreScreen Variables   ---------  
NHSS_Dozen          HEX        00
NHSS_Unit           HEX        00
NHSS_ScoreCompact   HEX        0000
NHSS_InsertPoint    HEX        0000                 ; Line Insert x2
NHSS_InsertColumn   HEX        0000                 ; Column Insert 0-9
NHSS_CurrentChar    HEX        00                   ; Current char
NHSS_CurrentName    ASC        '           ',00     ; Current Name


*************************************************************
*  LoadHighScoreTable() :  Load High Score Table from Disk. *
*************************************************************
LoadHighScoreTable  LDX        #LHST_FilePath       ; ScoreTable.bin
                    LDA        LoadUnpackBank
                    JSL        LoadFile
                    STA        LHST_FileSize
                    BCS        LHST_Default
*----
                    CMP        #$010E               ; File Size should be 270 bytes
                    BNE        LHST_Default
*--
                    LDA        LoadUnpackBank
                    STA        LHST_Check+2
                    LDX        #$010C               ; Copy File Score Table
LHST_Check          LDAL       $AA0000,X
                    STA        HSS_FileScore,X
                    DEX
                    DEX
                    BPL        LHST_Check
*--
                    SEP        #$20
                    LDX        #$010D               ; Check Character Set : [A-Z] [0-9] . Space
LHST_Check_1        LDA        HSS_FileScore,X
                    JSR        IsValidChar
                    BCS        LHST_Default         ; Wrong Character
LHST_Check_3        DEX
                    BPL        LHST_Check_1           
*--
                    LDX        #$0000               ; Check Score Character Set : [0-9]
LHST_Check_4        LDA        HSS_FileScore+21,X 
                    JSR        IsValidNum
                    BCS        LHST_Default         ; Wrong Character
                    LDA        HSS_FileScore+22,X
                    JSR        IsValidNum
                    BCS        LHST_Default         ; Wrong Character
                    LDA        HSS_FileScore+23,X
                    JSR        IsValidNum
                    BCS        LHST_Default         ; Wrong Character
                    LDA        HSS_FileScore+24,X
                    JSR        IsValidNum
                    BCS        LHST_Default         ; Wrong Character
                    REP        #$30
                    TXA
                    CLC
                    ADC        #$001B               ; 27
                    TAX
                    SEP        #$20
                    CPX        #$010E
                    BNE        LHST_Check_4
                    REP        #$30 
                    BRA        LHST_Score
*----
LHST_Default        REP        #$30                 ; Copy Default Score Table
                    LDX        #$010C
LHST_Default_1      LDA        LHST_DefaultScore,X
                    STA        HSS_FileScore,X
                    DEX
                    DEX
                    BPL        LHST_Default_1
*----
LHST_Score          LDA        #HSS_FileScore       ; Decode/Compact High Score
                    CLC
                    ADC        #$0015               ; Score @ 4 Ascii 0-9 values
                    TAX
                    LDY        #$0000
LHST_Score_1        TXA
                    JSR        CompactAsciiScore
                    STA        HSS_HighScoreTab,Y
                    TXA
                    CLC
                    ADC        #$001B
                    TAX
                    INY
                    INY
                    CPY        #$0014
                    BNE        LHST_Score_1
*--
LHST_End            RTS

*----------   LoadHighScoreTable Variables   ---------  
LHST_FilePath       STRL       '1/Data/ScoreTable.bin'
LHST_FileSize       HEX        0000
LHST_DefaultScore   ASC        'ANTOINE       10     060000'
                    ASC        'OLIVIER        9     050000'
                    ASC        'ERIC           8     040000'
                    ASC        'ANNE           7     030000'
                    ASC        'LEE            6     025000'
                    ASC        'KYLE           5     020000'
                    ASC        'MIKE           4     018000'
                    ASC        'STEEVE         3     012000'
                    ASC        'DAVID          2     008000'
                    ASC        'JANKO          1     004000'


***********************************************************
*  SaveHighScoreTable() :  Save High Score Table to Disk. *
***********************************************************
SaveHighScoreTable  JSL        GSOS                 ; Delete File
                    dw         $2002                ; Code for command 'Destroy'
                    adrl       SHST_gsosDESTROY     ; Parameters Table Address   
*--                    
                    SEP        #$20
                    LDA        #$E3
                    STA        SHST_gsosCREATE+6    ; File Access
                    LDA        #$06
                    STA        SHST_gsosCREATE+8    ; File Type (Binary)
                    LDA        #$01
                    STA        SHST_gsosCREATE+14   ; Storage Type (Standard)
                    REP        #$30
                    LDA        #$0000
                    STA        SHST_gsosCREATE+10   ; File AuxType (0000)
                    STZ        SHST_gsosCREATE+12
*--
                    JSL        GSOS                 ; Create File
                    dw         $2001                ; Code for command 'Create'
                    adrl       SHST_gsosCREATE      ; Parameters Table Address                    
*----
SHST_WriteData      LDA        #$0002               ; Open access Mode = Write
                    STA        SHST_gsosOPEN+8     
                    JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       SHST_gsosOPEN        ; Parameters Table Address (Open in Read Mode)
                    BCS        SHST_End             ; Error
                    LDA        SHST_gsosOPEN+2      ; File Descriptor
                    STA        SHST_gsosSETMARK+2
                    STA        SHST_gsosWRITE+2
                    STA        SHST_gsosCLOSE+2
*--
                    LDA        #$0001               ; EOF
                    STA        SHST_gsosSETMARK+4
                    JSL        GSOS                 ; Set Mark to End
                    dw         $2016                ; Code for command 'SetMark'
                    adrl       SHST_gsosSETMARK     ; Parameters Table Address   
                    BCS        SHST_Close           ; Error
*--
                    LDA        #$010E               ; Data Length = 270 bytes
                    STA        SHST_gsosWRITE+8
                    STZ        SHST_gsosWRITE+10
                    JSL        GSOS                 ; Write Data to file
                    dw         $2013                ; Code for command 'Write'
                    adrl       SHST_gsosWRITE       ; Parameters Table Address   
                    BCS        SHST_Close           ; Error
*--                 
SHST_Close          JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       SHST_gsosCLOSE       ; Parameters Table Address 
*----
SHST_End            RTS               

*----------   SaveHighScoreTable Variables   ---------
SHST_gsosDESTROY    dw         1                    ; Number of parameters
                    adrl       LHST_FilePath        ; +2  File Pathname
*--                    
SHST_gsosCREATE     dw         5                    ; Number of parameters
                    adrl       LHST_FilePath        ; +2  File Pathname
                    ds         2                    ; +6  File Access
                    ds         2                    ; +8  File Type
                    ds         4                    ; +10 File AuxType       
                    ds         2                    ; +14 Storage Type
*--
SHST_gsosOPEN       dw         3                    ; Number of parameters (Open in Write Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LHST_FilePath        ; +4 File Pathname
                    ds         2                    ; +8 Request Access (1=Read,2=Write)
*--
SHST_gsosSETMARK    dw         3                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         2                    ; +4 Base
                    ds         4                    ; +6 Displacement
*--
SHST_gsosWRITE      dw         5                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    adrl       HSS_FileScore        ; +4  Data Buffer Address
                    ds         4                    ; +8  Data Size (Byte)
                    ds         4                    ; +12 Number of byte written to disk
                    ds         2                    ; +16 Cache Priority (#$0000 = No Cache)
*--
SHST_gsosCLOSE      dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


****************************************************************************************
*  CompactAsciiScore() :  Compact a 4 Ascii Score as 16 Bit (IN:A=@Score,OUT:A=Score). * 
****************************************************************************************
CompactAsciiScore   STA       CompactAscScore_1+1   ; 4*Ascii Score
                    INC
                    STA       CompactAscScore_2+1
                    INC
                    STA       CompactAscScore_3+1
                    INC
                    STA       CompactAscScore_4+1
*--
CompactAscScore_1   LDA       $AAAA
                    AND       #$00FF
                    SEC
                    SBC       #$0030
                    XBA
                    ASL
                    ASL
                    ASL
                    ASL
                    STA       CompactAscScore_End+1
*--
CompactAscScore_2   LDA       $AAAA
                    AND       #$00FF
                    SEC
                    SBC       #$0030
                    XBA
                    ORA       CompactAscScore_End+1
                    STA       CompactAscScore_End+1                    
*--
CompactAscScore_3   LDA       $AAAA
                    AND       #$00FF
                    SEC
                    SBC       #$0030
                    ASL
                    ASL
                    ASL
                    ASL
                    ORA       CompactAscScore_End+1
                    STA       CompactAscScore_End+1                    
*--
CompactAscScore_4   LDA       $AAAA
                    AND       #$00FF
                    SEC
                    SBC       #$0030
                    ORA       CompactAscScore_End+1
                    STA       CompactAscScore_End+1                    
*--
CompactAscScore_End LDA       #$0000                ; 1*16 bit Score
                    RTS


******************************************************************************************************
*  CompactHexaScore() :  Compact a 4 Hexa Score as 16 Bit ((IN:A=@Score,IN:X=BankScore,OUT:A=Score). * 
******************************************************************************************************
CompactHexaScore    STX       CompactHexScore_1+2   ; 4*Ascii Score
                    STX       CompactHexScore_2+2
                    STX       CompactHexScore_3+2
                    STX       CompactHexScore_4+2
                    STA       CompactHexScore_1+1
                    INC
                    INC
                    STA       CompactHexScore_2+1
                    INC
                    INC
                    STA       CompactHexScore_3+1
                    INC
                    INC
                    STA       CompactHexScore_4+1
*--
CompactHexScore_1   LDAL      $BBAAAA
                    AND       #$000F
                    XBA
                    ASL
                    ASL
                    ASL
                    ASL
                    STA       CompactHexScore_End+1
*--
CompactHexScore_2   LDAL      $BBAAAA
                    AND       #$000F
                    XBA
                    ORA       CompactHexScore_End+1
                    STA       CompactHexScore_End+1                    
*--
CompactHexScore_3   LDAL      $BBAAAA
                    AND       #$000F
                    ASL
                    ASL
                    ASL
                    ASL
                    ORA       CompactHexScore_End+1
                    STA       CompactHexScore_End+1                    
*--
CompactHexScore_4   LDAL      $BBAAAA
                    AND       #$000F
                    ORA       CompactHexScore_End+1
                    STA       CompactHexScore_End+1                    
*--
CompactHexScore_End LDA       #$0000                ; 1*16 bit Score
                    RTS


************************************************************************************************
*  IsValidChar() :  Check if the Character is [A-Z0-9.Space] (IN:A8=Char,OUT:CarryClearIfTrue) *
************************************************************************************************
IsValidChar         PHY 
                    LDY        #$0025
IsValidChar_1       CMP        Alpha_ASCII_Tab,Y
                    BEQ        IsValidChar_True
                    DEY
                    BPL        IsValidChar_1
*--
IsValidChar_False   PLY                             ; Not in Range
                    SEC
                    RTS
*--
IsValidChar_True    PLY                             ; In Range
                    CLC
                    RTS
                       

**************************************************************************************
*  IsValidNum() :  Check if the Character is [0-9] (IN:A8=Char,OUT:CarryClearIfTrue) *
**************************************************************************************
IsValidNum          PHY 
                    LDY        #$0009
IsValidNum_1        CMP        Alpha_ASCII_Tab+1,Y  ; '0' - '9'
                    BEQ        IsValidNum_True
                    DEY
                    BPL        IsValidNum_1
*--
IsValidNum_False    PLY                             ; Not in Range
                    SEC
                    RTS
*--
IsValidNum_True     PLY                             ; In Range
                    CLC
                    RTS

*****************************************************************************************