******************************************************************************************
*                                                                                        *
*  AppleIIgsKarate.Sound.s :  Library for Sound Effects Handling.                        *
*                                                                                        *
******************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  April 2022  *
******************************************************************************************

****************************************************
*  EntrySoundVector() :  Sound Interrupt Function. *
****************************************************
EntrySoundVector    PHK                             ; Data Bank is Program Bank
                    PLB
*--
                    CLC                             ; 16 BIT
                    XCE
                    REP        #$30
*--
                    LDA        ESV_Function         ; Function to call to populate DOC Ram with Sound Data
                    BEQ        ESV_End
                    STA        ESV_FunctionCall+1
                    LDA        ESV_Function+1
                    STA        ESV_FunctionCall+2
                    INC        ESV_Counter
ESV_FunctionCall    JSL        $BBAAAA
*--
ESV_End             CLC                             ; OK
                    RTL
                    
*----------   EntrySoundVector Variables   ---------
ESV_Function        HEX        000000               ; Feed DOC Ram Function
ESV_Counter         HEX        0000                 ; Count how many times the 16 KB / 2KB sound data have been updated


***********************************************************************************************************************************
*  CopySoundDataToRAM() :  Copy Sound Data to Sound RAM (IN:A=Sound@Bank|@DOC,IN:X=Sound@,IN:Y=SoundSize,IN:CarrySetIfInterrupt). *
***********************************************************************************************************************************
CopySoundDataToRAM  ENT
                    PHB
                    PHK
                    PLB
                    STX        CSDTR_1+1            ; Sound Address Src
                    STY        CSDTR_3+1            ; Sound Length
                    SEP        #$20
                    STA        CSDTR_0+1            ; DOC RAM Address : 00 40 80
                    XBA
                    STA        CSDTR_1+3            ; Sound Bank Address A=XX
                    REP        #$30
*--
                    LDA        SoundToolError       ; No Sound ?
                    BEQ        CSDTR_SI
                    JMP        CSDTR_END
*--
CSDTR_SI            STZ        CSDTR_EIOE
                    BCS        CSDTR_DP             ; Called from Sound Interrupt
                    INC        CSDTR_EIOE
*--
                    JSL        StopSound            ; Stop Current Sound
*--
                    SEI                             ; Disable Interrupts
CSDTR_DP            PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
*--
CSDTR_0             LDA        #$00                 ; Copy Sound Data into Sound RAM at $00.00 | $40.00 | $80.00
                    STAL       SOUNDADRH
                    LDA        #$00
                    STAL       SOUNDADRL            ; Sound Ram Address = $0000            
                    LDX        #$0000
CSDTR_1             LDAL       $BBAAAA,X            ; Copy Raw Sound Data
CSDTR_2             STA        $3D                  ; SOUNDDATA = $C03D
                    INX
CSDTR_3             CPX        #$AAAA               ; Sound Size
                    BNE        CSDTR_1
*--
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
                    REP        #$30
                    PLD                             ; Restore Direct Page Address
                    LDA        CSDTR_EIOE
                    BEQ        CSDTR_END 
                    CLI                             ; Enable Interrupts
*--                                                 
CSDTR_END           PLB
                    RTL

*----------   CopySoundDataToRAM Variables   ---------
CSDTR_EIOE          HEX        0000                 ; Enable Interrupts on Exit


*********************************************************************************************************************************
*  CopyData2BankToDOC() :  Copy Sound Data from Data2Bank to DOC RAM (IN:A=@DOC,IN:X=Sound@,IN:Y=SoundSize,IN:CarrySetIfAdd00). *
*********************************************************************************************************************************
CopyData2BankToDOC  ENT
                    PHB
                    PHK
                    PLB
                    STA        CSDTDOC_DOCAddress   ; Dst DOC RAM Address
                    LDA        Data2Bank
                    STA        CSDTDOC_1+2          ; Src Sound Bank Address A=XX
                    STX        CSDTDOC_1+1          ; Src Sound Address
                    STY        CSDTDOC_3+1          ; Sound Length
*--
                    LDA        SoundToolError       ; No Sound ?
                    BNE        CSDTDOC_END
*--
CSDTDOC_EZ          STZ        CSDTDOC_AddZero      ; Check if we need to add 6 extra 00 at the end of the sound
                    BCC        CSDTDOC_DP           
                    INC        CSDTDOC_AddZero
*--
CSDTDOC_DP          PHD                             ; Save Direct Page Address
                    LDA        #$C000
                    TCD                             ; Define $C000 as the new Direct Page Address
*--
                    SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F
                    ORA        #$60                 ; Access Sound Ram + Auto Increment Enabled
                    STAL       SOUNDCTL
*--
CSDTDOC_0           LDA        CSDTDOC_DOCAddress+1 ; Copy Sound Data into Sound RAM at $00.00 | $40.00 | $80.00
                    STAL       SOUNDADRH
                    LDA        CSDTDOC_DOCAddress
                    STAL       SOUNDADRL            ; Sound Ram Address = CSDTDOC_DOCAddress
                    LDX        #$0000
CSDTDOC_1           LDAL       $BBAAAA,X            ; Copy Raw Sound Data
CSDTDOC_2           STA        $3D                  ; SOUNDDATA = $C03D
                    INX
CSDTDOC_3           CPX        #$AAAA               ; Sound Size
                    BNE        CSDTDOC_1
*--
                    LDA        CSDTDOC_AddZero
                    BEQ        CSDTDOC_4
                    LDA        #$00                 ; Add few #$00 to stop the Sound
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
                    STA        $3D
*--
CSDTDOC_4           REP        #$30
                    PLD                             ; Restore Direct Page Address
*--                                                 
CSDTDOC_END         PLB
                    RTL

*----------   CopyData2BankToDOC Variables   ---------
CSDTDOC_DOCAddress  HEX        0000                 ; DOC Sound Data Address
CSDTDOC_AddZero     HEX        0000                 ; Add 6 extra 00 at the ed of the Sound


*************************************************************************************************
*  PlayOneShotSound() :  Start Sound in One Shot Mode (IN:A=Size|Freq,IN:X=Sound@DOC,IN:Y=Osc). *
*************************************************************************************************
PlayOneShotSound    ENT
                    PHB
                    PHK
                    PLB
                    STA        POSS_SoundFreq       ; Size|Freq
                    TYA
                    AND        #$00FF
                    STA        POSS_OscA            ; Oscillator Even
                    INC                             ; Oscillator Odd
                    STA        POSS_OscB
                    STX        POSS_SoundAddress    ; Sound Address
*--
                    LDA        SoundToolError       ; No Sound ?
                    BEQ        POSS_Init
                    JMP        POSS_End
*--
POSS_Init           SEP        #$20
                    SEI                             ; Disable Interrupts
*------
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*----
                    LDA        #$E1                 ; Oscillators Enable Register
                    STAL       SOUNDADRL
                    LDA        #$3E                 ; # Oscillators x 2 => We want all the 32 oscillators running
                    STAL       SOUNDDATA
*------
                    LDA        #$A0                 ; Control Register for Oscillator Even
                    CLC
                    ADC        POSS_OscA                
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators Even : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    LDA        #$A0                 ; Control Register for Oscillator Odd
                    CLC
                    ADC        POSS_OscB              
                    STAL       SOUNDADRL
                    LDA        #$13                 ; Stop Oscillators Odd : Canal #0=0001 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*------
                    LDA        POSS_OscA            ; Frequency Low Register for Oscillator Even
                    STAL       SOUNDADRL
                    LDA        POSS_SoundFreq       ; Frequency Low value (Hz*32/1645), 11025 hz => D6
                    STAL       SOUNDDATA
                    LDA        #$24                 ; Frequency High Register for Oscillator Even
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*--
                    LDA        POSS_OscB            ; Frequency Low Register for Oscillator Odd
                    STAL       SOUNDADRL
                    LDA        POSS_SoundFreq       ; Frequency Low value (Hz*32/1645), 11025 hz => D6
                    STAL       SOUNDDATA
                    LDA        #$25                 ; Frequency High Register for Oscillator Odd
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*------
                    LDA        #$40                 ; Sound Volume Register for Oscillator Even
                    CLC
                    ADC        POSS_OscA
                    STAL       SOUNDADRL
                    LDY        POSS_OscA
                    JSL        GetOscVolume         ; Volume : 00 (Min) / XX (Volume)
                    STAL       SOUNDDATA
*--
                    LDA        #$40                 ; Sound Volume Register for Oscillator Odd
                    CLC
                    ADC        POSS_OscB
                    STAL       SOUNDADRL
                    LDY        POSS_OscB
                    JSL        GetOscVolume         ; Volume : 00 (Min) / XX (Volume)
                    STAL       SOUNDDATA
*------
                    LDA        #$80                 ; Sound Address Register for Oscillator Even
                    CLC
                    ADC        POSS_OscA
                    STAL       SOUNDADRL
                    LDA        POSS_SoundAddress+1  ; Sound Address = $F800 => $F8
                    STAL       SOUNDDATA
*--
                    LDA        #$80                 ; Sound Address Register for Oscillator Odd
                    CLC
                    ADC        POSS_OscB
                    STAL       SOUNDADRL
                    LDA        POSS_SoundAddress+1  ; Sound Address = $F800 => $F8
                    STAL       SOUNDDATA
*------
                    LDA        #$C0                 ; Oscillator Even
                    CLC
                    ADC        POSS_OscA
                    STAL       SOUNDADRL
                    LDA        POSS_SoundSize       ; Sound Size = 1KB, 2KB, 4KB, 8KB, 16 KB...
                    STAL       SOUNDDATA
                    LDA        #$C0                 ; Oscillator Odd
                    CLC
                    ADC        POSS_OscB
                    STAL       SOUNDADRL
                    LDA        POSS_SoundSize       ; Sound Size = 1KB, 2KB, 4KB, 8KB, 16 KB...
                    STAL       SOUNDDATA
*------
                    LDA        #$A0                 ; Control Register for Oscillator Even
                    CLC
                    ADC        POSS_OscA
                    STAL       SOUNDADRL
                    LDA        #$02                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A0                 ; Control Register for Oscillator Odd
                    CLC
                    ADC        POSS_OscB
                    STAL       SOUNDADRL
                    LDA        #$12                 ; Canal #1=0001 / No Interrupt=0 / One-Shot=01 / Start Sound=0
                    STAL       SOUNDDATA                    
*------
                    CLI                             ; Enable Interrupts
                    REP        #$30
POSS_End            PLB
                    RTL 

*----------   PlayOneShotSound Variables   ---------
POSS_OscA           HEX        0000                 ; Oscillator Even  
POSS_OscB           HEX        0000                 ; Oscillator Odd
POSS_SoundFreq      HEX        00                   ; Sound Frequence
POSS_SoundSize      HEX        00                   ; Sound Size
POSS_SoundAddress   HEX        0000                 ; Sound Address


**********************************************************************************************************************
*  PlayLoopSound() :  Start Sound in Swap Mode xKB+xKB (IN:A=SoundAddress|SoundSize,IN:X=@IntFunc,IN:Y=BankIntFunc). *
**********************************************************************************************************************
PlayLoopSound       ENT
                    PHB
                    PHK
                    PLB
                    STA        PLS_SoundSize        ; Sound Address for Odd Oscillator | Sound Size
                    STY        ESV_Function+1       ; Sound Interrupt Function Bank
                    STX        ESV_Function         ; Sound Interrupt Function Address
*--
                    LDA        SoundToolError       ; No Sound ?
                    BEQ        PLS_Init
                    JMP        PLS_End
*--
PLS_Init            SEP        #$20
                    SEI                             ; Disable Interrupts
*------
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*----
                    LDA        #$E1                 ; Oscillators Enable Register
                    STAL       SOUNDADRL
                    LDA        #$3E                 ; # Oscillators x 2 => We want all the 32 oscillators running
                    STAL       SOUNDDATA
*------
                    LDA        #$A0                 ; Control Register for Oscillator #00                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators #00 : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    LDA        #$A1                 ; Control Register for Oscillator #01                
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators #01 : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    LDA        #$A2                 ; Control Register for Oscillator #02                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators #02 : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    LDA        #$A3                 ; Control Register for Oscillator #03                
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Stop Oscillators #03 : Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*------
                    LDA        #$00                 ; Frequency Low Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => D6
                    STAL       SOUNDDATA
                    LDA        #$20                 ; Frequency High Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA                 
*--
                    LDA        #$01                 ; Frequency Low Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => D6
                    STAL       SOUNDDATA
                    LDA        #$21                 ; Frequency High Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*--
                    LDA        #$02                 ; Frequency Low Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => D6
                    STAL       SOUNDDATA
                    LDA        #$22                 ; Frequency High Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA
*--
                    LDA        #$03                 ; Frequency Low Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$D6                 ; Frequency Low value (Hz*32/1645), 11025 hz => D6
                    STAL       SOUNDDATA
                    LDA        #$23                 ; Frequency High Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Frequency High value (Hz*32/1645)
                    STAL       SOUNDDATA                    
*------
                    LDA        #$40                 ; Sound Volume Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDY        #$0000
                    JSL        GetOscVolume         ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$41                 ; Sound Volume Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDY        #$0001
                    JSL        GetOscVolume         ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$42                 ; Sound Volume Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDY        #$0002
                    JSL        GetOscVolume         ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA
*--
                    LDA        #$43                 ; Sound Volume Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDY        #$0003
                    JSL        GetOscVolume         ; Volume : 00 (Min) / FF (Max)
                    STAL       SOUNDDATA                    
*------
                    LDA        #$80                 ; Sound Address Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Sound Address = $0000 => $00
                    STAL       SOUNDDATA
*--
                    LDA        #$81                 ; Sound Address Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        PLS_SoundAddress     ; Sound Address = $4000 => $40, $0800 => $08
                    STAL       SOUNDDATA
*--
                    LDA        #$82                 ; Sound Address Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$00                 ; Sound Address = $0000 => $00
                    STAL       SOUNDDATA
*--
                    LDA        #$83                 ; Sound Address Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        PLS_SoundAddress     ; Sound Address = $4000 => $40, $0800 => $08
                    STAL       SOUNDDATA
*------
                    LDA        #$C0                 ; Oscillator #00
                    STAL       SOUNDADRL
                    LDA        PLS_SoundSize        ; Sound Size = 16 KB / 2KB
                    STAL       SOUNDDATA
                    LDA        #$C1                 ; Oscillator #01
                    STAL       SOUNDADRL
                    LDA        PLS_SoundSize        ; Sound Size = 16 KB / 2KB
                    STAL       SOUNDDATA
                    LDA        #$C2                 ; Oscillator #02
                    STAL       SOUNDADRL
                    LDA        PLS_SoundSize        ; Sound Size = 16 KB / 2KB
                    STAL       SOUNDDATA
                    LDA        #$C3                 ; Oscillator #03
                    STAL       SOUNDADRL
                    LDA        PLS_SoundSize        ; Sound Size = 16 KB / 2KB
                    STAL       SOUNDDATA                    
*------
                    LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$0E                 ; Canal #0=0000 / Interrupt=1 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01
                    STAL       SOUNDADRL
                    LDA        #$0F                 ; Canal #0=0000 / Interrupt=1 / Swap=11 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02
                    STAL       SOUNDADRL
                    LDA        #$16                 ; Canal #1=0001 / No Interrupt=0 / Swap=11 / Start Sound=0
                    STAL       SOUNDDATA
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03
                    STAL       SOUNDADRL
                    LDA        #$17                 ; Canal #1=0001 / No Interrupt=0 / Swap=11 / Stop Sound=1
                    STAL       SOUNDDATA                    
*------
                    CLI                             ; Enable Interrupts
                    REP        #$30
PLS_End             PLB
                    RTL 

*----------   PlayLoopSound Variables   ---------
PLS_SoundSize       HEX        00                   ; $36 = 16 KB, $1B = 2 KB
PLS_SoundAddress    HEX        00                   ; Odd Oscillator Address : $40 = $4000, $08 = $0800, $00 = $0000 


**********************************************************************
*  EnableDisableAudio() :  Enable / Disable Music and Sound effects. *
**********************************************************************
EnableDisableAudio  ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--                      
                    SEP        #$30
                    SEI                             ; Disable Interruption
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
                    LDA        EDA_STATUS
                    BEQ        EDA_DisableAudio
*--------
EDA_EnableAudio     STZ        EDA_STATUS           ; Enable Audio
*--
EDA_EA_Ready        LDAL       SOUNDCTL             ; Is DOC Busy ?
                    BMI        EDA_EA_Ready
*--
                    LDX        #$00
EDA_Restaure        TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDAL       EDA_SoundRegister,X  ; Restaure Sound Registers
                    STAL       SOUNDDATA
                    INX
                    CPX        #$20
                    BNE        EDA_Restaure
*--
                    BRA        EDA_End
*--------
EDA_DisableAudio    INC        EDA_STATUS           ; Disable Audio
*--
EDA_DA_Ready        LDAL       SOUNDCTL             ; Is DOC Busy ?
                    BMI        EDA_DA_Ready
*----
                    LDX        #$00
EDA_DA_Backup       TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDAL       SOUNDDATA            ; Double Read
                    LDAL       SOUNDDATA
                    STAL       EDA_SoundRegister,X  ; Backup Sound Registers
                    TXA
                    CLC
                    ADC        #$A1
                    STAL       SOUNDADRL
                    LDAL       SOUNDDATA            ; Double Read
                    LDAL       SOUNDDATA
                    STAL       EDA_SoundRegister+1,X
*--
                    LDY        #$02                 ; x2 for the Loop Songs
EDA_DA_Stop         TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDA        #%0000_0001          ; Stop Oscillator
                    STAL       SOUNDDATA
                    TXA
                    CLC
                    ADC        #$A1
                    STAL       SOUNDADRL
                    LDAL       #%0000_0001
                    STAL       SOUNDDATA
                    DEY
                    BNE        EDA_DA_Stop
*--
                    INX
                    INX
                    CPX        #$20
                    BNE        EDA_DA_Backup
*----
EDA_End             CLI                             ; Enable Interruption
                    REP        #$30
                    PLB
                    RTL

*----------   EnableDisableAudio Variables   ---------
EDA_STATUS          HEX        00                   ; Enable (00) / Disable (01)
EDA_SoundRegister   ds         32                   ; Sound Register values
         
         
**************************************************************         
*  MuteUnmuteAudio() :  Mute / Unmute Music & Sound effects. *          
**************************************************************
MuteUnmuteAudio     ENT
                    PHB
                    PHK
                    PLB
*--
                    SEP        #$20                 ; Inverse Sound Status
                    SEI                             ; Disable Interrupts
                    LDA        MUA_STATUS
                    BNE        MUA_Unmute
*--
MUA_Mute            INC        MUA_STATUS           ; Disable Sound/Music : 00 -> 01
                    BRA        MUA_Begin
MUA_Unmute          STZ        MUA_STATUS           ; Enable Sound/Music  : 01 -> 00
*--
MUA_Begin           LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
                    LDY        #$0000
                    LDA        #$40                 ; Sound Control Registers : 40->5F
                    STA        MUA_Loop+1
MUA_Loop            LDA        #$AA 
                    STAL       SOUNDADRL
                    INC        MUA_Loop+1
                    PHY
                    LDA        MUA_STATUS
                    BEQ        MUA_GetVolume
                    LDA        #$00                 ; Mute
                    BRA        MUA_SetVolume
MUA_GetVolume       JSL        GetOscVolume         ; Get Sound Volume : 00 (Mute) -> FF
MUA_SetVolume       STAL       SOUNDDATA
                    PLY
                    INY
                    CPY        #$0020
                    BNE        MUA_Loop
*--
                    CLI                             ; Enable Interrupts
                    REP        #$30            
                    PLB
                    RTL  

*----------   MuteUnmuteAudio Variables   ---------
MUA_STATUS          HEX        0000                 ; Enable (00) / Mute (01)


*************************************************************************         
*  SetStatusMusic() :  Define Music Status for this game (IN:A=Status). *          
*************************************************************************
SetStatusMusic      ENT                             ;  Enble/Disable Music for this game
                    PHB
                    PHK
                    PLB
*--
                    STA        MUA_Status_Music     ; 0=ON,1=OFF
*--
                    PLB
                    RTL  
                    
                    
*************************************************************************         
*  SetStatusSound() :  Define Sound Status for this game (IN:A=Status). *          
*************************************************************************
SetStatusSound      ENT                             ; Enble/Disable Sounds for this game
                    PHB
                    PHK
                    PLB
*--
                    STA        MUA_Status_Sound     ; 0=ON,1=OFF
*--
                    PLB
                    RTL  

*----------   MuteUnmuteAudio / SetStatusMusic / SetStatusSound Variables   ---------
MUA_Status_Music    HEX        0000                 ; 1:Mute/0:Volume
MUA_Status_Sound    HEX        0000                 ; 1:Mute/0:Volume


*****************************************************************************************         
*  GetOscVolume() :  Get Oscillator Sound Volume (IN:Y=Osc[00-1F],OUT:A=Volume[00-FF]). *          
*****************************************************************************************
GetOscVolume        ENT                             ; Works in 8 and 16 bit
                    PHB
                    PHK
                    PLB
*--
                    PHP
                    PHP
                    PLA
                    ASL
                    ASL
                    ASL
                    BCC        GOV_16Bit
*--
                    mx         %10
GOV_8Bit            PLA                             ; A is 8 bit wide
                    LDA        GOV_LEVEL,Y
                    CPY        #$0004               ; 00-03=Music 04-1F=Sound
                    BGE        GOV_8Bit_Sound
GOV_8Bit_Music      LDY        MUA_STATUS           ; Music
                    BNE        GOV_8Bit_Mute
                    LDY        MUA_Status_Music
                    BNE        GOV_8Bit_Mute
                    BRA        GOV_End              ; Volume
GOV_8Bit_Sound      LDY        MUA_STATUS           ; Sound
                    BNE        GOV_8Bit_Mute
                    LDY        MUA_Status_Sound
                    BNE        GOV_8Bit_Mute
                    BRA        GOV_End              ; Volume
GOV_8Bit_Mute       LDA        #$00                 ; Mute
                    BRA        GOV_End
*--
                    mx         %00
GOV_16Bit           LDA        GOV_LEVEL,Y          ; A is 16 bit wide
                    AND        #$00FF              
                    CPY        #$0004               ; 00-03=Music 04-1F=Sound
                    BGE        GOV_16Bit_Sound
GOV_16Bit_Music     LDY        MUA_STATUS           ; Music
                    BNE        GOV_8Bit_Mute
                    LDY        MUA_Status_Music
                    BNE        GOV_16Bit_Mute
                    BRA        GOV_End              ; Volume
GOV_16Bit_Sound     LDY        MUA_STATUS           ; Sound
                    BNE        GOV_16Bit_Mute
                    LDY        MUA_Status_Sound
                    BNE        GOV_16Bit_Mute
                    BRA        GOV_End              ; Volume
GOV_16Bit_Mute      LDA        #$0000               ; Mute
                    BRA        GOV_End
*--
GOV_End             PLB
                    RTL


****************************************************************************************         
*  SetOscVolume() :  Get Oscillator Sound Volume (IN:A=Volume[00-FF],IN:Y=Osc[00-1F]). *          
****************************************************************************************
SetOscVolume        ENT                             ; Work in 8 and 16 bit
                    PHB
                    PHK
                    PLB
*--
                    STA        SOV_Volume+1 
                    PHP
                    PHP
                    PLA
                    ASL
                    ASL
                    ASL
                    BCC        SOV_16Bit
*--
SOV_8Bit            STA        GOV_LEVEL,Y          ; A is 8 bit wide
                    PLA                             ; Get back the extra PHP
                    BRA        SOV_End
*--
SOV_16Bit           LDA        GOV_LEVEL,Y          ; A is 16 bit wide
                    AND        #$FF00
SOV_Volume          ORA        #$0000
                    STA        GOV_LEVEL,Y
*--
SOV_End             PLB
                    RTL

*----------   GetOscVolume / SetOscVolume Variables   ---------
GOV_LEVEL          HEX        FF,FF,FF,FF,FF,FF,FF,FF ; Volume (00-FF)
                   HEX        FF,FF,FF,FF,FF,FF,FF,FF
                   HEX        FF,FF,FF,FF,FF,FF,FF,FF
                   HEX        FF,FF,FF,FF,FF,FF,FF,FF


*************************************************
*  FadeOutSound() :  Fade Out Sound to Silence. *
*************************************************
FadeOutSound        ENT
                    PHB
                    PHK
                    PLB
                    SEP        #$30                 ; Init
                    LDY        #$10
*--
FOS_Begin           LDAL       SOUNDCTL             ; Busy ?
                    AND        #$80
                    BNE        FOS_Begin
*--
                    SEI                             ; Disable Interruption
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
                    LDX        #$40      
FOS_Loop            TXA
                    STAL       SOUNDADRL
FOS_SoundLevel      LDAL       SOUNDDATA            ; Double Read
                    LDAL       SOUNDDATA
                    CMP        #$10
                    BLT        FOS_Next
                    SEC
                    SBC        #$10                 ; FF to 00
                    STAL       SOUNDDATA
FOS_Next            INX
                    CPX        #$60
                    BNE        FOS_Loop
*--
                    CLI                             ; Enable Interruption
*--
                    DEY
                    BEQ        FOS_End
*--
                    REP        #$30
                    JSR        WaitForNextVBL       ; Tempo (1/60 sec)
                    SEP        #$30
                    BRA        FOS_Begin
*--
FOS_End             REP        #$30
                    PLB
                    RTL


********************************************
*  StopSound() :  Stop the 32 oscillators. *
********************************************
StopSound           ENT
                    PHB
                    PHK
                    PLB
                    STZ        ESV_Function+1       ; Disable Interrupt Routine
                    STZ        ESV_Function
*--
                    SEP        #$30
                    SEI                             ; Disable Interrupts
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
                    LDY        #$04                 ; x2 for the Loop Songs
StopSound_1         LDX        #$A0                 ; Stop Oscillators
StopSound_2         TXA                             ; Control Register for Oscillator #00 - #1F
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    INX
                    CPX        #$C0
                    BNE        StopSound_2
                    DEY
                    BNE        StopSound_1
*--
                    CLI                             ; Enable Interrupts
                    REP        #$30
                    PLB
                    RTL

BOOT_SOUND          HEX        80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,80,D2,70,A7,90,80,80,7F,78,73,74,7C,84,93,A3
                    HEX        B5,BF,C6,C7,CA,CA,C3,B3,A5,8F,7C,6C,54,41,3A,3B,41,48,53,58,56,4F,47,3B,30,2C,2A,2A,30,3D,42,48
                    HEX        54,64,6C,77,80,89,94,A5,B7,C3,CD,D6,DD,D8,D5,D6,D4,CA,C9,C6,C5,BB,AD,A2,9C,96,96,9A,A3,A9,A9,A9
                    HEX        A9,A8,97,88,7F,7A,74,77,80,8C,8F,8D,8C,86,7F,78,71,6A,65,64,66,6A,6A,69,64,62,5E,56,50,4D,48,45
                    HEX        39,2F,2B,28,24,24,23,20,1E,1C,1E,28,33,3E,50,62,7B,93,AE,C9,E2,EF,EE,E2,DC,D6,C7,B8,B9,C6,D1,D8
                    HEX        E4,E6,D6,C6,B8,AE,A5,9E,9E,A3,A9,A5,9F,93,82,77,6D,65,5D,57,57,5B,5F,66,6C,73,77,74,70,6A,61,4D
                    HEX        36,23,15,11,15,19,21,33,45,53,65,7A,82,93,9E,A2,AD,B0,AE,B5,C2,CD,D5,DC,E4,E7,E4,D9,D2,CB,C7,C7
                    HEX        D0,D4,D2,D0,C5,B0,9B,88,7B,65,56,50,54,62,66,61,5F,5B,49,3A,3A,3E,42,45,41,36,2A,12,06,07,08,12
                    HEX        2A,3A,47,58,6D,77,7E,88,9E,A8,A9,A9,A9,A8,A5,A8,B3,BF,CA,D5,DC,DC,D4,CD,C7,BC,B5,AC,A1,94,86,7E
                    HEX        73,65,64,6A,73,7C,85,94,96,89,80,6D,5E,53,49,4B,53,57,5B,61,66,66,5F,57,53,49,41,3E,3E,3E,42,48
                    HEX        53,5F,6A,6C,6D,70,6D,69,6D,78,80,85,94,A5,B3,B8,BF,CD,DD,E4,F2,FD,FC,EF,DF,CA,B1,A5,A8,AD,B8,C9
                    HEX        CD,C9,BF,AD,9C,8C,80,7B,7B,7A,6C,58,4B,37,1E,0E,0F,1A,27,2F,44,52,53,52,54,56,54,54,5D,6D,7A,80
                    HEX        84,84,85,85,86,91,A3,B1,BE,CB,D9,E4,EB,EE,EA,DF,D4,CD,CD,CA,C3,BF,C3,C2,BC,BB,B5,A3,94,86,7F,7B
                    HEX        78,77,78,70,5F,4F,39,2C,24,21,2B,30,39,48,54,52,49,49,49,48,49,53,57,56,52,47,39,27,15,0F,16,1E
                    HEX        2B,3B,4B,53,5B,5E,61,62,65,71,80,88,96,AA,BF,CD,D8,DF,E6,EA,ED,EF,EF,F1,F6,F3,EB,E6,E0,D5,BE,A3
                    HEX        91,81,70,5F,58,54,50,4D,4B,45,3E,36,2B,24,1E,1E,1E,20,1C,1E,23,2B,37,49,5E,6C,7E,91,A8,B8,C9,D4
                    HEX        D4,D5,DF,E6,E6,E6,E7,E7,EB,EB,E6,EA,F1,F1,EE,E7,E2,DC,C6,B1,9C,81,65,50,48,4B,54,5D,69,7B,80,80
                    HEX        7A,71,6F,6D,6C,70,78,77,6C,5D,53,4F,48,4B,53,53,49,3E,37,3D,4B,5F,71,7E,84,86,86,8F,9C,A5,AD,AE
                    HEX        AC,A2,94,90,91,96,9E,A9,B8,C2,C3,C7,C3,B1,97,80,73,5F,57,5D,66,7A,85,82,78,6A,54,41,35,2C,23,1E
                    HEX        24,2B,35,42,42,37,35,37,3B,3A,3B,45,4D,4D,50,5B,5F,62,66,6D,77,80,8C,9F,AA,AE,B3,B3,AC,A3,AA,AE
                    HEX        AA,AA,AE,B0,B8,CA,D6,E0,E3,D9,CA,B8,9F,8C,80,77,70,71,7C,82,80,7E,74,61,41,2C,2B,2F,37,44,53,5E
                    HEX        5E,5F,62,6A,78,89,A3,B9,CA,D5,D4,CA,BF,B1,AA,AA,B0,BB,CA,D4,D4,D1,D4,D9,DC,D9,D5,D1,C5,B1,A2,9A
                    HEX        91,8F,88,82,82,81,80,7B,6D,5F,54,4F,4F,54,58,5E,62,5D,53,4B,45,42,47,50,65,6F,71,71,70,70,6F,70
                    HEX        70,6D,6F,77,7B,7E,80,89,9A,AA,BB,BE,B9,B3,B9,C3,C3,C6,CB,CB,C7,B9,A5,9A,8D,85,86,85,81,7E,6F,5E
                    HEX        49,30,1C,04,19,13,13,16,18,19,19,1D,28,39,47,4B,4C,4A,45,4A,56,60,6C,76,7E,8B,94,93,91,9C,AD,C7
                    HEX        E2,F8,FC,F3,D9,BD,9D,8B,81,82,88,9B,AF,B8,BC,B3,A7,92,80,74,6D,69,69,6D,6D,6D,67,68,71,7C,83,91
                    HEX        92,8C,82,7A,6E,6E,7F,99,B8,D5,DA,CE,B6,97,80,67,53,4C,4C,51,5D,6E,80,97,A4,AC,AC,A8,A8,B1,B2,B7
                    HEX        BA,AD,A1,A3,A6,A3,A3,AD,BD,C7,CF,D5,CF,C2,B8,B1,9F,91,8B,88,8B,8C,8E,8C,87,80,79,6C,63,59,56,57
                    HEX        59,61,63,5D,4F,3D,26,14,0B,16,2C,3F,53,64,68,63,5D,65,73,81,92,99,9B,9C,9E,A3,A7,B1,BD,BE,B2,9F
                    HEX        87,6F,63,65,71,83,9E,B3,BC,B4,A6,91,84,80,79,72,63,46,2C,1E,26,34,4A,64,74,79,73,63,54,43,34,2B
                    HEX        26,2E,43,59,61,61,62,67,68,67,64,68,74,82,93,93,84,7D,75,6F,6E,7C,8C,A9,B7,B6,AC,9E,8B,82,8C,9C
                    HEX        AA,AC,A9,A8,A7,A7,AD,B4,B4,AF,A7,A8,AD,AF,AD,A9,A6,A1,9E,99,93,91,8C,88,88,80,7C,75,74,71,65,5C
                    HEX        58,56,5A,65,74,8D,A8,BA,B8,AC,9C,82,68,5C,63,72,7F,84,88,8D,97,9D,A7,B3,B8,BA,B3,AA,A6,9E,9C,A4
                    HEX        B1,AF,9F,8F,82,7E,79,79,79,78,74,7A,83,8C,99,A7,AF,A4,8C,74,68,64,73,8C,9B,98,8F,82,7A,73,6D,6E
                    HEX        75,80,8B,84,73,5C,4D,3F,36,32,36,43,46,43,43,48,51,58,53,43,34,36,4C,68,7D,83,86,88,88,80,7C,7C
                    HEX        80,83,8B,8E,8E,91,A4,B4,BA,B8,B2,AA,A1,8F,80,71,68,6F,78,80,82,7D,64,46,34,2B,28,2C,2E,2E,34,41
                    HEX        4E,5A,64,6D,72,6E,64,5A,58,61,79,92,B8,D7,E0,D9,D2,C9,BE,B8,B3,A8,99,8D,86,87,8F,9F,B6,C8,CD,CE
                    HEX        CF,CC,B8,9F,84,74,63,4F,4D,58,64,73,80,81,7E,7C,75,6F,71,7E,8F,9F,AF,A8,92,87,8F,A3,B4,C2,CA,D3
                    HEX        CF,C1,B3,AD,A6,A3,A7,AA,AA,A9,A7,9F,97,8D,81,79,71,6C,5A,49,49,58,5D,5D,5F,63,65,6C,73,81,8F,8F
                    HEX        83,7A,67,58,51,4E,53,5D,6E,7D,80,82,92,9E,9F,98,97,91,8D,8F,97,9C,9B,8D,78,61,56,59,63,72,80,86
                    HEX        87,81,79,67,58,57,5D,59,51,48,46,49,4A,51,65,79,7D,7A,75,6E,61,4E,41,46,53,59,5C,54,48,49,54,5D
                    HEX        5C,57,63,80,99,AC,A8,94,81,7F,82,8C,9C,B1,BA,B7,AC,99,81,75,78,80,97,A6,B2,BA,B8,AC,9C,8D,84,80
                    HEX        79,7F,97,AD,B1,AA,9C,8C,87,88,8E,98,99,8E,80,73,61,49,36,3C,56,6D,6F,65,63,69,6F,74,7E,82,87,93
                    HEX        A3,AA,A9,A8,B1,BF,C7,C7,C8,BE,A6,93,93,A8,C7,E2,ED,E3,D9,D3,CC,C1,B7,B4,B4,B2,AC,A1,91,79,62,5C
                    HEX        57,4F,57,65,68,5A,51,59,65,64,4F,39,34,38,39,34,31,37,46,54,61,6C,74,75,67,57,5D,71,84,A6,C2,CF
                    HEX        CE,BE,9E,84,7E,7E,81,8F,9D,A7,A8,A4,9D,A1,A4,A1,9C,97,91,88,7A,63,4E,44,4A,54,51,49,4D,57,59,53
                    HEX        41,28,12,10,17,1D,27,2D,34,34,32,33,39,4F,65,7C,81,80,80,82,84,8D,A6,C1,CC,C9,C9,C4,B4,9D,94,A8
                    HEX        C7,D5,D4,D3,CA,BD,B3,B2,AF,A9,A3,9B,82,71,64,58,4D,46,4D,58,67,6A,65,5F,6A,80,8D,8E,8F,8C,81,7F
                    HEX        7D,73,67,67,76,8A,9B,A7,AD,AC,A5,A1,9E,9A,97,98,9D,9B,95,97,A1,AE,B6,BF,C6,C5,B8,A8,9E,9A,93,91
                    HEX        A2,B6,BE,B8,A7,91,80,72,66,70,82,89,85,84,82,79,6A,69,79,86,8A,84,79,68,56,4B,43,32,27,36,4C,54
                    HEX        58,63,72,73,72,7E,8B,8C,7E,74,79,77,6B,68,72,7B,7E,80,8A,9A,A1,9B,96,91,8E,8B,8A,8F,8D,81,7A,72
                    HEX        6B,67,6F,7D,8A,93,93,8A,7D,6F,62,5E,5D,57,50,49,3B,2A,29,3A,46,44,41,49,58,65,69,6F,7B,80,83,8A
                    HEX        8D,80,6F,63,61,6D,80,94,A8,BE,D4,D9,CE,BB,A8,9F,9A,97,8E,85,86,85,7E,7F,89,98,A1,A1,9E,9B,97,8E
                    HEX        8B,8E,98,A0,9F,96,8A,81,7F,7D,6C,58,54,62,7A,82,82,82,82,83,8D,9E,A1,94,84,80,81,81,81,84,91,9E
                    HEX        A5,A9,A3,93,86,8A,97,A7,B2,BE,C3,B7,AC,A7,A1,98,8D,80,7D,7B,7B,7F,8A,99,A1,9D,8D,77,61,5C,62,61
                    HEX        57,4F,57,65,7A,84,89,93,9A,9A,93,84,73,5C,50,58,67,71,73,74,75,72,72,7B,89,95,95,8E,83,75,68,6A
                    HEX        70,74,73,72,75,6D,56,3B,35,3B,42,53,66,72,7A,8E,A1,96,7C,62,52,4D,57,66,7B,8B,90,8F,93,91,8D,8B
                    HEX        86,80,73,5D,4B,4C,59,61,65,69,6F,6D,66,62,61,67,6A,66,69,74,80,8B,8D,8E,99,B0,BE,BF,AE,94,83,7F
                    HEX        80,87,A2,C4,D9,D3,C9,BE,A8,8F,8B,9B,A0,99,9B,A3,99,7D,6A,61,53,49,55,69,75,7C,82,89,8B,8A,86,80
                    HEX        7A,7B,81,8C,94,94,96,A1,B1,BD,C3,C6,C8,C3,BE,B8,A7,98,93,8D,89,8D,8D,82,7F,86,95,9F,9D,90,7A,66
                    HEX        60,62,69,7E,96,AB,AE,A0,89,6D,4A,2E,2D,3A,42,52,6D,85,8C,83,79,6D,6A,6C,68,5B,55,65,73,73,6C,69
                    HEX        71,7E,84,8C,9E,AD,AB,99,86,7F,73,5C,42,46,5C,63,68,74,87,91,99,A8,AA,9F,95,89,80,7D,77,6F,66,5F
                    HEX        5C,57,4E,4C,56,5F,5E,55,4F,42,30,29,35,4A,63,77,80,84,89,8D,82,68,55,59,66,71,7D,84,8D,9B,AE,CB
                    HEX        E5,EE,DB,C7,BE,BB,A5,8F,83,86,8E,8A,89,94,A2,AC,B3,B8,B1,97,7D,6D,66,52,42,48,55,5E,6D,85,9B,A7
                    HEX        A9,A9,A3,8E,7B,77,7A,7B,87,AB,C4,BB,98,8A,91,98,9E,99,8A,80,89,96,9B,99,9F,9F,8C,71,5C,59,63,74
                    HEX        87,A0,BB,CA,C5,A4,8D,8A,8E,85,79,7D,90,A0,99,8F,89,82,71,57,4F,54,53,5C,7D,96,96,90,8C,87,85,86
                    HEX        75,55,44,42,44,43,43,45,49,53,69,81,8E,93,97,98,8E,80,70,68,6A,70,73,7F,9A,AD,A7,93,80,79,7D,81
                    HEX        82,7F,76,70,71,74,7D,80,7A,6F,5E,4E,4A,44,36,3A,4A,4D,4C,5C,7B,82,7E,7E,70,4B,37,43,55,61,71,86
                    HEX        98,A4,B1,C0,C6,BE,B2,AA,A7,9A,8F,8E,8C,8A,89,8C,96,A9,B4,AC,9E,94,89,7D,71,52,37,42,6A,8C,9B,9A
                    HEX        95,91,91,93,84,81,8F,9A,9E,9F,93,8A,8D,97,A4,AC,A9,A5,A0,99,8C,80,82,8A,84,8A,A8,C5,C7,B7,A2,91
                    HEX        87,84,89,84,7C,71,68,5D,52,5F,7F,99,A3,B0,BA,AC,97,7F,6A,62,65,6C,7D,8C,86,75,7A,83,80,7D,80,8C
                    HEX        98,95,71,48,42,4F,52,4C,50,66,86,A4,AC,A3,8F,80,6F,57,4D,60,80,8A,85,8D,99,A0,97,87,7B,74,71,60
                    HEX        50,56,72,80,77,62,52,49,50,6A,87,99,99,93,80,5E,2F,0E,14,2C,44,6D,9A,B3,AA,83,5F,59,67,6F,73,80
                    HEX        8E,87,79,72,7A,80,91,A5,B4,C0,BE,A7,91,8C,8C,80,66,57,60,77,91,9B,82,6D,77,91,A2,98,96,9A,94,89
                    HEX        7C,6F,74,85,96,A1,A4,98,81,6A,62,61,62,73,93,AC,AC,AD,B6,AB,86,73,80,9D,AA,A3,96,7C,62,5E,66,74
                    HEX        86,9A,A5,B1,BA,BD,B4,8F,7A,7A,74,75,97,B2,9A,8C,A8,C1,AC,85,80,87,8B,8B,93,A1,AA,A5,9B,9B,9D,91
                    HEX        81,71,57,41,2C,22,2F,3D,49,62,6D,59,57,76,8D,97,94,90,8B,80,73,66,52,52,79,AB,BF,B7,AC,9E,8A,81
                    HEX        81,7D,77,7A,7B,81,85,7C,77,7E,74,6A,72,73,6D,72,7F,80,6A,4B,40,45,4B,4E,55,63,6F,61,5D,75,82,89
                    HEX        9B,99,7A,58,5D,6B,6D,76,85,95,A0,A5,91,6A,41,31,39,4D,5F,71,86,90,96,97,89,82,8B,93,96,95,8F,80
                    HEX        67,60,72,84,8E,86,91,C5,FD,F2,C5,9F,89,80,80,8E,A5,AA,9B,94,9D,9A,86,82,8B,97,A8,A7,81,43,29,48
                    HEX        68,71,7C,82,75,56,42,42,59,76,85,94,99,94,8D,89,95,B3,CD,DC,E5,E8,E1,CD,B8,A3,8B,81,8A,9B,A4,9E
                    HEX        96,A5,B1,9D,7A,60,66,76,75,7A,91,9D,84,6A,50,46,4D,65,77,6D,57,59,7C,96,8F,84,80,7C,73,67,50,41
                    HEX        41,53,71,8C,A1,9F,90,7A,58,44,4A,58,68,6D,71,73,5F,41,44,68,8D,AB,A9,97,91,93,8C,81,84,91,8F,80
                    HEX        80,97,AB,A4,85,7E,93,A3,8D,65,5D,65,6A,79,7B,6C,5F,62,6B,6B,5C,46,31,1F,1F,3C,58,67,75,7F,87,8E
                    HEX        87,80,79,7B,7E,7D,7C,81,8A,87,83,83,8A,9B,A7,AA,B4,C7,CE,C4,BF,C5,BC,A9,A2,A3,9B,8D,8C,9A,A7,A7
                    HEX        8C,5F,50,70,86,80,5C,5B,7B,82,80,84,7E,63,59,6C,80,82,7B,71,7C,8A,90,8F,85,80,8E,A8,B5,9F,8A,96
                    HEX        AA,98,6B,50,6D,95,AC,BC,B5,9F,9E,9D,8F,89,87,80,73,68,6B,80,93,9D,9D,9A,99,A0,AC,A7,A2,AE,B4,B0
                    HEX        A2,8D,71,45,28,3F,6B,77,6F,75,8A,84,59,43,4A,52,45,3C,45,61,7D,73,50,40,52,54,3D,3D,6D,99,B0,A5
                    HEX        8D,89,8B,8B,8A,91,95,95,94,9B,A9,AB,96,80,7B,8A,95,84,66,4D,4D,69,80,8B,8A,80,74,72,74,77,63,38
                    HEX        25,38,4A,52,55,5B,62,79,85,85,74,57,55,65,6A,70,7F,97,A9,A5,8B,80,89,91,94,94,9A,AE,B7,AA,A5,A8
                    HEX        A9,AC,9D,8F,8F,8C,8F,A1,AA,93,86,8E,90,93,8C,77,6D,84,9A,A5,B2,A4,87,7E,6D,68,7A,81,80,8F,BB,C4
                    HEX        9B,6D,57,5F,74,7C,70,66,5B,56,60,6B,74,87,A3,B0,A5,99,AC,BD,A9,95,99,A1,9B,90,91,99,A1,A4,9A,91
                    HEX        9B,B6,BA,9A,7E,82,99,91,76,66,79,96,90,79,6D,67,54,45,58,6D,69,61,4B,35,3A,4D,63,7C,82,7E,80,7D
                    HEX        70,54,3C,33,43,5C,75,81,8B,A9,A4,80,80,98,A3,9D,9A,9A,9B,97,7D,65,5F,70,80,83,75,6D,7E,86,93,9F
                    HEX        95,70,3B,1E,28,52,80,8B,74,70,8C,A4,81,4F,3F,54,5E,56,50,68,90,A3,99,82,71,71,80,81,73,77,90,A7
                    HEX        AC,A8,99,84,76,6C,6B,80,9D,9D,87,7E,72,60,67,80,83,8D,AA,C0,CE,CE,BB,9F,93,8E,89,93,A7,AA,9A,90
                    HEX        9F,AA,9B,80,70,7F,9B,B3,B3,98,83,96,A3,89,68,55,4C,4D,4D,54,5E,5F,61,76,8E,98,96,93,95,93,86,94
                    HEX        B2,B5,90,6A,67,68,65,7F,94,A7,BB,B2,A4,B0,B2,93,80,8F,A2,9E,8F,A2,BB,B3,A7,90,63,38,38,4F,5D,63
                    HEX        74,80,80,7D,7D,74,72,80,82,75,60,48,2C,23,41,4E,58,76,8E,8F,7E,69,66,6D,71,7A,84,80,6A,72,7F,6A
                    HEX        6D,95,AC,9A,98,9A,91,96,A4,A4,80,62,62,71,74,70,69,61,55,69,86,89,91,95,82,81,87,6C,4A,56,76,71
                    HEX        33,1C,42,52,57,55,61,7A,83,8A,AC,BE,A5,84,68,5F,7D,8D,8B,87,79,70,7E,94,A4,98,95,B2,BD,A2,9E,A3
                    HEX        9D,9E,A4,BE,CD,C4,B1,9F,97,85,73,71,7A,7C,84,A4,CB,DA,B7,85,70,6A,6A,65,61,7A,7D,5D,50,56,62,74
                    HEX        8A,99,95,85,83,8D,97,9B,98,89,72,69,80,9E,9A,8E,98,AB,A9,99,9E,A3,9E,80,61,7C,9F,7D,66,83,A4,B1
                    HEX        AB,94,80,74,65,63,79,98,A1,99,97,89,7B,66,5D,74,79,69,65,75,81,75,50,3A,54,72,74,67,5B,56,57,4F
                    HEX        43,43,5F,66,50,42,45,5E,6F,66,7D,B4,BE,93,8B,BC,D6,A1,63,5B,7D,90,81,8B,97,8B,8E,9E,9E,80,69,84
                    HEX        AD,B5,98,80,77,6A,52,45,4F,46,46,56,5B,45,31,39,61,8E,91,81,76,5C,57,62,6A,77,84,87,82,84,93,8A
                    HEX        72,74,8F,C3,E5,D6,C9,C0,AA,8C,91,AA,A3,93,95,AC,C6,BD,A1,8F,96,96,80,7D,84,87,81,80,7C,7F,75,63
                    HEX        63,5D,7C,A7,9F,80,73,66,6C,75,74,79,81,94,9A,82,7B,80,8F,A8,BB,AB,80,6B,87,AC,A7,97,91,93,94,8B
                    HEX        82,8C,A5,A0,83,81,93,8B,68,54,6C,79,66,75,8C,97,AC,BD,BC,AD,90,6C,5D,59,4C,30,32,76,A0,80,4F,45
                    HEX        67,85,8A,95,B4,9F,73,52,45,45,44,45,55,61,63,70,8D,A3,99,8C,8A,89,89,8B,89,68,5C,77,83,8D,8C,87
                    HEX        80,6D,61,71,82,6C,46,4B,6F,83,74,5F,69,8B,86,73,7A,6D,46,53,80,87,5D,3C,4C,6D,7D,83,8C,AA,AD,8C
                    HEX        86,A4,B4,9B,8B,90,85,74,58,45,5D,7E,9A,C1,D7,BD,A7,B7,A9,7B,53,5B,7D,8C,95,A7,B3,A7,80,73,81,80
                    HEX        7B,8D,B8,C9,B6,8F,7B,73,6A,73,8A,8D,91,A0,9E,93,90,8A,7C,71,71,72,7A,90,A3,A2,94,85,80,70,4E,36
                    HEX        44,5C,65,80,A7,B5,B4,CA,D2,C8,B3,A1,9F,A1,96,84,83,99,9B,86,6D,4C,55,8B,A1,A2,BF,BF,A2,89,7D,63
                    HEX        3C,27,2F,3B,40,43,55,77,84,6B,4D,4B,55,52,6D,94,8A,5F,67,84,6D,46,59,73,75,7B,8B,9F,A8,B2,C1,B6
                    HEX        95,7D,5F,67,97,AD,A7,B6,AC,9D,8F,75,53,38,3A,44,32,39,79,A8,9E,79,6F,5F,5C,6D,67,5C,67,81,8E,7B
                    HEX        57,56,4D,3F,57,86,A1,A4,AC,BB,AD,95,75,60,6F,80,7C,69,72,91,97,8A,93,AD,AB,97,8A,8E,9D,AE,AC,8D
                    HEX        77,82,79,61,70,7F,89,9F,AA,AD,B1,A4,A0,BA,BA,99,81,99,AD,9F,7A,4E,59,70,74,77,77,70,6B,6B,7C,99
                    HEX        A3,B1,A5,7D,5C,43,40,4E,56,63,81,96,98,A0,A2,A2,BE,D6,C3,AA,AD,B5,A3,87,7C,77,7A,7D,6B,6F,9E,C3
                    HEX        CA,BA,97,8C,A8,9F,52,0E,38,7D,7E,7A,8E,85,6F,6B,86,96,57,0C,1C,56,6F,55,5C,7E,9F,9B,7F,83,99,8D
                    HEX        6C,72,75,52,4E,75,82,60,5D,8C,A4,93,80,7C,80,95,9E,7D,68,80,8E,75,4B,4D,60,50,5C,8E,A4,9F,9D,9B
                    HEX        83,7E,75,5D,75,87,7B,67,60,6F,6A,4E,54,7E,A7,A9,8C,7F,7E,8C,89,6C,57,5F,67,52,57,6C,6C,61,59,69
                    HEX        87,9D,91,70,6D,86,99,A8,A8,B7,BD,9F,98,B0,A4,81,7E,9E,BE,CF,CB,AA,A1,C0,C9,A5,90,A7,99,7E,77,68
                    HEX        4C,5B,6B,54,59,76,80,70,7F,96,8B,8A,99,86,66,56,5D,69,6D,7E,7C,7E,89,91,99,98,9E,A9,A7,8E,80,7C
                    HEX        76,80,99,BE,C8,BC,BE,A7,79,6C,8D,A3,8D,86,96,96,83,67,49,4F,77,9E,B2,8B,74,8C,95,73,41,32,53,7C
                    HEX        6A,58,91,AD,97,7E,77,76,6F,67,5F,65,80,87,7C,67,53,29,15,68,B5,9A,59,68,94,80,4D,43,56,6D,70,5C
                    HEX        6F,8E,94,8D,80,87,B0,A3,8D,95,83,67,8C,C0,B8,9A,84,80,7A,5E,63,71,7A,86,97,9A,85,66,45,42,61,6F
                    HEX        67,67,69,65,58,4D,53,58,56,4C,5E,84,A1,A5,A1,9F,97,A0,A4,97,82,71,8E,B3,BF,AD,7A,75,AA,CB,B4,99
                    HEX        95,95,8A,72,80,AE,B4,A2,A5,A8,8A,75,74,80,93,7F,72,9A,AA,86,79,81,83,5B,44,7F,B1,86,4F,5F,82,8D
                    HEX        79,5E,62,71,84,94,82,7B,97,BB,9B,75,7F,8E,9D,A7,9B,93,8D,77,6A,79,7E,85,AC,B3,A5,A0,A1,A8,A7,95
                    HEX        7E,83,9A,98,82,73,75,72,62,67,7E,81,80,87,82,83,87,5E,37,48,5E,67,4D,2F,37,58,77,7C,85,8C,80,71
                    HEX        62,5B,5B,60,6A,80,96,89,80,A7,A8,6C,3F,62,A2,B7,97,91,B2,B4,9A,80,69,7E,9A,97,8C,7B,6B,5D,49,32
                    HEX        33,4E,6D,77,80,85,7A,6B,62,66,71,73,73,72,6B,57,56,71,8C,8D,79,74,A2,CD,A9,66,5B,94,A3,7C,62,73
                    HEX        89,99,A3,7C,6D,8E,91,77,76,98,96,94,C9,D9,98,74,89,97,AB,D1,C1,8D,8C,A8,8D,6B,76,8F,93,94,BB,AE
                    HEX        75,5D,72,86,7B,73,98,BC,B5,90,63,3D,41,5B,73,7D,81,89,82,80,87,86,6C,52,72,98,8E,86,8E,90,80,7B
                    HEX        91,AD,BE,BF,B1,AB,B1,B6,A5,80,76,90,AA,86,7C,8B,93,80,7A,7D,85,7C,5D,5C,71,61,44,41,5D,7B,6D,68
                    HEX        61,4F,4C,54,60,48,42,7D,8F,6D,57,7E,91,8D,8C,8B,85,89,93,70,53,77,90,80,8E,AC,9F,95,A1,98,6D,4C
                    HEX        60,80,89,99,96,74,5D,54,50,46,57,6A,7D,79,6F,82,A1,A0,8B,8E,72,32,4F,7E,74,50,4F,75,8A,8B,82,6F
                    HEX        68,84,A3,97,70,75,8F,7C,5D,68,95,AD,9F,80,7E,8E,AB,AC,9D,C1,D6,A3,62,62,6F,5B,7F,B1,C3,C6,B8,B1
                    HEX        A0,6F,4D,68,90,A7,C0,BE,86,55,5F,84,8D,96,97,85,82,8D,84,4F,3B,79,8A,67,65,7D,8C,9B,A5,A8,9D,61
                    HEX        53,98,A5,77,7A,90,A9,A8,7D,74,97,B2,A8,AB,B2,A3,85,7F,8A,85,8D,A2,9D,8A,58,3B,77,9E,7F,71,91,A9
                    HEX        90,68,5F,6F,5E,3A,40,83,AD,7D,5E,75,69,49,5C,5D,5D,93,A3,97,89,97,8B,81,7F,67,53,55,60,83,80,77
                    HEX        8C,7A,6A,65,6A,7D,70,5D,71,8A,84,79,80,90,79,45,46,80,95,86,93,9B,96,8E,91,7E,6B,79,79,6A,74,A5
                    HEX        B7,80,63,68,53,37,4A,71,80,9E,A4,95,94,89,5E,45,3D,56,70,71,7B,79,72,7F,84,86,90,85,7E,8E,A5,99
                    HEX        8C,8A,99,A4,A5,AC,BF,BA,A5,A9,93,89,91,A3,BB,B2,82,61,6B,9A,9E,73,73,87,9F,A7,84,58,5B,55,57,6F
                    HEX        96,AC,A5,96,89,82,7E,6F,76,7D,71,7F,9A,98,96,95,98,8C,80,8D,A3,B2,B1,B4,9F,7A,84,90,85,67,67,7F
                    HEX        76,7A,77,72,87,9B,7E,5D,6C,91,85,7A,91,98,85,6C,5F,63,5E,54,57,77,9F,AB,A2,A1,94,6F,4F,68,8E,79
                    HEX        55,60,6F,68,7F,6F,5C,80,8B,6A,4F,69,87,85,72,6F,82,72,54,65,69,52,62,89,8D,6A,70,8B,8D,80,6A,79
                    HEX        7F,7F,A2,91,66,83,B5,A3,69,5D,80,8E,7B,6D,76,8A,8D,83,7F,7E,83,7C,46,42,54,59,67,6A,7D,95,81,77
                    HEX        84,72,68,75,86,B0,BE,A3,AB,B2,9D,8F,93,87,80,85,7D,85,96,A3,C1,C1,A7,8B,8E,9D,83,62,8E,A4,8D,87
                    HEX        94,6A,50,83,8E,96,B1,BD,9F,80,6C,61,73,7F,6D,72,8F,A5,97,70,5F,6D,82,96,AA,90,5C,80,B5,AE,66,40
                    HEX        70,8C,91,93,85,87,96,A2,90,69,70,7E,8E,9D,81,84,AA,B7,94,83,91,9B,A0,87,69,4D,54,77,6F,56,5B,7D
                    HEX        A3,95,7A,72,89,61,4B,6F,77,43,38,61,7C,82,63,4A,7D,93,61,70,94,9F,84,66,80,84,57,58,83,93,95,AA
                    HEX        A9,8B,79,82,8C,6F,67,A3,BC,6D,45,84,80,4B,42,72,8B,67,5D,80,90,80,79,77,79,6A,4A,42,6C,87,7D,62
                    HEX        6B,80,9B,9B,5E,69,99,9A,87,A7,B3,7C,6D,83,AA,B2,91,93,96,81,6C,70,91,80,7A,7F,8C,9F,A0,80,7A,99
                    HEX        9E,97,91,7C,72,7E,7D,7A,98,9B,84,9A,A4,A7,AE,AE,9D,8F,83,7C,63,5B,73,B3,C4,99,90,9F,8D,77,7F,6C
                    HEX        45,59,77,8C,94,87,7B,87,A2,9A,87,76,6A,6D,7A,6C,79,A3,B6,9B,83,7F,90,A3,9E,A7,8F,6F,73,87,8E,9F
                    HEX        8E,6D,72,72,6D,84,A2,81,67,73,79,5E,59,63,69,57,73,7F,52,4F,8A,82,4E,65,80,6B,5C,5D,75,80,8A,93
                    HEX        75,68,94,91,65,67,68,50,6A,A8,BA,90,94,AD,99,71,48,43,68,75,6C,95,91,66,67,7C,90,90,61,54,7A,84
                    HEX        89,80,7F,7F,8A,8E,83,7D,63,5E,65,68,87,B4,A8,75,68,80,9B,8C,65,66,8D,82,6C,74,80,8C,85,7C,93,A2
                    HEX        85,8A,B2,AE,96,99,A4,8E,73,70,9D,BB,98,8B,A0,9E,9B,A1,7F,71,70,80,A3,A1,7F,75,93,B3,95,75,97,B4
                    HEX        90,57,4F,73,80,77,70,6A,77,83,99,A9,94,7D,63,75,80,59,4C,68,8F,9E,A4,B8,91,66,7B,AD,B5,A1,80,80
                    HEX        A2,8F,53,4C,77,A5,A8,74,7C,82,67,69,93,9D,9B,A1,87,79,60,52,65,7B,7C,79,89,AD,A5,6F,4B,60,72,71
                    HEX        6D,8E,AB,84,70,95,95,42,20,57,6A,65,82,81,6B,54,49,52,87,76,3C,62,9E,91,60,60,7C,7F,76,70,70,7C
                    HEX        93,A1,81,6B,8E,B6,BC,A8,99,8B,91,BA,91,68,79,96,87,59,73,80,56,5E,89,B3,95,63,55,77,72,46,5E,77
                    HEX        72,54,65,8B,84,6C,7A,95,8D,84,98,9A,8F,91,91,80,8A,9E,A4,97,80,A1,C6,B4,B7,9D,7B,79,8A,A8,7F,6D
                    HEX        A9,C4,9A,94,99,82,7A,5E,40,5C,72,74,87,BB,B6,72,5E,A3,BD,8C,67,61,7D,80,7A,77,80,8E,8D,B0,B5,80
                    HEX        83,91,A3,95,65,69,7A,7A,74,7B,6D,61,7F,AA,99,67,5D,7E,74,70,73,80,9B,9B,89,87,80,9A,A0,7A,5C,7F
                    HEX        A2,76,56,67,86,AA,86,6A,9E,B0,86,7E,72,67,52,45,5B,57,56,6F,8C,87,6B,67,6B,4F,39,3F,60,82,8C,71
                    HEX        6D,80,83,7A,57,5B,80,AA,9F,8D,95,97,94,8E,A1,85,79,A0,8B,54,69,7D,75,84,90,7B,5B,79,AB,9D,74,6F
                    HEX        71,77,83,63,65,7E,82,76,67,9D,AA,75,71,85,74,4B,5E,90,9B,A4,A9,9E,A3,97,82,90,79,73,8B,82,8F,9D
                    HEX        AC,A9,82,6C,72,98,BE,BB,9D,A4,B2,7D,4B,65,7F,90,9A,94,8C,BD,B4,9A,B1,8D,79,AB,A4,6D,63,5E,68,7B
                    HEX        9D,9A,59,49,93,85,25,35,6D,79,86,82,87,B8,A5,75,6F,66,4C,62,97,A0,8D,A0,CB,9E,52,4E,90,9F,8C,B6
                    HEX        CF,B6,96,8E,86,5C,4F,7A,82,7B,93,8A,76,98,8E,5D,62,71,49,54,81,77,4B,42,71,6B,4A,4F,7F,89,44,52
                    HEX        85,90,7B,75,80,7B,72,60,72,98,95,80,76,77,9A,A4,8D,84,67,57,7C,8E,73,5F,94,8D,6B,7C,7A,6A,85,AC
                    HEX        AA,7B,49,53,8C,9D,7C,7C,87,94,A5,90,79,77,65,5F,62,5B,69,99,90,67,81,9A,90,90,9D,7E,4F,80,BA,96
                    HEX        5B,76,6F,5F,76,90,80,79,75,8B,B1,AE,91,7C,6D,9E,E7,E3,98,80,A9,BA,7B,65,A9,BE,91,B0,CA,83,55,63
                    HEX        85,8D,6C,79,9B,B6,9A,61,61,7C,75,7B,7C,73,5B,69,8F,84,76,98,B7,84,69,77,7C,77,7E,8B,86,93,85,6B
                    HEX        8D,98,80,73,87,B5,C8,A8,87,95,80,61,70,6F,5C,7C,98,9E,99,83,7D,80,67,57,79,6F,6A,8C,79,60,74,72
                    HEX        75,59,40,6B,80,59,49,5E,82,89,82,85,71,58,63,6B,5D,53,53,5E,5D,80,A1,7B,56,69,95,C6,BC,9F,93,97
                    HEX        84,74,80,83,80,8E,A3,86,71,82,89,7A,8F,B2,97,71,86,72,46,7A,A0,80,6D,56,49,8B,7C,3D,73,8B,68,6C
                    HEX        76,80,8B,82,74,6F,7F,86,7D,5F,80,B8,93,7D,9A,8B,7F,94,9A,B5,B1,9F,B8,BE,AB,9A,8F,70,63,87,A8,A0
                    HEX        8E,98,A7,85,66,82,94,90,7E,7F,A3,BD,A4,84,8F,82,6A,5D,5D,61,A1,B6,7F,6D,97,79,42,63,95,8E,98,A8
                    HEX        79,49,4D,70,86,7D,5C,77,98,8B,72,80,95,76,80,C1,A8,56,4C,80,8A,82,84,71,79,AE,D7,A5,77,80,B3,B1
                    HEX        80,82,87,87,95,80,5C,69,8F,6D,31,49,80,70,79,76,7B,85,70,5C,72,81,42,1F,55,6C,44,42,5D,6F,8A,7E
                    HEX        72,9F,86,5D,66,7C,85,82,97,87,63,7A,A5,B5,7E,76,98,A4,85,85,81,86,85,61,8A,C4,A0,6A,80,AE,8C,77
                    HEX        7C,58,7D,A0,6D,52,5C,7B,87,89,8D,89,7C,73,7A,7C,58,49,77,5C,44,76,94,8D,8C,95,AA,9F,93,8E,77,74
                    HEX        81,80,8E,BA,DA,BC,8C,81,AB,AE,76,7A,A7,A4,AC,CD,B0,8D,93,99,82,65,73,87,65,4E,7F,AB,87,72,95,A9
                    HEX        9E,79,56,7A,96,72,4D,73,8D,97,81,6F,89,9F,82,63,80,A2,97,84,8D,9F,80,6A,8A,A5,7C,50,82,AD,95,5E
                    HEX        73,94,67,54,7A,7C,7C,81,7A,93,A2,8B,72,94,A9,59,20,4D,80,5D,32,5F,9E,B7,A9,94,77,63,50,60,71,77
                    HEX        71,76,7D,6C,55,69,7F,56,62,7D,6D,74,75,5E,5F,99,A8,84,7F,7A,8B,8C,65,5B,65,58,82,94,81,9D,D1,B6
                    HEX        93,BC,AC,76,52,60,73,75,53,6C,A3,AB,8A,65,7B,BF,A3,5E,6F,7F,70,4E,44,58,67,90,8F,68,76,82,65,91
                    HEX        CD,96,49,69,B5,7F,57,BA,C3,9E,9F,A2,9F,8C,82,A3,A2,87,76,87,CA,AE,91,BD,D6,BA,81,5C,75,8D,7C,90
                    HEX        80,84,AA,97,75,79,7B,5D,7B,91,65,5C,80,8A,8C,89,61,75,90,76,6F,6C,45,4E,73,8B,75,5B,80,BE,D2,94
                    HEX        63,7B,74,89,86,71,89,B4,9D,72,80,94,99,87,90,AB,AC,89,8D,B8,8D,54,54,68,76,75,AD,B2,5B,5E,8C,7A
                    HEX        2E,19,57,70,6F,6F,7F,86,69,57,66,6C,38,20,49,70,84,77,53,67,99,AA,A2,87,81,8F,8C,73,5C,75,82,8E
                    HEX        A1,8E,7A,B0,A0,83,AC,A2,A0,90,61,71,8E,7B,6A,6F,69,66,6A,76,80,79,7A,7D,56,63,BD,A8,53,55,61,42
                    HEX        73,89,6B,84,B6,7F,61,AD,A5,6A,5F,82,AC,B0,9E,90,8B,8B,9D,9D,8A,84,95,98,98,86,6B,7B,9D,A7,97,AB
                    HEX        C6,8D,65,95,CA,B3,6C,82,B3,9B,8E,99,73,67,77,6D,7E,A7,A3,96,B0,9B,7F,81,6C,3B,4B,70,8D,9F,63,55
                    HEX        69,68,65,6D,8C,90,77,73,85,90,7E,69,7A,80,82,94,8D,84,8B,9F,A4,9B,97,B6,B5,9A,8C,7F,7F,58,74,83
                    HEX        63,7C,AD,9F,7B,7C,89,77,6F,6A,43,4D,81,6D,4E,56,53,55,48,59,6C,5C,3B,4B,74,86,70,4D,67,8A,85,57
                    HEX        8E,D3,A0,4B,37,76,AA,89,70,A0,BD,89,56,58,89,BF,B1,96,A0,89,82,95,69,4A,80,C0,AE,7E,86,A2,91,58
                    HEX        3A,71,8C,70,63,7B,9A,99,7D,86,A4,7A,5B,4F,53,6B,6B,54,68,81,9E,B7,98,7E,6B,61,72,77,82,96,97,8F
                    HEX        80,86,B8,8F,67,A8,C3,C8,D9,C5,AC,B0,AB,95,93,9F,B6,A0,93,9B,9B,98,86,6F,76,89,A2,83,60,76,82,73
                    HEX        6A,58,4E,55,84,7D,40,5B,97,83,58,7A,A5,95,5C,61,7F,77,84,7B,49,6C,99,87,85,8B,A3,B3,A3,86,87,90
                    HEX        79,60,60,89,9D,90,BE,AE,8E,89,74,7B,7A,73,62,7B,8F,7E,73,7B,89,7B,84,99,8A,7A,4B,54,65,52,4A,6F
                    HEX        99,7A,79,97,6C,52,5B,7D,74,6F,7A,42,3D,60,5F,80,A5,73,4A,83,A4,66,49,5F,6F,79,76,8C,99,97,82,93
                    HEX        A9,9F,B0,9D,80,82,9A,7E,72,91,77,74,B8,D8,BB,97,7F,6B,81,94,80,5F,50,85,94,55,3A,62,91,83,56,69
                    HEX        7F,73,5B,63,7F,8C,84,71,71,86,84,7E,8D,AE,8C,90,AD,B8,9D,A1,BF,BF,AB,9E,75,74,85,86,9E,C3,D4,A9
                    HEX        86,AB,A9,9A,8A,71,66,80,89,81,7D,6F,70,8A,80,7C,97,73,58,77,80,53,5C,89,89,59,42,7B,9F,A8,8A,8E
                    HEX        8B,66,3D,53,8E,9B,85,99,B5,95,80,85,86,80,85,8E,96,7E,56,67,9F,8C,65,82,9F,9B,84,6F,6A,6C,83,A7
                    HEX        9D,87,7A,6D,84,72,4D,5B,50,70,8C,5E,4F,7D,68,4A,65,7F,65,6C,7A,57,36,5C,71,65,82,74,4D,70,A0,91
                    HEX        86,91,85,7B,6F,8E,BE,98,67,99,BC,9A,86,95,8F,7C,98,93,7B,85,7E,7D,A0,8F,7D,7E,81,93,6F,4E,63,4F
                    HEX        46,7C,A4,94,68,5C,68,86,A8,80,65,7C,99,81,55,61,85,8B,76,9D,AE,AC,9F,7D,6A,80,8E,7C,7F,B0,D0,AA
                    HEX        7B,93,A2,95,AB,A2,86,7E,A9,CD,C3,83,71,C5,CF,95,95,AE,A5,8F,6C,5F,5D,55,58,6B,81,7C,66,8B,A0,7C
                    HEX        80,79,3A,44,69,7F,96,82,4D,4B,80,9B,7A,5D,60,67,77,86,80,81,AB,A7,7E,87,81,8C,9A,83,75,9E,C3,99
                    HEX        7B,9E,A5,8D,98,93,83,8C,8F,89,80,76,5D,59,74,81,5C,59,89,83,57,50,52,50,4C,3D,7A,8B,59,31,4B,7C
                    HEX        84,83,6D,5E,61,49,54,98,9D,7B,7D,7B,8D,A8,BB,A3,96,86,6A,6D,A4,9B,65,7E,9F,A8,AC,A7,7F,62,62,79
                    HEX        8B,93,80,67,8E,82,6A,6C,6C,89,80,85,8D,80,79,79,76,73,70,91,96,7A,83,79,53,80,8D,5C,79,89,85,7A
                    HEX        80,85,9A,AD,96,B0,8F,55,8A,A9,A8,AE,A2,A2,9E,96,75,86,AC,AE,A4,98,B1,98,82,85,74,87,AD,9E,80,74
                    HEX        72,89,A3,80,74,79,55,59,76,7E,75,82,82,75,7B,8C,66,2D,54,98,B3,95,71,7E,80,6C,7C,7F,80,A3,B5,95
                    HEX        89,9B,72,73,95,A1,B6,95,69,73,90,8C,6F,80,95,81,62,76,9A,65,42,50,81,98,81,68,6D,7E,74,66,6D,73
                    HEX        75,6D,44,41,66,65,5B,6B,80,83,6F,4A,77,68,70,80,7F,7A,70,7D,82,6B,6D,9D,A2,57,46,6D,7E,A5,C4,99
                    HEX        86,8C,B4,94,6B,8F,B1,97,9D,A5,80,80,90,7D,7B,A5,AD,83,69,7A,67,4A,53,6C,7E,7C,A1,85,5F,68,5B,5F
                    HEX        63,7C,7E,7D,84,72,6B,86,74,71,84,9D,B4,9E,87,A4,A9,94,84,7A,9F,D9,BD,98,9B,B1,BD,B2,A8,82,7D,A1
                    HEX        B2,75,83,C6,9E,7A,89,A4,83,60,6A,6B,46,48,80,BA,A2,73,7D,8F,7D,2F,48,80,70,7C,8E,A2,89,6A,6C,80
                    HEX        5F,48,89,C5,A8,65,7F,A8,81,3F,46,89,A3,9A,B3,95,7C,83,84,84,71,5E,AD,A9,75,8D,79,4A,72,9F,9F,90
                    HEX        79,7F,65,4B,7A,8E,84,50,89,B4,8E,74,5D,54,55,5E,63,50,62,5D,4B,6B,87,8C,6D,57,4F,5E,7B,8B,80,73
                    HEX        65,71,87,73,55,7F,9B,81,84,8E,90,AD,8C,71,7D,8D,AD,BB,81,7D,96,B2,9D,6D,65,B0,B5,6D,3A,6C,93,7E
                    HEX        95,C5,A8,71,4B,59,65,82,8D,8D,75,60,94,7F,62,6B,4A,5F,9D,BC,90,93,9E,61,4F,74,91,AA,A2,89,80,B1
                    HEX        CF,9E,7F,60,72,97,BB,D8,B1,9A,93,96,9D,93,7E,71,87,AC,A9,7D,73,80,89,86,B8,A7,93,98,6F,4F,68,5D
                    HEX        45,53,87,BE,7E,54,79,77,58,69,94,74,65,8A,8E,65,53,83,87,6B,6A,8E,9D,9F,AE,80,6B,97,A8,86,70,85
                    HEX        A4,A4,A8,AA,94,89,80,72,76,77,81,52,40,68,90,82,67,75,83,65,6F,7A,4A,39,4E,6A,71,66,79,77,6A,73
                    HEX        7F,5C,33,55,7F,79,80,9A,9A,60,4D,7C,81,79,9A,A3,6A,60,95,87,73,7A,81,8D,AD,DD,B3,95,A2,82,57,5F
                    HEX        83,9B,81,5B,8A,A4,8E,86,8B,B2,B7,8A,79,71,62,4C,53,67,75,9B,A9,80,62,89,B6,96,44,42,7B,91,7D,65
                    HEX        6C,8E,66,61,93,97,97,9D,81,A0,BA,9D,7E,79,A2,C0,C0,8E,97,C4,AE,89,8E,8F,70,8B,AC,B3,AC,9D,7B,77
                    HEX        A7,B3,A4,A5,80,4B,67,85,71,67,81,8E,96,80,68,58,4A,38,35,54,72,8D,8A,76,8B,AD,7A,3C,55,6F,73,77
                    HEX        83,74,84,99,72,69,A4,D7,A9,76,80,BA,A8,7B,5D,5E,69,74,7A,A2,BE,A0,84,8C,81,62,5D,8B,82,6F,80,7F
                    HEX        72,53,4E,70,9A,93,7F,69,48,37,31,4B,80,82,60,58,B3,C4,86,41,3C,6D,72,61,76,8A,4F,33,66,B6,BF,9E
                    HEX        97,A1,A3,6A,7A,7C,63,86,95,80,80,86,B7,BF,B0,80,61,99,BC,99,6D,7D,9E,93,82,5C,56,77,85,77,5D,81
                    HEX        8C,70,4F,63,8E,89,77,AA,CB,85,49,62,60,5B,72,99,AE,A2,A9,B4,AE,8D,57,5D,8C,AA,D3,BA,84,7E,97,91
                    HEX        8F,A1,8B,96,AC,9B,7B,A5,A3,6B,7A,95,85,7A,89,A0,9A,8A,81,80,68,50,66,8E,66,4E,7A,A1,BE,8F,5F,87
                    HEX        9F,97,7B,4E,70,80,70,81,A9,8A,62,80,7F,6F,44,25,80,BD,8F,8B,98,83,84,9B,91,80,81,7C,76,9D,7B,4D
                    HEX        36,4F,A3,CF,A3,76,93,7A,5E,53,6F,77,60,66,7B,48,2F,63,99,97,66,84,8D,54,6D,97,7A,56,48,3A,54,80
                    HEX        82,6F,66,79,96,96,8C,70,60,83,9F,A3,9E,82,5E,5E,76,90,97,98,AE,AB,A4,AD,7B,63,97,9A,80,7C,7E,8A
                    HEX        96,89,83,84,86,97,66,5D,89,7B,70,6F,68,80,85,81,82,87,83,6B,89,8D,6F,58,6C,90,86,6C,99,CB,A7,83
                    HEX        80,A2,BA,94,8A,A0,80,8B,91,86,B5,A4,99,B2,AE,8C,79,A1,BD,76,4F,94,A0,8E,8E,87,7D,81,79,8D,BC,7B
                    HEX        5E,86,8A,5E,66,76,80,80,4F,52,9B,AE,62,44,55,6F,81,79,79,80,76,62,63,7B,A0,8D,60,65,93,AB,9A,8E
                    HEX        80,84,80,69,69,7F,80,83,84,67,71,9D,B3,A8,7E,80,9F,87,67,65,45,46,62,4F,61,9B,A4,9B,9F,A9,69,4C
                    HEX        67,52,61,5C,3A,45,8A,96,7F,8A,81,69,72,80,86,7F,76,66,55,7E,80,77,68,6A,71,8C,9E,8B,94,99,B0,A1
                    HEX        54,4E,A5,A8,7C,8E,AC,AD,80,94,AD,70,49,7B,B1,8E,74,8A,9E,A2,75,63,8A,7A,7C,9D,63,66,89,99,75,76
                    HEX        80,82,8E,8D,89,68,60,71,69,75,AA,91,89,AA,84,77,96,9F,95,96,96,7F,91,BE,B0,6D,60,83,9F,BD,B8,CB
                    HEX        A9,79,52,7A,AB,94,71,7E,96,79,6F,85,A3,8C,85,7A,81,A5,8E,5E,43,7E,93,76,5F,6D,66,75,71,4D,6F,A0
                    HEX        A4,8C,82,7A,67,66,84,5F,3B,4F,84,91,83,95,99,A0,8C,5B,73,9A,9A,80,8E,87,60,42,77,A9,A5,80,6D,8C
                    HEX        A1,8A,8B,96,77,5D,5C,57,68,7C,57,54,80,7E,7E,80,66,63,72,3F,35,6D,7A,55,56,60,6A,A4,9F,54,53,7D
                    HEX        99,8E,7B,94,9F,91,7D,7E,90,94,83,9A,A7,A8,93,AD,A7,7E,93,76,77,96,99,94,8C,63,69,A4,94,73,60,7B
                    HEX        7F,60,7F,9E,6C,6C,A1,93,7E,69,6F,80,9A,8C,71,61,93,BB,87,67,80,98,98,94,8D,B4,BB,89,80,84,72,77
                    HEX        69,71,89,93,C1,D0,9E,A1,A8,7F,6B,6D,7B,76,84,83,80,AA,BB,A7,AC,A4,91,8B,7D,9D,9E,7F,54,4A,4C,65
                    HEX        7E,96,7A,6F,94,84,74,65,75,71,41,54,8F,84,61,58,6B,67,57,7B,A5,8E,50,5B,72,8D,B7,A3,84,80,84,7D
                    HEX        70,83,93,8B,7C,7B,AB,BA,90,80,71,4D,63,90,7E,6F,82,70,68,79,73,66,77,85,6F,61,60,6D,71,3C,3B,5C
                    HEX        94,A8,94,80,7E,7E,80,80,71,7B,76,71,7B,8C,81,59,6A,87,A5,A9,97,A4,94,87,8A,4B,43,7A,98,A9,9B,98
                    HEX        9F,89,6D,7F,8B,81,70,6D,94,9E,AB,9D,80,8C,89,7E,62,7F,75,70,82,76,86,90,9B,C8,B0,82,7F,68,6C,7D
                    HEX        7A,5B,62,97,AA,AA,A1,90,7F,5E,75,BE,D4,80,8D,B7,7E,4A,80,B3,8A,6F,80,B1,C4,91,6D,7A,84,7B,7C,8E
                    HEX        9E,A3,75,4E,5E,72,95,8E,66,66,8F,96,69,52,72,8B,7E,72,83,87,79,7F,8C,80,80,7D,7D,80,91,99,80,5E
                    HEX        56,75,98,A5,90,7B,9B,97,69,62,80,7D,7B,8F,9D,80,72,72,4E,3D,5C,96,A5,71,7B,83,76,4E,30,6A,80,82
                    HEX        6F,74,7C,5B,52,55,7D,93,74,87,A8,8F,4B,40,79,8D,79,66,73,8C,A1,97,82,5C,7B,97,80,65,8C,97,99,86
                    HEX        59,6F,8B,96,C0,B2,84,82,63,8E,99,97,89,69,7F,B2,BE,9B,69,70,86,A1,70,5E,AA,8A,57,4D,65,91,9F,80
                    HEX        86,A0,8B,7F,87,8A,6C,65,87,91,7F,8C,B1,9D,97,74,8C,BE,85,7C,A9,A1,85,7D,7F,97,B1,94,80,BA,A8,7B
                    HEX        7F,74,7B,62,62,74,98,9B,8A,7F,87,9B,7B,76,7D,74,7F,89,73,7F,84,7E,8C,98,A1,71,71,87,54,3D,83,96
                    HEX        77,71,87,85,76,5B,4B,65,62,5C,73,7D,93,9A,79,79,8C,90,7F,7E,71,7F,B1,AA,70,5C,70,8D,97,80,95,B3
                    HEX        90,5C,73,7E,5B,76,7D,5D,7C,9B,84,5F,55,6B,84,8D,5F,5E,71,5C,57,36,61,84,82,77,54,97,9E,7E,54,61
                    HEX        85,8B,5F,74,9F,8A,80,90,A8,B8,B6,74,75,85,AB,8B,5D,73,87,85,98,A1,8E,A2,B6,A0,87,80,76,75,7C,61
                    HEX        76,9F,80,75,82,98,80,74,9B,7E,77,7B,72,8B,7D,70,99,8B,8A,80,63,86,A8,8B,74,9B,AB,80,80,82,90,8D
                    HEX        8D,9A,A4,96,95,94,91,73,6B,7B,99,90,6A,80,84,9E,BD,AA,80,66,80,91,96,97,74,74,A5,80,6A,7C,7A,62
                    HEX        82,90,66,6B,95,72,5E,67,66,8D,7B,76,82,68,56,4C,67,7F,9B,9F,95,A4,97,84,5D,37,74,8A,67,73,87,84
                    HEX        98,8D,83,80,80,7E,7D,99,91,8A,80,85,80,7F,7C,75,7D,7A,7D,8D,97,79,58,54,61,4D,56,68,5E,6A,85,91
                    HEX        87,73,7D,5D,48,60,5B,5F,80,7A,6C,71,80,A9,A8,79,5F,80,85,9B,93,7F,90,94,83,73,80,91,A7,A1,8C,AE
                    HEX        C0,A4,B0,8D,6D,59,80,84,77,8B,B6,94,67,94,8C,82,81,8D,65,52,99,99,5F,7C,91,81,7F,7D,7F,93,87,66
                    HEX        65,79,7C,7D,90,81,84,A5,80,71,8D,8F,80,94,BA,BA,99,7C,94,80,91,BA,9D,8E,93,B5,A2,75,7B,7F,6C,80
                    HEX        A1,99,80,82,82,69,55,6F,8F,83,5D,70,8E,73,67,93,A1,67,42,81,9B,80,58,58,5C,7A,8C,65,79,A2,7C,50
                    HEX        83,A8,85,54,68,80,7B,8B,81,80,68,81,A2,A4,97,76,83,7C,4C,65,8E,8D,65,7C,A4,96,86,7D,71,54,68,76
                    HEX        50,69,A0,9B,73,75,8B,8A,6D,66,5B,63,77,5F,58,6C,81,98,9F,5F,63,80,83,76,68,7F,87,94,82,65,9F,B5
                    HEX        80,66,94,BA,9B,84,85,9B,81,7B,86,65,6C,7A,83,A8,B3,9A,8F,96,8A,87,60,5E,7F,7A,7E,91,84,70,8A,A0
                    HEX        9B,8E,73,6B,79,7E,70,60,7D,9E,8F,80,8D,9E,8D,6A,87,8F,9E,9F,8C,9A,8E,81,82,8A,B2,9D,7D,87,98,8A
                    HEX        80,9F,AB,7D,55,8A,A4,98,83,67,80,90,8B,7F,8D,8E,80,60,67,95,9D,8D,77,58,74,80,63,67,7B,80,7A,7F
                    HEX        7F,91,A0,79,39,40,58,79,77,85,9A,8D,AC,A5,7C,71,65,52,75,B1,AB,6A,71,89,6D,85,8B,6A,79,B7,B3,75
                    HEX        70,76,85,80,53,63,89,7D,76,73,66,7C,8D,84,60,52,50,7B,80,5E,5B,71,7B,70,6D,56,56,83,8B,80,8F,8D
                    HEX        5F,81,8B,62,77,80,83,80,96,A5,7E,7A,AC,CA,74,4E,8C,7F,7B,7E,8B,99,9A,9F,A2,87,79,A1,A8,71,72,87
                    HEX        9B,85,71,82,79,74,A0,A7,76,73,8E,8F,81,7D,70,68,98,93,59,62,93,8C,8A,94,80,6B,96,89,8D,86,6D,7E
                    HEX        84,6C,80,97,9E,A1,91,7B,7F,85,80,8B,87,8D,AA,B0,A3,96,7E,72,8C,9E,89,8A,9F,9A,81,74,7B,80,87,7E
                    HEX        89,91,80,79,67,7C,7F,63,63,87,9D,86,85,75,5E,70,59,59,8C,69,57,76,86,6F,5B,83,7B,72,76,71,7C,71
                    HEX        69,7B,80,72,81,AB,B6,B6,87,53,7A,8A,82,99,7C,70,83,82,91,8E,81,84,6F,8B,8A,70,80,77,57,53,6A,7F
                    HEX        8C,8A,71,60,79,74,6A,75,7F,63,54,71,8A,8A,66,5C,87,80,74,80,73,80,7D,7F,80,71,7D,87,A0,9F,81,95
                    HEX        9E,94,7C,82,81,79,94,A5,9D,94,8F,AB,A3,85,91,9A,95,90,70,7D,7C,5E,7D,99,80,73,98,9F,73,67,5F,66
                    HEX        74,94,96,77,70,65,6D,83,8D,A3,A1,96,80,6C,81,7F,80,9F,89,89,91,94,9A,93,B5,97,61,93,90,66,7D,93
                    HEX        A1,A8,81,85,8B,80,8E,8C,7C,8C,AC,97,7C,76,6B,7E,7E,6F,82,7E,7B,80,74,5D,76,9E,8D,68,5E,62,86,86
                    HEX        67,58,7A,8D,77,55,72,93,80,69,9A,B0,77,61,69,7B,7E,67,86,8F,8A,86,7B,84,71,9A,A1,7A,7E,76,7B,71
                    HEX        74,81,6F,68,61,6D,86,AA,A7,6F,5E,87,86,6A,55,4D,75,67,69,8B,94,87,8E,95,70,62,7D,80,63,46,5F,6B
                    HEX        69,8A,A1,80,7B,86,8A,95,8D,6F,74,94,83,80,9B,86,7C,AC,9F,86,8C,8D,87,94,9F,97,AD,A3,72,60,8D,8F
                    HEX        87,7F,87,B2,9A,80,89,86,6A,66,74,71,8C,8D,7F,83,7A,77,71,6B,67,73,7F,7F,8F,83,70,74,7E,9B,9F,7D
                    HEX        79,90,A4,9A,99,A7,B2,AC,85,7B,A3,97,80,94,96,94,96,A2,98,80,81,91,7F,6C,66,6A,6A,73,96,7F,5D,6A
                    HEX        68,5F,82,7D,61,69,6D,77,7A,61,79,8D,7C,82,97,6D,6D,79,79,8F,7A,85,B8,AC,79,5F,61,72,93,9D,80,80
                    HEX        9B,9A,81,67,4B,75,9E,83,5B,63,90,84,57,70,96,93,89,7A,6D,54,6A,76,59,50,67,79,89,7F,60,8B,A1,7A
                    HEX        66,6B,71,77,72,80,9B,96,68,63,8F,83,83,A8,B3,8C,7B,8F,77,6D,68,71,90,AC,9E,90,6D,58,79,8A,89,9F
                    HEX        96,66,7B,7D,89,A9,84,8F,95,8C,8F,7B,79,77,6C,7B,89,86,84,8A,AE,C3,7F,76,A1,82,4A,5F,82,70,75,84
                    HEX        85,8E,84,89,B8,B0,89,62,76,7C,65,7C,93,95,B3,B3,87,94,99,83,69,67,A4,B6,84,7A,84,9F,8C,75,73,7E
                    HEX        9F,97,84,7F,72,80,74,5B,57,63,8F,91,71,79,86,74,75,7A,7F,A3,86,69,8F,75,5F,4A,53,8B,9A,82,71,8B
                    HEX        8E,7A,76,7F,75,83,8C,8F,7E,60,75,8C,66,70,82,81,99,B4,9E,86,84,6C,67,7F,81,6C,54,7B,89,8C,85,5F
                    HEX        57,79,84,75,80,94,7E,54,37,5F,80,80,81,6F,75,89,77,60,60,7F,7B,6B,80,8F,8D,75,84,7E,73,8E,91,9E
                    HEX        99,85,8C,85,77,93,95,77,85,B3,AB,91,6D,6F,95,9B,8A,A0,8C,75,95,86,5D,6A,80,98,9F,98,9D,86,84,81
                    HEX        50,59,77,9F,C0,99,60,82,A1,80,63,66,81,9F,85,79,86,86,81,74,77,99,94,8B,7D,80,9E,85,85,79,7B,A3
                    HEX        8A,70,8B,B4,8E,7D,84,8E,94,98,A9,99,6A,74,9B,85,6C,72,94,9F,85,82,84,97,91,73,55,4E,83,85,60,52
                    HEX        7F,9F,67,6A,82,80,70,58,7D,7C,68,81,7D,6D,60,57,83,9E,7D,5E,61,5F,7F,98,7E,7C,B0,AE,82,65,6D,7A
                    HEX        83,7D,55,80,AC,A1,8B,7F,9F,9D,8E,6D,5C,6F,70,81,8B,7B,7A,7A,7F,74,58,82,83,5F,42,53,97,90,62,7F
                    HEX        94,65,4C,52,6B,81,7C,87,96,7C,6C,8B,7C,69,74,95,B7,BB,98,87,8C,94,83,82,95,9E,90,82,99,9E,8B,9D
                    HEX        B1,8E,7B,70,93,9F,83,76,5D,80,9A,81,7B,81,9E,AA,6F,60,8B,80,63,5E,68,87,A0,9F,80,7D,80,80,75,7B
                    HEX        91,95,6C,56,7D,8F,A2,86,86,AC,C6,9E,7B,77,8A,A4,90,62,75,90,89,90,94,87,6F,80,9E,80,75,9F,9A,86
                    HEX        86,8C,80,6F,71,66,6F,99,99,76,7F,80,5C,6A,86,87,67,69,84,61,61,8D,6C,61,73,60,53,74,83,72,7E,85
                    HEX        72,9B,A3,82,60,77,90,8F,7D,7A,53,74,98,80,7F,8C,86,7E,83,7D,80,8C,A7,7F,4A,72,8E,83,68,67,8B,A0
                    HEX        8C,63,53,73,80,87,80,80,5C,6D,87,6D,50,6C,82,69,7A,8E,8A,6F,7C,80,66,80,94,81,80,8A,7C,7C,8C,80
                    HEX        7F,89,79,8E,C0,C5,97,62,96,AB,4F,6C,A5,BE,BA,96,8C,A0,AA,67,4C,85,94,73,72,A8,AA,69,5D,79,7D,89
                    HEX        8B,9B,A2,A0,8B,7A,6C,53,55,69,86,93,80,81,A7,A0,84,7E,89,95,7C,73,87,79,69,80,94,90,7E,67,89,BE
                    HEX        B3,A7,95,A4,A5,95,77,5B,80,90,7D,67,70,97,8C,7B,7C,70,8A,83,70,5B,45,6A,80,94,90,7E,6C,70,6D,77
                    HEX        7A,7E,8E,7D,80,80,6A,7C,98,97,8C,76,74,8D,80,6B,74,8A,90,76,59,6A,60,46,7C,96,7A,6D,72,7D,96,61
                    HEX        50,87,81,62,5D,71,79,6D,75,86,8A,9B,9D,84,63,8D,B0,99,97,98,83,65,63,5C,68,8D,A3,9A,7C,85,8C,83
                    HEX        6A,6B,8C,99,79,72,65,74,65,74,89,76,8B,83,99,8B,6F,77,75,7E,85,83,81,94,CB,C5,7A,5B,80,80,71,89
                    HEX        93,98,AE,9E,A0,A2,8F,98,96,8F,83,85,97,80,59,5C,90,A8,7F,83,84,67,79,79,6D,99,90,76,7E,8B,8D,7B
                    HEX        67,68,84,87,82,8D,B4,A1,7E,7E,8A,8A,80,74,81,8D,8E,8D,7C,73,79,7E,9B,8F,61,74,99,82,7C,72,97,AE
                    HEX        80,5F,60,81,87,6D,67,9A,A4,7C,75,80,97,6B,4B,7C,80,68,80,79,85,8E,7C,6F,61,66,75,82,76,77,83,80
                    HEX        7F,75,6B,95,8A,80,80,73,79,81,5F,59,94,A8,74,63,81,84,71,56,67,85,9D,8F,7B,8A,8E,69,50,66,87,9E
                    HEX        85,65,74,84,82,7B,72,7F,9A,A1,99,6D,6A,87,80,66,61,84,9D,87,7A,81,9D,87,6D,60,8F,A8,8C,74,94,A1
                    HEX        77,62,73,90,8E,79,74,8B,BA,BE,7D,71,8E,A0,8E,7C,79,70,97,A2,7C,71,8B,B6,91,76,83,8D,A0,90,80,8F
                    HEX        65,72,9A,91,82,70,76,99,A5,93,8E,7A,54,67,79,70,6B,70,9B,8A,55,8A,A9,8A,67,85,AA,8C,67,5D,86,96
                    HEX        77,77,A4,A5,A1,A8,8C,76,80,93,8B,7F,7A,71,82,80,81,8E,81,7E,79,80,7A,7C,7B,7D,52,50,75,82,6C,61
                    HEX        69,7A,7A,80,77,69,59,73,7B,7D,7D,76,6C,68,6B,80,80,80,9A,A1,7A,80,86,7F,9A,7F,6F,7E,7D,8E,90,81
                    HEX        7F,71,8B,93,91,8F,66,55,86,8F,62,5D,6F,58,74,91,95,8D,7D,74,7E,90,80,6A,73,74,61,65,80,7C,80,9E
                    HEX        BF,BE,98,73,6F,7E,97,80,5C,7E,9A,9D,85,98,97,7D,98,A2,8A,81,75,81,79,6D,97,A8,84,46,7F,B2,8A,6D
                    HEX        91,B7,89,74,83,93,81,80,7B,6C,82,91,8C,A4,A5,80,7A,7F,75,7E,8E,9B,7C,8C,B1,7D,6D,8E,87,69,74,80
                    HEX        84,7C,77,84,82,67,7C,7B,81,87,73,7A,77,73,80,98,98,6F,76,93,99,81,79,80,91,8B,83,76,79,8D,8B,82
                    HEX        85,8F,94,8B,70,73,7B,5E,62,8E,87,66,6B,8E,86,6B,67,72,74,77,70,66,72,5E,6F,8D,77,70,68,6F,81,74
                    HEX        69,8C,89,73,95,96,80,65,7C,98,76,74,84,79,7A,98,94,8F,76,68,5E,7E,A5,9A,87,8A,7E,6D,70,7B,69,75
                    HEX        77,74,89,85,94,9F,80,7C,6D,71,69,80,90,80,6B,6F,9A,9F,7D,80,80,8D,97,86,65,7B,B1,BE,95,79,6F,85
                    HEX        89,80,86,87,80,A0,AC,81,82,8C,89,73,81,86,8E,9E,8D,A2,9E,8D,71,6B,80,77,74,97,A1,91,8E,8C,8A,80
                    HEX        7A,83,80,71,7E,7D,68,75,90,99,8E,85,8B,7C,79,93,74,76,91,81,84,74,84,8F,8D,77,75,7C,77,77,83,8C
                    HEX        8E,6F,70,84,79,62,66,6B,98,9F,8B,7D,83,82,75,93,80,68,70,73,75,7C,9D,90,81,8E,81,5C,76,71,67,89
                    HEX        89,71,76,7B,5C,66,65,6C,5C,7C,C3,9B,5D,55,61,63,69,62,7C,90,83,8C,86,7A,70,81,A2,96,7D,A7,96,79
                    HEX        80,79,76,76,66,73,90,A8,B8,97,85,7B,69,59,6D,76,73,7F,8F,90,9A,8F,74,69,74,94,86,80,7B,6C,73,6C
                    HEX        71,96,9D,89,85,99,87,72,80,93,93,98,90,8E,89,87,98,90,87,81,AA,CE,AA,6C,77,95,81,7C,7E,69,65,7C
                    HEX        9F,8A,7C,97,8D,6C,65,79,83,89,7C,7D,7F,90,8C,7F,7B,80,7D,81,89,8E,80,7D,A4,B7,76,65,91,79,6F,86
                    HEX        6D,63,98,BE,A8,8B,5E,4B,7F,93,76,81,9A,87,77,7F,8A,7F,57,63,8C,AE,86,65,6F,69,5F,6D,80,83,87,93
                    HEX        80,85,80,66,56,65,7D,86,8A,77,5D,80,85,6C,66,77,89,87,82,84,80,70,71,8B,7C,7A,6D,7C,A8,8D,80,7A
                    HEX        67,5C,5F,79,9E,9E,8A,84,80,6C,74,83,8A,8D,80,82,7B,55,70,8F,9F,74,57,86,AC,97,6A,69,80,77,74,6D
                    HEX        6F,87,76,6C,7B,A2,9F,82,80,8C,8D,82,8B,8C,A8,9D,87,89,80,73,76,8B,93,87,91,95,A5,B1,86,6F,85,8E
                    HEX        79,7B,81,74,7D,A5,B3,87,74,93,A0,85,73,6C,98,9E,71,60,83,9F,97,6D,80,97,7F,6D,85,85,6C,68,80,8E
                    HEX        87,72,75,7D,8F,6C,61,89,97,99,8D,7F,83,8A,76,74,80,80,8F,AA,A5,95,7E,8A,8D,62,6C,8E,89,91,8C,76
                    HEX        6A,74,75,70,74,7F,6B,5C,7C,83,61,5F,7E,74,58,69,77,79,7A,7C,71,74,82,87,7D,65,79,79,90,96,70,50
                    HEX        6D,8E,8B,68,73,9E,96,8A,90,90,7B,71,80,97,86,8B,8A,8C,94,81,69,70,66,6D,7B,80,89,8A,75,91,86,67
                    HEX        68,70,73,72,68,7F,89,7A,79,80,8A,A0,87,75,9B,AA,76,69,87,8F,75,5B,89,A7,8C,83,87,9E,A9,98,91,83
                    HEX        79,80,8D,9B,85,6D,8D,A8,7A,65,9E,B7,8B,71,84,AD,8F,70,79,89,77,6B,81,97,BA,A4,7A,72,8F,8C,6D,74
                    HEX        8B,9D,83,61,61,7A,93,7F,80,84,7A,81,7E,73,7E,80,77,79,75,65,6D,80,94,99,9A,97,84,73,76,71,6C,9E
                    HEX        AD,8F,71,65,83,86,82,9A,9F,80,94,82,6B,6D,6D,8A,93,83,71,7E,81,6D,75,7C,85,83,7B,5C,6F,71,48,53
                    HEX        71,94,76,5B,72,7E,6D,69,74,65,77,8E,7D,6A,8C,86,7A,7E,80,77,81,96,99,84,69,79,80,80,74,7B,9E,B6
                    HEX        9E,87,90,8A,71,60,7F,97,91,A3,8F,6A,61,89,8F,69,68,82,91,90,80,6B,74,7C,80,6C,6A,6D,6B,79,80,85
                    HEX        85,8A,9A,96,9D,86,70,6F,72,8C,8E,7A,80,8C,8C,9B,9F,8F,91,A0,8D,A7,99,75,81,AD,94,75,99,A7,9A,A0
                    HEX        90,82,94,83,76,79,7A,7C,84,8C,8B,73,71,97,86,61,77,7F,76,90,8B,66,74,63,55,70,9D,A3,71,6D,72,8A
                    HEX        9E,82,7F,83,6F,73,89,96,73,61,82,A9,87,80,9E,8A,89,76,75,94,91,6C,6B,83,AC,8F,69,70,73,5D,66,8B
                    HEX        94,82,8F,8A,76,70,6C,69,6D,7C,6D,75,7F,87,6D,50,7B,A8,86,69,66,6B,6C,66,86,73,5B,82,9F,79,62,5F
                    HEX        5B,8A,9B,79,72,99,94,60,7B,AA,85,5E,82,99,90,91,8A,94,97,8B,84,76,81,91,A0,8F,66,7D,9A,77,66,79
                    HEX        8C,B0,AA,7A,79,6C,53,5D,6F,81,8C,7F,7D,7A,74,7E,81,71,85,A2,99,7E,7A,7B,70,6C,91,A7,8C,83,A5,AA
                    HEX        AC,A5,7E,90,99,8B,8A,9D,A3,94,AC,98,61,62,82,9B,80,80,B6,A2,7F,57,5E,69,75,63,60,84,8B,7C,7B,77
                    HEX        72,7E,7A,77,76,68,79,9B,8B,7E,93,8F,87,82,6C,7D,8E,8E,95,90,85,9A,80,69,7D,90,86,89,87,89,8F,81
                    HEX        72,83,72,72,7C,7E,7E,70,6C,7F,83,71,81,86,6D,6C,75,8B,80,5D,61,77,85,6B,62,76,8C,87,7D,80,83,7F
                    HEX        70,80,7E,75,6B,73,80,80,82,77,73,82,9D,7F,6A,8F,8D,77,7A,80,94,8F,82,8A,84,6D,6F,7B,80,9D,A8,7D
                    HEX        68,87,79,7A,93,7C,72,87,8B,7E,82,8C,91,82,7A,72,73,74,6F,6D,98,8B,70,90,99,84,96,80,65,80,81,81
                    HEX        7B,7E,8A,85,8E,9E,9D,91,9D,9A,82,87,9B,8E,83,8D,7A,69,7B,8F,90,85,87,84,8F,97,8C,7F,90,98,7B,59
                    HEX        7D,8A,84,85,79,7C,90,8B,82,7B,81,85,86,79,82,83,76,76,6B,6C,80,80,7A,77,7F,72,74,98,87,87,99,7B
                    HEX        6D,72,80,8C,8C,84,9B,85,72,80,8A,76,82,90,86,8B,86,7F,73,70,80,84,87,85,7F,69,75,87,81,73,54,53
                    HEX        7B,6C,58,71,97,8D,6C,83,8D,62,6B,80,82,67,56,68,7D,84,8D,81,7F,80,7F,80,86,8B,85,85,97,93,80,7A
                    HEX        7A,80,86,79,8C,8D,76,7D,90,95,89,82,8B,82,80,7D,63,80,8C,70,5C,7A,82,91,82,6C,8A,9F,8C,69,80,90
                    HEX        8D,75,7C,82,7D,82,80,97,99,95,8C,80,80,81,91,8E,7D,91,99,80,6A,70,7F,8C,80,7D,94,9A,87,85,8A,80
                    HEX        76,9D,A4,86,82,83,8B,8E,82,80,84,96,82,81,82,80,90,9A,8D,82,72,7E,8A,80,68,77,8C,79,55,67,80,80
                    HEX        81,86,81,76,65,6C,74,6F,7B,85,8E,9D,8A,6C,57,72,87,93,8D,84,90,84,79,80,86,7C,96,96,82,85,76,70
                    HEX        84,7F,93,8B,7D,8B,84,72,6B,82,86,7B,71,60,66,80,8B,7C,6C,60,5C,81,7A,5D,6A,75,80,6A,60,7C,84,80
                    HEX        7A,81,8C,8F,87,7D,7E,80,87,8E,80,6B,81,86,84,8F,80,7D,99,93,93,A3,8E,74,7B,7F,79,73,77,7B,87,86
                    HEX        8D,91,8D,90,85,7F,8B,80,71,74,6A,72,7C,81,80,7D,7A,8B,8A,77,87,97,90,83,80,8E,80,6B,74,8B,8F,86
                    HEX        84,8D,84,86,A7,A3,97,8A,80,79,82,80,85,8D,9F,99,93,89,7C,80,80,7E,82,7F,90,94,71,65,7C,7B,7C,96
                    HEX        9D,9E,8A,79,6C,66,7E,8D,73,87,8F,6D,69,7D,8C,84,74,80,80,81,86,86,79,75,80,76,72,6C,76,86,87,79
                    HEX        7A,93,8F,80,80,80,83,82,80,87,79,70,81,84,87,7A,74,7F,7C,67,65,84,89,7F,81,82,75,71,77,82,7E,6C
                    HEX        6B,7F,79,74,74,7A,8C,80,80,8E,83,80,80,80,6F,62,6C,6F,74,7F,83,82,89,85,80,75,89,AB,A0,8C,8B,76
                    HEX        70,72,76,84,8C,83,80,81,8C,95,86,7D,90,82,70,79,87,8E,8C,76,73,80,7A,7F,76,72,89,9E,90,7D,71,81
                    HEX        94,7A,6F,87,90,8C,91,8F,8E,70,74,95,96,8F,97,87,7E,80,87,9B,A5,99,85,8C,8D,89,7C,81,85,81,7E,7F
                    HEX        8B,6D,71,8C,85,7D,94,84,87,86,70,71,89,8E,7C,75,67,7D,83,5E,79,94,8E,7D,72,80,80,86,8B,80,7C,83
                    HEX        7D,62,77,8C,8B,7E,94,8A,7C,7E,8E,8C,7C,84,A2,96,73,6B,74,74,7D,7A,70,75,70,73,85,8D,77,7B,86,80
                    HEX        81,80,83,74,52,59,75,77,6F,7A,87,8D,90,81,7C,74,73,7C,7E,7D,82,7C,7E,71,77,83,80,73,7D,90,A0,96
                    HEX        82,7F,79,82,7E,76,83,87,7E,86,91,7D,76,82,80,8C,86,7B,80,80,69,66,7B,91,94,95,91,85,7A,80,80,7C
                    HEX        80,7F,72,76,85,89,7F,80,9E,9A,7E,79,81,90,81,75,8D,80,76,8A,8E,89,80,7D,7E,8B,97,94,8B,93,97,8F
                    HEX        80,74,82,8D,7E,81,89,85,93,86,91,8F,7C,81,8F,96,8C,83,85,81,77,6A,69,81,8F,86,82,8B,7F,60,5C,79
                    HEX        8B,91,81,7D,7A,80,86,81,80,76,79,80,75,80,7C,6B,6C,87,9D,9A,83,87,85,7C,86,80,80,7B,6C,70,7E,80
                    HEX        80,83,91,89,7F,82,86,80,80,81,87,6F,52,67,7B,81,7E,7B,82,7C,73,72,79,87,7F,6D,73,75,6C,76,75,7B
                    HEX        7E,94,80,60,82,81,7B,8E,79,76,87,73,68,85,9B,93,7B,76,8B,A0,97,84,86,8F,7F,6A,7E,85,7F,7B,85,8D
                    HEX        8C,9B,8F,7C,80,7E,79,77,82,8C,83,81,75,63,63,81,94,7C,80,9D,9D,81,65,77,96,93,75,72,8A,8C,7F,7B
                    HEX        8D,95,85,80,8F,98,85,79,85,8D,93,82,87,8F,80,80,86,90,87,80,89,95,7E,82,9B,9D,96,7E,73,84,80,84
                    HEX        8C,87,80,6F,79,7C,81,93,8D,82,81,80,76,73,80,7B,7C,7F,77,6C,75,75,62,77,81,82,80,80,75,79,8D,83
                    HEX        68,5E,82,8C,80,7E,8D,84,80,81,7D,7E,89,83,80,80,80,87,87,82,7D,87,90,85,85,86,7F,76,69,6A,85,7E
                    HEX        6D,74,94,96,73,63,74,6A,5E,72,80,75,79,7B,73,75,75,6C,70,7E,87,83,89,8F,7B,7B,84,8A,8E,7F,7C,80
                    HEX        7F,89,8F,95,9B,8B,72,7F,94,90,86,80,77,7B,83,93,8E,85,7C,7F,87,84,7C,6C,94,90,5E,68,89,94,89,7F
                    HEX        81,90,84,6C,6B,82,94,7B,79,8A,89,81,91,97,83,7E,80,8B,97,98,86,89,99,7F,66,76,7C,80,8A,90,8F,9A
                    HEX        9F,85,82,8C,87,7E,65,7D,80,76,7A,89,93,84,89,93,8B,7E,80,80,83,76,74,84,89,7B,6D,80,A0,90,79,7E
                    HEX        7A,7F,80,7C,8F,87,79,72,67,74,7B,82,8C,83,83,89,77,61,77,7F,69,6F,73,70,85,9F,96,8A,82,82,79,7B
                    HEX        84,7E,77,85,7B,6A,68,69,80,93,71,7E,93,93,81,67,67,73,7B,79,77,80,7D,85,87,72,79,8D,84,86,87,80
                    HEX        7A,80,75,6A,79,8C,8A,67,74,86,8A,8C,90,82,7D,80,82,7F,8E,7F,70,85,82,7E,81,82,89,84,7E,84,87,7E
                    HEX        80,85,86,91,80,70,7C,87,8A,8C,87,8B,8A,79,7D,82,75,79,91,9D,93,86,83,8B,75,71,84,99,9D,8C,74,74
                    HEX        80,81,84,8E,94,80,87,9B,A0,8D,73,6A,7C,80,80,93,8E,7F,86,93,82,6A,69,80,8A,8A,84,7C,80,85,7A,7D
                    HEX        85,81,8B,8B,80,7D,7C,7A,70,6B,72,7A,8B,83,7E,82,84,8E,81,79,7C,7D,82,86,7D,76,8C,87,7A,7A,7E,84
                    HEX        8D,94,90,7E,74,73,74,6D,7B,86,83,89,7B,6A,72,7E,80,81,74,7A,83,7F,80,7B,6A,76,79,76,7E,75,7B,82
                    HEX        7E,7C,7A,7C,77,7A,81,7F,7A,81,81,7D,8C,8D,73,6A,73,98,8E,7B,89,90,83,7C,7C,95,8E,81,7D,7B,8D,91
                    HEX        83,8D,82,69,66,73,8D,98,85,8E,9F,80,67,75,81,82,74,7F,8F,84,79,7F,93,85,77,7B,8D,9E,90,77,6C,80
                    HEX        8B,80,80,8D,9D,97,77,80,96,87,7A,7D,7D,7A,82,97,96,82,80,80,8E,98,82,80,93,9B,83,81,8A,79,6B,7E
                    HEX        8A,86,7D,7D,84,95,8C,81,7B,7F,81,6A,74,87,7B,62,70,80,7B,82,9B,95,81,7A,7C,74,6F,7F,80,7C,7C,8B
                    HEX        81,68,7B,96,93,80,7F,80,84,7C,81,83,7B,7B,7F,8B,87,7E,7D,7A,7B,81,86,87,8B,7B,67,74,79,73,79,7C
                    HEX        84,87,81,75,83,7E,59,63,7C,7C,81,85,7D,76,70,76,80,7E,80,87,84,75,76,79,80,84,7E,80,82,82,80,82
                    HEX        80,83,A1,99,75,7B,95,8C,77,6B,80,8E,82,83,85,87,80,76,85,8D,91,89,80,84,86,7F,75,7D,81,87,8C,81
                    HEX        7E,87,8F,87,81,7A,68,72,80,84,94,9B,95,80,6B,73,80,81,85,8A,8D,89,89,86,84,8A,8F,96,8C,82,84,84
                    HEX        7B,8A,94,81,76,81,9D,84,7A,81,90,90,81,79,80,86,74,74,7F,89,87,7F,73,77,7C,85,8D,89,82,71,6D,6C
                    HEX        73,80,80,8B,85,7A,68,67,73,8B,93,80,82,8C,7E,67,73,8F,96,80,7D,83,80,79,73,7F,87,89,86,80,89,8A
                    HEX        76,71,7C,7E,85,85,86,7B,76,80,7A,7E,7E,76,80,81,69,67,7D,80,74,74,83,84,76,75,8C,85,7C,73,77,7A
                    HEX        74,80,91,9B,87,6F,67,73,7F,81,82,8E,99,8E,80,77,7C,7E,73,8A,95,8F,87,87,83,7C,82,81,8C,95,8D,8A
                    HEX        81,7F,7F,79,85,93,76,69,7B,8F,85,80,83,8A,85,73,85,86,8A,7E,76,80,7A,7B,87,89,8C,8C,82,87,87,86
                    HEX        80,7F,86,89,8A,7C,80,85,8F,8B,84,86,91,91,86,83,85,80,7D,82,8E,8B,8A,7E,71,76,74,74,86,96,8E,80
                    HEX        7B,80,76,66,73,8D,8A,80,74,79,7D,7C,89,8E,80,7D,81,80,7C,7E,7B,80,7F,7F,85,80,70,7D,83,81,82,83
                    HEX        7F,7F,93,7F,71,83,84,7B,77,69,6F,86,94,8A,7A,7D,7E,71,67,76,8A,8E,85,7D,76,6F,70,7D,87,81,7C,74
                    HEX        77,7B,74,80,7D,7F,85,7C,85,86,77,73,7A,75,7A,93,93,80,80,80,80,80,84,85,86,8D,87,7E,7A,7D,84,8D
                    HEX        90,82,75,7C,85,89,80,8A,84,79,7D,84,8E,8F,80,7F,7F,74,75,7E,93,98,89,87,87,7D,71,7A,80,86,8C,86
                    HEX        86,80,80,7E,80,80,8D,8F,97,97,8A,72,73,86,82,83,7B,82,8E,83,7C,7F,8F,A1,93,74,7A,8C,7E,6F,80,8F
                    HEX        80,7E,87,91,8A,80,83,84,7A,75,80,8B,89,75,77,80,81,74,7B,87,86,83,7F,80,8A,7D,6D,7D,7C,79,80,7B
                    HEX        83,81,68,71,7F,82,81,7D,8D,91,87,7D,66,6F,73,79,8A,8D,8A,86,7F,85,8A,80,7C,7F,82,7D,7F,80,7B,7A
                    HEX        6F,5F,7A,90,8E,81,80,77,76,75,74,69,6D,86,90,82,7D,82,80,7B,79,76,74,8B,90,7F,7A,80,7F,7C,80,8D
                    HEX        81,83,86,80,7D,7C,8D,95,7D,7C,84,8F,86,82,89,7F,77,7D,89,79,79,8D,8B,7C,74,82,95,8F,7B,79,8A,8B
                    HEX        7C,7B,8D,9A,98,83,7A,7F,80,80,7F,86,80,80,90,8B,81,81,80,75,75,7F,90,8E,80,71,76,80,80,81,89,8D
                    HEX        98,97,80,7A,79,7A,7B,80,8E,91,86,8A,90,8B,82,7E,7D,8B,80,7E,8E,96,80,69,74,8A,80,7B,83,8A,8A,7E
                    HEX        75,77,79,6A,6D,7A,81,81,7D,85,7F,73,80,86,80,80,86,89,81,7A,77,80,80,7E,83,87,87,7E,79,7E,7D,80
                    HEX        80,80,7D,75,7F,84,80,7C,70,74,84,82,79,80,80,80,85,85,80,80,77,74,69,71,7E,86,84,86,84,80,79,76
                    HEX        80,7A,80,85,80,7C,7D,89,80,7C,79,82,80,7F,86,87,81,82,82,7D,79,80,8D,8B,80,80,87,81,75,79,80,8E
                    HEX        95,98,91,84,80,80,80,7F,7A,74,81,8D,95,8E,7E,7A,80,89,86,8B,8D,7F,74,7F,86,74,72,80,89,83,87,8C
                    HEX        96,96,79,7E,80,80,80,76,77,87,89,82,8A,8E,8C,80,81,89,86,84,85,8E,90,80,75,81,7F,80,81,80,80,84
                    HEX        81,80,7F,7C,85,7C,77,77,80,81,80,7C,77,83,82,84,7F,80,86,83,80,74,7B,89,84,7A,71,87,8F,7C,7D,80
                    HEX        84,80,7A,75,76,80,80,86,7A,66,73,80,8A,85,86,85,81,7B,6B,63,6C,80,81,7E,80,8D,97,83,75,77,7F,7F
                    HEX        7C,73,80,89,83,7B,87,8B,80,80,80,84,76,79,80,83,80,7C,73,75,80,85,8B,94,84,72,74,7B,7A,7F,84,84
                    HEX        85,80,80,83,85,80,80,84,82,85,84,80,7F,86,8D,80,7D,80,81,86,87,83,89,8F,96,91,82,74,70,79,87,8B
                    HEX        80,7E,87,83,7B,80,86,8B,93,97,8B,7D,80,80,7C,7A,7A,84,83,86,8A,8A,90,87,82,8C,87,7C,7B,80,7B,7C
                    HEX        7C,73,7A,86,84,83,8E,82,7A,84,7E,6C,73,81,84,83,8D,8E,89,80,81,80,79,77,7E,83,82,81,80,7D,81,86
                    HEX        85,85,83,7E,79,7D,7B,7F,83,80,7B,7A,83,82,7F,74,77,7D,84,89,81,7B,77,77,76,80,80,7F,80,85,7D,79
                    HEX        7E,77,79,7A,80,80,7C,7A,82,82,7A,6D,79,7E,7D,80,80,80,8A,87,7C,7F,81,87,86,82,85,80,84,7D,7B,83
                    HEX        86,80,80,87,87,7C,73,7C,80,80,89,83,83,87,80,75,7B,82,7F,7F,86,82,80,7D,7F,84,80,85,93,8B,80,84
                    HEX        84,80,74,76,85,8E,8D,84,8E,94,8A,75,73,89,8D,84,7D,8D,90,80,7A,84,8E,84,87,81,77,7A,83,82,79,7B
                    HEX        86,83,80,8C,84,7E,83,89,84,81,87,86,80,7B,79,7C,83,84,83,8D,8F,80,76,80,81,79,7B,7F,7D,7B,80,8E
                    HEX        8A,7D,7B,7E,7E,7E,84,86,8A,86,80,70,67,71,80,85,80,7E,85,86,80,80,7F,81,80,80,7D,7B,7E,79,7F,7E
                    HEX        80,85,7F,80,80,80,7A,7D,7A,76,7E,80,80,6C,73,8A,8E,84,80,7F,7D,7B,80,82,7F,7D,80,7A,7A,82,8A,83
                    HEX        81,87,84,7E,7A,79,7A,80,80,7E,80,7C,7F,83,83,84,85,83,83,7F,7E,80,81,82,84,83,7C,85,84,7A,76,7E
                    HEX        83,8F,9A,82,7B,7F,7D,7E,87,84,85,8F,86,87,81,7E,82,81,80,89,82,80,81,8B,8B,80,7F,80,82,83,84,8C
                    HEX        8A,82,84,80,77,84,82,74,85,8F,87,80,82,87,7A,7A,86,87,87,82,7A,77,7A,7E,7D,83,90,99,8A,80,7E,79
                    HEX        79,72,7A,86,87,84,84,86,7C,7E,83,85,82,80,86,80,7B,7B,7B,7E,79,7B,84,87,84,80,86,81,80,80,74,76
                    HEX        7B,7F,80,83,8D,86,7F,6D,71,80,84,7E,7E,7D,7B,76,79,80,84,80,7D,7A,76,7A,7F,86,89,83,85,82,79,71
                    HEX        74,7D,85,83,81,84,87,80,76,7E,81,7F,87,8E,89,7E,77,72,73,7D,7F,86,85,83,80,7E,80,8B,80,76,7B,80
                    HEX        81,87,8D,7C,77,81,8A,80,80,87,8B,87,7C,79,80,80,80,84,84,85,89,8A,80,75,77,84,84,7F,91,9E,8C,80
                    HEX        80,7F,7F,7B,7C,8A,94,8A,85,84,7F,77,85,8B,89,90,87,80,84,80,7B,7A,79,7A,7A,80,87,8D,89,82,80,80
                    HEX        83,7F,7D,80,84,80,7A,80,8A,8A,80,82,89,80,7A,7D,82,80,7C,81,84,82,80,7D,82,83,80,7D,84,83,7D,7D
                    HEX        7F,86,83,7D,80,84,80,7D,7F,80,7D,7E,80,85,87,80,75,80,86,77,72,77,80,77,74,87,83,80,7D,71,72,7F
                    HEX        94,86,74,76,7D,83,7C,74,77,83,8A,87,86,7A,76,80,80,77,76,7F,8D,8B,84,8E,89,80,7A,73,7A,83,85,89
                    HEX        87,82,82,80,80,7B,80,7D,82,80,80,80,80,80,79,7E,7F,7E,83,8B,82,7D,81,7E,7B,80,82,80,80,80,85,84
                    HEX        80,82,89,8D,90,8A,84,85,82,82,7F,79,80,8A,8B,87,89,83,80,80,8C,8F,84,7C,80,8C,89,7B,7E,8D,85,79
                    HEX        7B,7E,81,86,8A,89,89,82,83,7E,76,7B,77,77,7B,80,8C,8A,81,7D,7E,79,7B,86,8B,84,80,83,7A,6D,70,80
                    HEX        84,80,86,8C,8F,8D,81,7E,7E,80,7C,7E,86,87,82,8A,8C,80,7A,7A,7D,82,86,80,81,7D,73,7D,86,80,77,7F
                    HEX        81,7D,7B,7A,7B,7A,79,77,7F,82,80,74,75,7F,7B,75,81,86,7F,80,82,80,7C,80,80,7D,7E,7B,7E,80,84,8A
                    HEX        82,81,86,84,80,8C,89,82,7F,80,7E,81,87,80,80,80,7E,7A,7F,8A,85,80,7F,7E,7F,7F,77,80,84,80,7B,80
                    HEX        8B,80,7B,7C,7D,7F,84,84,86,8E,89,83,80,76,7A,80,80,86,8D,8C,87,83,83,7D,7E,8C,8E,87,87,84,85,86
                    HEX        79,7C,89,90,84,89,91,87,7E,79,82,87,8B,87,7F,80,81,7A,75,82,84,7B,7F,80,7E,80,7F,81,7F,79,7E,80
                    HEX        84,83,80,80,80,7C,79,7E,80,80,86,8F,8F,81,7B,80,83,7C,80,84,8B,89,87,80,76,7A,80,80,80,89,8A,7B
                    HEX        73,74,83,8E,80,7D,84,7C,75,79,80,80,80,80,80,7B,77,7A,80,82,80,7B,80,81,7D,7B,7B,7B,7B,7D,80,80
                    HEX        80,80,7D,81,83,7D,7A,7F,81,7D,7A,81,84,89,86,83,7E,79,81,84,84,80,79,7D,81,82,84,8A,8B,8B,84,7D
                    HEX        7D,80,80,80,80,86,84,77,7A,83,84,7D,7E,84,8A,87,82,80,7D,7C,79,7A,7E,82,83,82,84,82,82,83,8A,8B
                    HEX        85,82,80,82,87,84,81,84,80,7F,83,8A,8A,8C,8F,8C,85,80,82,7E,80,87,81,80,84,85,80,7D,80,80,80,80
                    HEX        80,82,80,79,7C,80,79,77,7D,81,85,85,85,81,82,83,7F,7B,76,79,7D,7F,82,85,87,84,80,80,86,89,84,81
                    HEX        80,80,81,7F,7A,7C,80,82,82,84,85,82,80,80,7F,81,82,80,80,80,80,80,7B,76,73,7D,82,81,84,81,7B,79
                    HEX        7D,7C,79,76,77,80,80,83,81,7E,77,77,7F,82,87,84,7E,7E,7F,7F,82,84,7F,80,83,7D,7A,80,86,86,81,84
                    HEX        83,80,7E,80,83,80,80,82,84,81,7B,80,80,7C,80,89,81,7D,85,87,80,7D,80,80,80,82,83,80,80,7F,80,80
                    HEX        7F,7C,7D,85,89,86,86,84,7F,7B,80,82,87,86,8C,8A,81,7E,7F,80,84,87,87,89,85,80,7E,80,85,85,86,87
                    HEX        8B,87,80,7C,7D,82,83,7D,80,84,80,80,80,80,7C,80,86,82,80,81,7D,7D,82,81,7C,77,77,7E,81,80,84,85
                    HEX        7F,7D,80,80,80,86,89,83,7F,7A,7A,81,83,81,81,81,82,80,82,84,83,85,85,84,80,80,80,7E,77,76,7B,80
                    HEX        86,83,7F,7D,7A,7C,7F,80,80,81,80,7D,74,73,79,7A,7A,7D,80,82,80,80,82,83,80,7E,7F,80,80,83,7C,7A
                    HEX        83,87,7F,7A,82,85,80,80,84,81,85,86,83,80,7E,7F,80,80,81,82,80,80,82,84,85,7F,7E,80,77,80,87,84
                    HEX        80,87,86,7E,7D,80,80,80,7E,83,84,80,80,81,80,82,84,80,80,83,8B,8B,84,82,80,82,80,80,83,81,80,80
                    HEX        80,86,8E,8B,85,84,84,86,82,80,80,84,80,7E,80,7F,7E,82,83,80,84,83,81,82,81,84,86,7B,72,7B,81,80
                    HEX        7D,7E,80,82,85,84,81,80,7D,7E,80,83,80,7C,7E,7E,80,84,82,80,81,81,7F,80,8D,8A,80,80,80,80,7F,7F
                    HEX        80,80,80,81,80,7E,80,7E,80,80,7B,7F,80,7C,7B,80,85,80,79,7C,7D,80,82,7E,7B,7E,7C,77,7F,81,84,80
                    HEX        7D,7D,80,83,80,7E,7C,80,80,80,80,7B,7D,83,8A,84,81,81,83,87,81,79,79,80,87,85,85,87,82,80,7E,7F
                    HEX        81,7D,77,83,8B,80,80,82,81,80,7F,7C,80,84,80,80,80,7A,7D,80,7F,7F,80,85,85,85,89,89,86,80,80,80
                    HEX        81,83,82,83,84,81,84,85,86,82,81,82,85,8A,89,81,80,83,82,7A,7B,80,80,80,83,85,81,81,84,81,80,82
                    HEX        80,80,7D,76,7A,80,80,7D,80,84,82,83,82,85,81,7D,7D,7F,83,82,83,81,7E,7E,80,84,83,81,84,84,81,80
                    HEX        80,7E,80,82,80,81,80,81,87,80,7D,80,7F,7D,7E,7E,80,80,80,80,80,7E,7F,80,7F,7C,7C,80,81,7F,7E,7F
                    HEX        7B,7A,7D,7C,7F,7F,7D,7F,81,83,80,80,80,80,7F,7C,7F,82,82,80,80,80,80,80,80,7C,80,8D,8A,83,89,84
                    HEX        80,7F,80,80,7E,80,83,82,7F,79,7C,85,81,80,84,87,81,7C,80,80,80,81,7D,7D,80,80,80,80,80,80,81,83
                    HEX        89,8C,85,7E,7F,80,7B,7B,80,82,86,87,85,81,80,82,86,82,89,8D,83,80,82,84,83,7D,7D,89,87,84,89,80
                    HEX        80,80,80,82,87,86,83,80,7A,7C,85,80,7C,80,84,81,7F,7C,7C,7C,83,87,7E,7C,81,82,7D,7A,7E,80,80,80
                    HEX        82,8C,8D,7D,75,7C,7E,79,80,87,8F,8D,86,82,7E,7C,7C,79,7E,85,8A,84,80,7D,76,7D,81,83,84,83,84,82
                    HEX        7C,7A,7E,7F,80,80,80,80,7A,76,81,82,7A,7E,81,80,7E,7D,7B,80,82,7F,7F,80,82,83,80,7B,7B,77,7A,80
                    HEX        85,85,85,86,85,83,7F,79,7C,7C,7D,82,87,80,80,80,81,81,80,84,8D,8D,86,7D,79,82,81,7A,7E,86,83,80
                    HEX        80,7F,80,84,80,85,8D,84,7E,7A,7B,7C,7D,80,82,84,81,7F,80,80,80,82,85,85,84,83,80,82,84,7C,7D,85
                    HEX        86,81,84,8A,87,85,8A,82,7D,7C,83,84,87,8C,85,84,84,7E,7C,7E,84,89,81,7E,7D,7B,79,76,7D,85,87,84
                    HEX        87,85,7E,7A,7B,7C,7E,81,82,80,80,81,7D,7D,89,82,7F,81,85,84,7F,7F,82,81,7F,7F,80,81,84,81,80,82
                    HEX        80,7E,7B,80,83,83,81,83,83,7D,79,7D,80,80,7C,7F,83,80,7B,7C,80,7B,7C,80,84,84,80,7D,80,84,81,7C
                    HEX        7D,7D,7C,7B,7C,80,80,85,85,80,80,7F,80,80,81,83,82,7F,81,80,7C,7D,80,83,82,80,00,00,00,00,00,00

******************************************************************************************